/*
 *  Licensed Materials - Property of IBM
 *  5725-G92 (C) Copyright IBM Corp. 2011, 2013. All Rights Reserved.
 *  US Government Users Restricted Rights - Use, duplication or
 *  disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 */

package com.ibm.appcenter;

import com.ibm.appcenter.i18n.I18N;
import com.ibm.appcenter.i18n.I18NResource;
import com.ibm.appcenter.ui.ConfigurationOptionsProvider;
import com.ibm.appcenter.ui.StartScreen;
import com.ibm.appcenter.ui.UIUtil;
import com.ibm.appcenter.util.Log;

import net.rim.blackberry.api.options.OptionsManager;
import net.rim.device.api.applicationcontrol.ApplicationPermissions;
import net.rim.device.api.applicationcontrol.ApplicationPermissionsManager;
import net.rim.device.api.applicationcontrol.ReasonProvider;
import net.rim.device.api.system.ApplicationDescriptor;
import net.rim.device.api.ui.UiApplication;

/**
 * This is the main entry point for the IBM Application Center
 * Blackberry Client.
 */
public class AppCenter 
extends UiApplication
implements I18NResource
{
    /**
     * Entry point for application
     * @param args Command line arguments (not used)
     */ 
    public static void main(String[] args)
    {
    	UIUtil.evaluateUIProperties(AppCenter.class);
    	Log.register();
    	Log.log("Start main ", args);
        if (args.length > 0 && args[0].equals("autostart")) {
        	Log.log("Autostart");
        	// This is run on auto start. It just registers the options provider
            // so that the application options can be found in the global
        	// options menu.
            ConfigurationOptionsProvider provider =
              new ConfigurationOptionsProvider();
            OptionsManager.registerOptionsProvider(provider);
            Log.log("Autostart done");
        } 
        else {
        	// This is the normal entry point of the application.
        	// Create a new instance of the application and make the currently
        	// running thread the application's event dispatch thread.
        	Log.log("Enter interactive event loop");
        	AppCenter theApp = new AppCenter();       
        	theApp.enterEventDispatcher();
        }
    }
    

    /**
     * Creates a new MyApp object
     */
    public AppCenter()
    {   
    	if (checkPermissions()) {
    		// Push a screen onto the UI stack for rendering.
    		pushScreen(new StartScreen());
    	} else {
    		// Hmmm ... we cannot show a warning dialog without a main screen.
    		// We could show an info screen instead of the normal screen ...
    	}
    } 
        
    /**
     * Check the permissions of this application.
     * If the permissions are not given, it asks the user for the permissions.
     * We need the following permissions.
     * <ul>
     * <li> To install and uninstall applications </li>
     * <li> The access the internet </li>
     * </ul>
     * Permissions are only checked when the application is signed
     * and when the device security is enabled. 
     * In the simulator, enable device security in the General tab
     * of the Simulator section of the launch configuration of Eclipse.
     * 
     * @return true if the permissions are granted.
     */
    private boolean checkPermissions()
    {
        // Capture the current state of permissions and check against the requirements
        ApplicationPermissionsManager apm = ApplicationPermissionsManager.getInstance();
        ApplicationPermissions original = apm.getApplicationPermissions();

        // Set up and attach a reason provider
        AppCenterReasonProvider drp = new AppCenterReasonProvider();
        apm.addReasonProvider(ApplicationDescriptor.currentApplicationDescriptor(), drp);

        if (original.getPermission(ApplicationPermissions.PERMISSION_APPLICATION_MANAGEMENT) == ApplicationPermissions.VALUE_ALLOW &&
            original.getPermission(ApplicationPermissions.PERMISSION_INTERNET) == ApplicationPermissions.VALUE_ALLOW) {
            // All of the necessary permissions are currently available
            return true;
        }

        // Create a permission request for each of the permissions.
        ApplicationPermissions permRequest = new ApplicationPermissions();
        permRequest.addPermission(ApplicationPermissions.PERMISSION_APPLICATION_MANAGEMENT);
        permRequest.addPermission(ApplicationPermissions.PERMISSION_INTERNET);
        
        return ApplicationPermissionsManager.getInstance().invokePermissionsRequest(permRequest);
    }
    
    /**
     * This class implements the ReasonProvider interface in order to provide
     * detailed permission request messages for the user.
     * These messages appear when the user does not confirm the application as trusted,
     * does not allow the permission in the initial dialog, and then runs in a situation
     * when the permission is really needed. If the device permissions setting is then
     * set to Prompt in "Options > Application Settings > this application", it asks
     * for the permission on the fly, with a link "Detail from the vendor".
     * The text that then appears in the dialog box is the text of this reason
     * provider.
     * 
     * Honestly, it is fairly seldom that this text appears, because either the
     * app is trusted or its permission is granted initially. No normal user
     * who is in sane mind would do anything else.
     */
    final class AppCenterReasonProvider implements ReasonProvider 
    {    
       /**
        * @see net.rim.device.api.applicationcontrol.ReasonProvider#getMessage(int)
        */
        public String getMessage(int permissionID) 
        {        
            // General message for other permissions
            String message = I18N.getString(PermissionsGeneric, new Integer(permissionID));
            
            // Set specific messages for specific permission IDs
            switch (permissionID) {
                         	
            case ApplicationPermissions.PERMISSION_APPLICATION_MANAGEMENT:
            	message = I18N.getString(PermissionsInstall);
            	break;
            	
            case ApplicationPermissions.PERMISSION_INTERNET:
            	message = I18N.getString(PermissionsInternet);
            	break;
            }
            
            return message;
        }
    }
}
