/*
 *  Licensed Materials - Property of IBM
 *  5725-G92 (C) Copyright IBM Corp. 2011, 2013. All Rights Reserved.
 *  US Government Users Restricted Rights - Use, duplication or
 *  disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 */

package com.ibm.appcenter.artifacts;

import net.rim.device.api.i18n.DateFormat;
import net.rim.device.api.system.Bitmap;
import net.rim.device.api.util.Arrays;
import net.rim.device.api.util.Comparator;

import org.json.me.JSONArray;
import org.json.me.JSONException;
import org.json.me.JSONObject;

import com.ibm.appcenter.util.BasicUtil;

public class AppDetail
extends AppSummary
{
	private CodModule[] _modules;
	private AppVersion[] _versions;
	private String _description;
	private boolean _mandatory;
	private String _authorDisplay;
	private String _authorLogin;
	private String _vendor;
	private String _installedVersion;
	private long _downloaded;
	private long _created;
	private int _size;
	private boolean _allowInstallAndDelete;
	
	/**
	 * Creates an application artifact from JSON.
	 * @param json The JSON object
	 * @throws JSONException
	 */
	public AppDetail(JSONObject json)
	throws JSONException
	{
		super(json);
		
		// since it came from JSON, this app must exist
		// on the server, hence we can allow install and delete.
		_allowInstallAndDelete = true;
		_description = json.optString("description");
		_mandatory = "Y".equals(json.optString("mandatory"));
		_authorDisplay = json.optString("author_display");
		_authorLogin = json.optString("author");
		_vendor = json.optString("vendor");
		_installedVersion = null;
		
		_downloaded = -1;
		_created = -1;
		_size = -1;
		
		collectCodModules(json.optJSONArray("url"));
		collectVersions(json.optJSONArray("versions"));
	}
	
	/**
	 * Creates an application artifact.
	 * @param json The JSON object
	 * @throws JSONException
	 */
	public AppDetail(
			String name,
			String id,
			String version, 
			Bitmap icon,
			String description,
			String vendor,
			long updated,
			long downloaded,
			long created,
			int size)
	{
		super(name, id, version, icon, updated);
		
		// This is called when we fill an app from the
		// CodeModuleManager. In that case, the app does 
		// not exist on the server, hence
		// allow installation must be false, cannot be done.
		// In theory, allow delete could be true, just I think
		// it would not be wise to delete stuff that does not
		// belong to appcenter.
		_allowInstallAndDelete = false;
		
		_description = description;
		_mandatory = false;
		_authorDisplay = null;
		_authorLogin = null;
		_vendor = vendor;
		_installedVersion = version;
		
		_downloaded = downloaded;
		_created = created;
		_size = size;
	}
	
	/**
	 * This is used to update the app information with info
	 * coming from the code module manager. The code module
	 * manager can detect the vendor ...
	 * @param vendor The vendor;
	 */
	public void update(String vendor, String installedVersion, long downloaded, long created)
	{
		_vendor = vendor;
		_installedVersion = installedVersion;
		_downloaded = downloaded;
		_created = created;
	}
	
	/**
	 * Collects the cod modules of the application.
	 * @param jsonarray
	 */
	private void collectCodModules(JSONArray jsonarray)
	throws JSONException
	{
		if (jsonarray == null) return;
		
		int n = jsonarray.length();
		// first count how many cod modules we have
		// since not all url entries are cod modules
		int count = 0;
		for (int i = 0; i < n; i++) {
			JSONObject json = jsonarray.getJSONObject(i);
			String rel = json.optString("rel");
			if ("cod".equals(rel)) {
				count++;
			}
		}
		// now get the cod modules
		_modules = new CodModule[count];
		count = 0;
		for (int i = 0; i < n; i++) {
			JSONObject json = jsonarray.getJSONObject(i);
			String rel = json.optString("rel");
			if ("cod".equals(rel)) {
				_modules[count++] = new CodModule(json);
			}
		}
		// and sort the cod modules according to sort key
		Arrays.sort(_modules, new Comparator() {

			public int compare(Object o1, Object o2) {
				CodModule m1 = (CodModule)o1;
				CodModule m2 = (CodModule)o2;
				return m1.getSortKey() - m2.getSortKey();
			}
		});
	}
	
	/**
	 * Collects the versions of the application.
	 * @param jsonarray
	 */
	private void collectVersions(JSONArray jsonarray)
	throws JSONException
	{
		if (jsonarray == null) return;
		
		int n = jsonarray.length();
		// first count how many versions we have
		int count = 0;
		for (int i = 0; i < n; i++) {
			JSONObject json = jsonarray.getJSONObject(i);
			String v = json.optString("version", null);
			String l = json.optString("link", null);
			if (v != null && l != null) {
				count++;
			}
		}
		// now get the versions
		_versions = new AppVersion[count];
		count = 0;
		for (int i = 0; i < n; i++) {
			JSONObject json = jsonarray.getJSONObject(i);
			String v = json.optString("version", null);
			String l = json.optString("link", null);
			if (v != null && l != null) {
				_versions[count++] = new AppVersion(json, getId());
			}
		}
		// and sort the versions, since that's nicer in the display.
		// Reverse order is preferred
		Arrays.sort(_versions, new Comparator() {
			public int compare(Object o1, Object o2) {
				String s1 = ((AppVersion)o1).getVersion();
				String s2 = ((AppVersion)o2).getVersion();
				return - BasicUtil.compareVersion(s1, s2);
			}
		});
	}
	
	/**
	 * Returns the description.
	 */
	public String getDescription()
	{
		return _description;
	}
	
	/**
	 * Returns the vendor.
	 */
	public String getVendor()
	{
		return _vendor;
	}
	
	/**
	 * Returns the installed version.
	 * If it returns null, we either don't know which version
	 * is installed, or none is installed.
	 */
	public String getInstalledVersion()
	{
		return _installedVersion;
	}
		
	/**
	 * Returns the updated time stamp as a string,
	 * formatted wrt. to the locale of the device.
	 * This is the time stamp when the application
	 * was updated on the server.
	 */
	public String getUpdatedAsString()
	{
		return getTimeAsString(getUpdated());
	}
	
	/**
	 * Returns the download time stamp.
	 * This is the time stamp when the application
	 * was downloaded to the device.
	 */
	public long getDownloaded()
	{
		return _downloaded;
	}
	
	/**
	 * Returns the download time stamp as a string,
	 * formatted wrt. to the locale of the device.
	 * This is the time stamp when the application
	 * was downloaded to the device.
	 */
	public String getDownloadedAsString()
	{
		return getTimeAsString(getDownloaded());
	}
	
	/**
	 * Returns the updated time stamp.
	 * This is the time stamp when the application
	 * was initially created (i.e. compiled and packaged).
	 *
	public long getCreated()
	{
		return _created;
	}
	
	/**
	 * Returns the updated time stamp as a string,
	 * formatted wrt. to the locale of the device.
	 * This is the time stamp when the application
	 * was initially created (i.e. compiled and packaged).
	 *
	public String getCreatedAsString()
	{
		return getTimeAsString(getCreated());
	}
	 */
	
	/**
	 * Returns a time stamp as formatted time string.
	 * @param time The time stamp.
	 */
	private String getTimeAsString(long time)
	{
		if (time < 0) return null;
		try {
			DateFormat fmt = DateFormat.getInstance(DateFormat.DATETIME_DEFAULT);
			return fmt.formatLocal(time);
		} catch (Exception ex) {
			return null;
		}
	}
	
	/**
	 * Returns whether the cod module is mandatory.
	 *
	public boolean isMandatory()
	{
		return _mandatory;
	}
	 */
	
	/**
	 * Returns the login name of the person who has
	 * uploaded this application.
	 */
	public String getAuthorLogin()
	{
		return _authorLogin;
	}
	
	/**
	 * Returns the display name of the person who has
	 * uploaded this application.
	 */
	public String getAuthorDisplay()
	{
		return _authorDisplay;
	}
	
	/**
	 * Returns the versions if any. Can be null.
	 */
	public AppVersion[] getVersions()
	{
		return _versions;
	}
	
	/**
	 * Return the cod modules. Can be null.
	 */
	public CodModule[] getModules()
	{
		return _modules;
	}
	
	/**
	 * Returns all data for the modules.
	 */
	public byte[][] getData()
	{
		byte[][] data = new byte[_modules.length][];
		for (int i = 0; i < _modules.length; i++) {
			byte[] chunk = _modules[i].getData();
			data[i] = chunk;
		}
		return data;
	}
	
	/**
	 * Returns the total length of bytes of all cod modules.
	 */
	public int getModulesTotalLength()
	{
		if (_size > 0)
			return _size;
		if (_modules == null)
			return 0;
		int count = 0;
		for (int i = 0; i < _modules.length; i++) {
			count += _modules[i].getSize();
		}
		return count;
	}
		
	/**
	 * Returns whether this application can be installed.
	 */
	public boolean allowInstall()
	{
		return _allowInstallAndDelete;
	}
	
	/**
	 * Returns whether this application can be deleted.
	 */
	public boolean allowDelete()
	{
		return _allowInstallAndDelete;
	}
	
	/**
	 * Returns a string representation for debugging.
	 */
	public String toString()
	{
		StringBuffer buf = new StringBuffer();
		buf.append("AppDetail{");
		innerToString(buf);
		buf.append("}");
		return buf.toString();
	}
	
	/**
	 * Prepared the inner of toString, for debugging.
	 * @param buf
	 */
	protected void innerToString(StringBuffer buf)
	{
		super.innerToString(buf);
		buf.append("," + _vendor);
		buf.append("," + _installedVersion);
		buf.append("," + _downloaded);
		buf.append("," + _description);
		buf.append("," + _mandatory);
		buf.append("," + _created);
	}
}
