/*
 *  Licensed Materials - Property of IBM
 *  5725-G92 (C) Copyright IBM Corp. 2011, 2013. All Rights Reserved.
 *  US Government Users Restricted Rights - Use, duplication or
 *  disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 */

package com.ibm.appcenter.artifacts;

import org.json.me.JSONArray;
import org.json.me.JSONException;
import org.json.me.JSONObject;

import com.ibm.appcenter.i18n.I18N;
import com.ibm.appcenter.util.BasicUtil;
import com.ibm.appcenter.util.IconBitmapProvider;
import com.ibm.appcenter.util.Log;

import net.rim.device.api.system.Bitmap;

/**
 * Information about an application.
 * This is the short variant that we need in order
 * to display the application list.
 */
public class AppSummary extends AppVersion
{
	private String _name;
	private String _iconURL;
	private Bitmap _icon;
	private double _popularity;
	private String _latestVersionURL;
	private double _ratingAllVersions;
	private long   _numRatingsAllVersions;
		
	/**
	 * Creates a new application artifact.
	 * @param name The name of the application artifact.
	 * @param version The version of the application artifact.
	 * @param icon The icon.
	 * @param updated The updated time stamp.
	 */
	public AppSummary(String name, String id, String version, Bitmap icon, long updated)
	{
		super(id, version, updated);
		_name = (name != null ? name.trim() : null);
		_popularity = 0;
		_ratingAllVersions = 0;
		_numRatingsAllVersions = 0;
		_icon = icon;
	}
	
	/**
	 * Creates an app summary from JSON.
	 * @param json The JSON object
	 * @throws JSONException
	 */
	public AppSummary(JSONObject json)
	throws JSONException
	{
		super(json, null);
		_name = json.getString("label");
		_latestVersionURL = json.optString("latest_version_link", null);
		try {
			// optDouble is buggy, hence we simulate it ourself
			_ratingAllVersions = BasicUtil.avgRatingFromServer(json.getDouble("avg_rating_all_versions"));
		} catch (JSONException ex) {
			_ratingAllVersions = 0.0;
		}
		try {
			// optDouble is buggy, hence we simulate it ourself
			_numRatingsAllVersions = (int)json.getDouble("nb_rating_all_versions");
		} catch (JSONException ex) {
			_numRatingsAllVersions = 0;
		}
		try {
			// optDouble is buggy, hence we simulate it ourself
			_popularity = json.getDouble("popularity");
		} catch (JSONException ex) {
			_popularity = 0;
		}
		_iconURL = null;
		JSONArray urls = json.optJSONArray("url");
		if (urls != null) {
			// search the icon
			for (int i = 0; i < urls.length(); i++) {
				try {
					JSONObject url = urls.getJSONObject(i);
					String rel = url.optString("rel");
					if ("icon".equals(rel)) {
						_iconURL = url.optString("link", null);
					}
				} catch (JSONException ex) {
					// no need to export this to outside, since it is
					// not fatal. It means simply that we found an array
					// entry that does not match the icon spec. But this
					// can happen since there is more than just icons
					// in the urls array.
					Log.log("Exception when searching icon in JSON", ex);
				}
			}
		}
		// this is filled later
		_icon = null;
	}
	
	/**
	 * Returns the apps from a device catalog response.
	 * @param catalog The catalog found in the response body.
	 * @param keyToArray The key of the field that contains the array.
	 * @return The list of apps.
	 * @throws JSONException 
	 */
	public static AppSummary[] getAppsFromJSON(JSONObject catalog, String keyToArray)
	throws JSONException
	{
		JSONArray array = catalog.getJSONArray(keyToArray);
		int n = array.length();
		AppSummary[] apps = new AppSummary[n];
		for (int i = 0; i < n; i++) {
			JSONObject jsonapp = array.getJSONObject(i);
			apps[i] = new AppSummary(jsonapp);
		}
		return apps;
	}
		
	/**
	 * Returns the name of the application.
	 * On the server side, this corresponds to the
	 * label.
	 */
	public String getName()
	{
		return _name;
	}
	
	/**
	 * Returns the URL of the latest version.
	 * @return
	 */
	public String getLatestVersionURL()
	{
		return _latestVersionURL;
	}
	
	/**
	 * Returns the rating for all versions, between 0 and 5.
	 */
	public double getRatingAllVersions()
	{
		return _ratingAllVersions;
	}
	
	/**
	 * Returns the number of rating for all versions.
	 */
	public long getNumberOfRatingsAllVersions()
	{
		return _numRatingsAllVersions;
	}
	
	/**
	 * Recalculate the average rating and the number of ratings
	 * when a rating is submitted.
	 * @param submittedRating The submitted rating.
	 */
	public void recalcRatingOnSubmission(int submittedRating)
	{
		super.recalcRatingOnSubmission(submittedRating);
		
		double sum = _ratingAllVersions * _numRatingsAllVersions + submittedRating;
		_numRatingsAllVersions++;
		_ratingAllVersions = sum/_numRatingsAllVersions;
	}
	
	/**
	 * The info line should display version info.
	 */
	public static final int VERSION_INFO = 0;
	
	/**
	 * The info line should display updated info.
	 */
	public static final int UPDATED_INFO = 1;
	
	/**
	 * The info line should display popularity.
	 */
	public static final int POPULARITY_INFO = 2;
	
	/**
	 * Returns the info line of the application,
	 * displayed under the name. 
	 */
	public String getInfoLine(int mode)
	{
		switch (mode) {
			case VERSION_INFO:
				return I18N.getString(AppArtifactVersionInfo, getVersion());
			case UPDATED_INFO:
				return I18N.getString(AppArtifactUpdatedInfo, getUpdatedAsString());
			case POPULARITY_INFO:
				return I18N.getString(AppArtifactPopularityInfo, BasicUtil.formatNumber(getPopularity(), 0));
		}
		return getVersion();
	}
		
	/**
	 * Returns the icon of the application, if any.
	 */
	public Bitmap getIcon(IconBitmapProvider provider, int listIndex)
	{
		if (_icon != null) {
			return _icon;
		}
		if (provider != null) {
			return provider.getIcon(_iconURL, listIndex);
		}
		return null;
	}
		
	/**
	 * Returns the popularity.
	 */
	public double getPopularity()
	{
		return _popularity;
	}
				
	/**
	 * Returns the readable name under which we register this app as module
	 * group when installing the app.
	 */
	public String getModuleGroupFriendlyName()
	{
		return getName();
	}
	
	/**
	 * Returns a string representation for debugging.
	 */
	public String toString()
	{
		StringBuffer buf = new StringBuffer();
		buf.append("AppSummary{");
		innerToString(buf);
		buf.append("}");
		return buf.toString();
	}
	
	/**
	 * Prepared the inner of toString, for debugging.
	 * @param buf
	 */
	protected void innerToString(StringBuffer buf)
	{
		super.innerToString(buf);
		buf.append("," + _name);
		buf.append("," + _iconURL);
		buf.append("," + _popularity);
		buf.append("," + _latestVersionURL);
		buf.append("," + _ratingAllVersions);
		buf.append("," + _numRatingsAllVersions);
	}
}
