/*
 *  Licensed Materials - Property of IBM
 *  5725-G92 (C) Copyright IBM Corp. 2011, 2013. All Rights Reserved.
 *  US Government Users Restricted Rights - Use, duplication or
 *  disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 */

package com.ibm.appcenter.artifacts;

import net.rim.device.api.i18n.DateFormat;
import org.json.me.JSONException;
import org.json.me.JSONObject;

import com.ibm.appcenter.i18n.I18N;
import com.ibm.appcenter.i18n.I18NResource;
import com.ibm.appcenter.system.BlackBerrySystem;
import com.ibm.appcenter.util.BasicUtil;

/**
 * Smallest possible information about an application
 * with a specific version.
 * This is the shortest variant, used when synchronizing
 * the app list with the server.
 */
public class AppVersion
implements I18NResource
{
	private String _version;
	private String _id;
	private String _url;
	private double _rating;
	private long   _numRatings;
	private long   _updated;
	
	/**
	 * Creates a new application artifact.
	 * @param id The ID of the application artifact.
	 * @param version The version of the application artifact.
	 * @param updated The updated time stamp.
	 */
	public AppVersion(String id, String version, long updated)
	{
		_version = version;
		_id = id;
		_rating = 0;
		_numRatings = 0;
		_updated = updated;
	}
	
	/**
	 * Creates an app summary from JSON.
	 * @param json The JSON object
	 * @param defaultPkg The default id if no id is found in the JSON object.
	 * @throws JSONException
	 */
	public AppVersion(JSONObject json, String defaultId)
	throws JSONException
	{
		if (defaultId == null) {
			_id = json.getString("pkg");
		} else {
			_id = json.optString("pkg", defaultId);
		}
		_version = json.optString("version", "???");
		_url = json.optString("link",null);
		String s = json.optString("updated");
		_updated = BasicUtil.parseDate(s);
		try {
			// optDouble is buggy, hence we simulate it ourself
			_rating = BasicUtil.avgRatingFromServer(json.getDouble("avg_rating"));
		} catch (JSONException ex) {
			_rating = 0.0;
		}
		try {
			// optDouble is buggy, hence we simulate it ourself
			_numRatings = (int)json.getDouble("nb_rating");
		} catch (JSONException ex) {
			_numRatings = 0;
		}
	}
	
	/**
	 * Returns the id of the application.
	 * On the server side, this corresponds to the
	 * package name.
	 */
	public String getId()
	{
		return _id;
	}
		
	/**
	 * Returns the vendor. Overwritten in AppDetail.
	 */
	public String getVendor()
	{
		return null;
	}
	
	/**
	 * Returns the version of the application.
	 */
	public String getVersion()
	{
		return _version;
	}
	
	/**
	 * Returns the URL to get the properties of the application. 
	 */
	public String getURL()
	{
		return _url;
	}
	
	/**
	 * Returns the rating, between 0 and 5.
	 */
	public double getRating()
	{
		return _rating;
	}
	
	/**
	 * Returns the number of rating.
	 */
	public long getNumberOfRatings()
	{
		return _numRatings;
	}
	
	/**
	 * Recalculate the average rating and the number of ratings
	 * when a rating is submitted.
	 * @param submittedRating The submitted rating.
	 */
	public void recalcRatingOnSubmission(int submittedRating)
	{
		double sum = _rating * _numRatings + submittedRating;
		_numRatings++;
		_rating = sum/_numRatings;
	}
	
	/**
	 * Returns the updated time stamp.
	 * This is the time stamp when the application
	 * was updated on the server.
	 */
	public long getUpdated()
	{
		return _updated;
	}
	
	/**
	 * Returns the updated time stamp as a string,
	 * formatted wrt. to the locale of the device.
	 * This is the time stamp when the application
	 * was updated on the server.
	 */
	public String getUpdatedAsString()
	{
		return getTimeAsShortString(_updated);
	}

	/**
	 * Returns the version line of the application version,
	 */
	public String getVersionInfoLine()
	{
		return I18N.getString(AppArtifactVersionInfo, getVersion());
	}
	
	/**
	 * Returns the info line of the application version,
	 */
	public String getUpdatedInfoLine()
	{
		return I18N.getString(AppArtifactUpdatedInfo, getUpdatedAsString());
	}
	
	/**
	 * Returns a time stamp as formatted time string.
	 * @param time The time stamp.
	 */
	private String getTimeAsShortString(long time)
	{
		if (time < 0) return null;
		try {
			DateFormat fmt = DateFormat.getInstance(DateFormat.DATE_SHORT);
			return fmt.formatLocal(time);
		} catch (Exception ex) {
			return null;
		}
	}
	
	/**
	 * Returns the internal name under which we register this app as module
	 * group when installing the app.
	 */
	public String getModuleGroupName()
	{
		// ideally we would have com.ibm.something here.
		// OTA uses vendor:id ... seems no real standard here.
		return BlackBerrySystem.groupIdFromJSON(getId(), getVendor());
	}
	
	/**
	 * Returns whether this application can be installed.
	 */
	public boolean allowInstall()
	{
		// the AppVersion contains not enough information
		// to actually do the install, since we don't know
		// yet whether this app exists in the server.
		return false;
	}
	
	/**
	 * Returns whether this application can be deleted.
	 */
	public boolean allowDelete()
	{
		// the AppVersion contains not enough information
		// to actually do the install, since we don't know
		// yet whether this app exists in the server.
		return false;
	}
	
	/**
	 * Returns a string representation for debugging.
	 */
	public String toString()
	{
		StringBuffer buf = new StringBuffer();
		buf.append("AppVersion{");
		innerToString(buf);
		buf.append("}");
		return buf.toString();
	}
	
	/**
	 * Prepared the inner of toString, for debugging.
	 * @param buf
	 */
	protected void innerToString(StringBuffer buf)
	{
		buf.append("" + _id);
		buf.append("," + _version);
		buf.append("," + _rating);
		buf.append("," + _numRatings);
		buf.append("," + _updated);
		buf.append("," + _url);
	}
}
