/*
 *  Licensed Materials - Property of IBM
 *  5725-G92 (C) Copyright IBM Corp. 2011, 2013. All Rights Reserved.
 *  US Government Users Restricted Rights - Use, duplication or
 *  disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 */

package com.ibm.appcenter.artifacts;

import net.rim.device.api.i18n.DateFormat;
import org.json.me.JSONArray;
import org.json.me.JSONException;
import org.json.me.JSONObject;

import com.ibm.appcenter.util.BasicUtil;
import com.ibm.appcenter.util.Log;

/**
 * A feedback coming from the server.
 */
public class Feedback 
{
	int _rating;
	String _author;
	String _deviceMaker;
	String _deviceModel;
	String _comment;
	String _version;
	long _timeStamp;
	
	public Feedback(JSONObject json)
	throws JSONException
	{
		String s = json.optString("created");
		try {
			_timeStamp = BasicUtil.parseDate(s);
		} catch (Exception ex) {
			Log.log("Feedback constructor", ex);
		}
		_author = json.optString("user_display", null);
		if (_author == null) {
			_author = json.optString("user");
		}
		_comment = json.optString("comment");
		_version = json.optString("package_version");
		_rating = BasicUtil.ratingFromServer(json.optInt("rating"));
		_deviceMaker = "";
		_deviceModel = "";
		JSONObject deviceInfo = json.optJSONObject("deviceInfo");
		if (deviceInfo != null) {
			_deviceMaker = deviceInfo.optString("maker");
			_deviceModel = deviceInfo.optString("model");
		}
	}
	
	/**
	 * Returns the feedbacks from a application feedback response.
	 * @param feedbacks The feedbacks found in the response body.
	 * @param keyToArray The key of the field that contains the array.
	 * @return The list of feedbacks.
	 * @throws JSONException 
	 */
	public static Feedback[] getFeedbacksFromJSON(JSONObject feedbacks, String keyToArray)
	throws JSONException
	{
		JSONArray array = feedbacks.getJSONArray(keyToArray);
		int n = array.length();
		Feedback[] feedbackArray = new Feedback[n];
		for (int i = 0; i < n; i++) {
			JSONObject jsonapp = array.getJSONObject(i);
			feedbackArray[i] = new Feedback(jsonapp);
		}
		return feedbackArray;
	}
	
	/**
	 * Returns the total number of feedbacks in a REST response.
	 * @param feedbacks The feedbacks found in the response body.
	 * @throws JSONException
	 */
	public static int getTotalNumberOfFeedbacks(JSONObject feedbacks)
	throws JSONException
	{
		return feedbacks.optInt("listSize");
	}
	
	/**
	 * Returns the rating.
	 */
	public int getRating()
	{
		return _rating;
	}
	
	/**
	 * Returns the author.
	 */
	public String getAuthor()
	{
		return _author;
	}
	
	/**
	 * Returns the comment.
	 */
	public String getComment()
	{
		return _comment;
	}
	
	/**
	 * Returns the version of the app that has this comment.
	 */
	public String getVersion()
	{
		return _version;
	}
	
	/**
	 * Returns the maker of the device from where the
	 * feedback was sent. In our case, this is always
	 * BlackBerry, hence it is not so interesting.
	 *
	public String getDeviceMaker()
	{
		return _deviceMaker;
	}
	 */

	/**
	 * Returns the model of the device from where the
	 * feedback was sent.
	 */
	public String getDeviceModel()
	{
		return _deviceModel;
	}
	
	/**
	 * Returns the time stamp of the feedback.
	 */
	public long getTimeStamp()
	{
		return _timeStamp;
	}
	
	/**
	 * Returns the time stamp of the feedback as a string,
	 * formatted wrt. to the locale of the device.
	 */
	public String getTimeStampAsString()
	{
		return getTimeAsString(_timeStamp);
	}
	
	/**
	 * Returns a time stamp as formatted time string.
	 * @param time The time stamp.
	 */
	private String getTimeAsString(long time)
	{
		if (time < 0) return null;
		try {
			DateFormat fmt = DateFormat.getInstance(DateFormat.DATETIME_DEFAULT);
			return fmt.formatLocal(time);
		} catch (Exception ex) {
			return null;
		}
	}
	
	/**
	 * Returns a string representation for debugging.
	 */
	public String toString()
	{
		StringBuffer buf = new StringBuffer();
		buf.append("Feedback{");
		buf.append("" + _rating);
		buf.append("," + _author);
		buf.append("," + _deviceMaker);
		buf.append("," + _deviceModel);
		buf.append("," + _comment);
		buf.append("," + _version);
		buf.append("," + _timeStamp);
		buf.append("}");
		return buf.toString();
	}
}
