/*
 *  Licensed Materials - Property of IBM
 *  5725-G92 (C) Copyright IBM Corp. 2011, 2013. All Rights Reserved.
 *  US Government Users Restricted Rights - Use, duplication or
 *  disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 */

package com.ibm.appcenter.connection;

import org.json.me.JSONException;
import org.json.me.JSONObject;

import com.ibm.appcenter.artifacts.AppDetail;
import com.ibm.appcenter.artifacts.CodModule;
import com.ibm.appcenter.system.BlackBerrySystem;
import com.ibm.appcenter.ui.UIUtil;
import com.ibm.appcenter.util.Log;
import com.ibm.appcenter.util.ProgressIndicator;

/**
 * Connection utilities for applications.
 */
public class ApplicationUtil {
		
	/**
	 * Private constructor since this is a singleton.
	 */
	private ApplicationUtil() {}
	
	/**
	 * Retrieves the catalog for this device.
	 * @param batch          The batch that controls the execution of REST calls.
	 * @param cont           The continuation.
	 * @param contInEvThread Whether to run the continuation in event thread.
	 * @param handleBusyInd  Whether to handle the busy indicator.
	 */
	public static void retrieveCatalog(
			RESTCallBatch batch,
			JSONRESTCallContinuation cont,
			boolean contInEvThread,
			boolean handleBusyInd)
	{
		Log.log("retrieveCatalog");					
		RESTCall call = 
				new JSONGetCall(ConnectionUtil.getCatalogURI(), 
				                cont, 
				                contInEvThread,
				                handleBusyInd);
		batch.prepend(call);
		batch.ensureBatchLength(1);
		batch.start();
	}
	
	/**
	 * Retrieves the list of installed apps for this device.
	 * @param batch          The batch that controls the execution of REST calls.
	 * @param cont           The continuation.
	 * @param contInEvThread Whether to run the continuation in event thread.
	 * @param handleBusyInd  Whether to handle the busy indicator.
	 */
	public static void retrieveMyApps(
			RESTCallBatch batch,
			JSONRESTCallContinuation cont,
			boolean contInEvThread,
			boolean handleBusyInd)
	{
		Log.log("retrieveMyApps");
		RESTCall call = 
				new JSONGetCall(ConnectionUtil.getMyAppsURI(), 
				                cont, 
				                contInEvThread,
				                handleBusyInd);
		batch.prepend(call);
		batch.ensureBatchLength(1);
		batch.start();
	}
	
	/**
	 * Retrieves the list of updated apps for this device.
	 * @param batch          The batch that controls the execution of REST calls.
	 * @param cont           The continuation.
	 * @param contInEvThread Whether to run the continuation in event thread.
	 * @param handleBusyInd  Whether to handle the busy indicator.
	 */
	public static void retrieveUpdates(
			RESTCallBatch batch,
			JSONRESTCallContinuation cont,
			boolean contInEvThread,
			boolean handleBusyInd)
	{
		Log.log("retrieveUpdatedApps");
		RESTCall call = 
				new JSONGetCall(ConnectionUtil.getUpdatedAppsURI(), 
				                cont, 
				                contInEvThread,
				                handleBusyInd);
		batch.prepend(call);
		batch.ensureBatchLength(1);
		batch.start();
	}
	
	/**
	 * Retrieves the properties of the maximal version of an app.
	 * @param uri            The URI of the application.
	 * @param batch          The batch that controls the execution of REST calls.
	 * @param cont           The continuation.
	 * @param contInEvThread Whether to run the continuation in event thread.
	 * @param handleBusyInd  Whether to handle the busy indicator.
	 */
	public static void retrieveAppProperties(
			String uri,
			RESTCallBatch batch,
			JSONRESTCallContinuation cont,
			boolean contInEvThread,
			boolean handleBusyInd)
	{
		Log.log("retrieveAppProperties");
		RESTCall call = 
				new JSONGetCall(ConnectionUtil.getURI(uri), 
				                cont, 
				                contInEvThread,
				                handleBusyInd);
		batch.prepend(call);
		batch.ensureBatchLength(1);
		batch.start();
	}
		
	/**
	 * Retrieves the data for cod modules for an application.
	 * @param app  The application.
	 * @param cont The continuation. It's failure and cancel methods are
	 *    called immediately, but the success method is only called after
	 *    all modules where successfully loaded.
	 *    Note that the continuation is NOT called in the event thread.
	 */
	public static void retrieveCodModulesData(
			AppDetail app,
			final RESTCallBatch batch,
			final RESTCallContinuation cont,
			final ProgressIndicator progress)
	{
		Log.log("retrieveCodModulesData");

		// need to chain all before starting the call
		if (progress != null) {
			progress.setProgressValue(0);
		}
		CodModule[] modules = app.getModules();
		if (modules == null) return;
		String user = ConnectionUtil.getUsername();
		String pass = ConnectionUtil.getPassword();
		for (int i = modules.length - 1; i >= 0; i--) {
			final CodModule module = modules[i];
			final boolean last = (i == modules.length - 1);
			final int index = i;
			RESTCall c = 
				new RESTCall(
					ConnectionUtil.getURI(module.getURL()),
					user, pass,
					RESTCall.GET,
					null,
					null,
					ConnectionUtil.getTimeout(),
					new RESTCallContinuation() {

						public void onSuccess(String contentType, byte[] bytes) {
							Log.log(module.getFileName() + " done. Received " + (bytes == null ? "no" : "" + bytes.length) + " bytes");
							if (bytes.length == module.getSize()) {
								module.setData(bytes);
								if (progress != null) {
									progress.increaseProgressValueTo(index + 1);
								}
								if (last) {
									cont.onSuccess(contentType, bytes);
								}
							} else {
								Log.log("Wrong size received: " + bytes.length + " vs. expected " + module.getSize());
								cont.onFailure(null, -1, contentType, bytes);
								batch.stop();
							}
						}

						public void onFailure(Exception ex, int status,
								String contentType, byte[] bytes) {
							Log.log(module.getFileName() + " failed.");
							cont.onFailure(ex, status, contentType, bytes);
							batch.stop();
						}

						public void onCancel() {
							Log.log(module.getFileName() + " canceled.");
							cont.onCancel();
							batch.stop();
						}},
					false,
					false);
			batch.prepend(c);
		}
		
		Log.log(app.getName() + " started.");
		batch.ensureBatchLength(modules.length);
		batch.start();
	}
	
	/**
	 * Returns the REST call that records the success of an installation or
	 * deinstallation to the server.
	 * @param app The app.
	 * @param wasInstalled Whether the app was previously installed.
	 * @param progress The progress indicator. At the end of this
	 *    call, the progess indicator will go to max.
	 * @param cleanup The action that needs to be done after all.
	 */
	public static RESTCall getRecordInstallationResultCall(
			AppDetail app,
			boolean wasInstalled,
			final ProgressIndicator progress,
			final Runnable cleanup)
	{	
		boolean isInstalled = BlackBerrySystem.isModuleGroupInstalled(app.getModuleGroupName());
		if (isInstalled) {
			return getRecordInstallationCall(app, progress, cleanup);
		} else {
			if (wasInstalled) {
				return getRecordDeinstallationCall(app, progress, cleanup);
			}
		}
		return null;
	}
	
	/**
	 * Returns the REST call that records an installation to the server.
	 * @param app The app.
	 * @param progress The progress indicator. At the end of this
	 *    call, the progess indicator will go to max.
	 * @param cleanup The action that needs to be done after all.
	 */
	private static RESTCall getRecordInstallationCall(
			AppDetail app,
			final ProgressIndicator progress,
			final Runnable cleanup)
	{	
		JSONObject payload = new JSONObject();
		try {
			payload.put("pkg", app.getId());
			payload.put("version", app.getVersion());
			payload.put("to_update", "Y");
		} catch (JSONException ex) {
			// should never happen
			Log.log("getRecordInstallationSuccessCall", ex);
		}

		RESTCall call = new JSONPostCall(
					ConnectionUtil.getInstallBookkeepingURI(),
			   	   	payload,
			   	   	new JSONRESTCallContinuation() {

						public void onSuccess(JSONObject response,
								String contentType, byte[] bytes) {
							Log.log("Recording app installation done");
							if (progress != null) {
								progress.increaseProgressValueTo(Integer.MAX_VALUE);
							}
							callCleanupInEventThread();
						}

						public void onFailure(Exception ex, int status,
								JSONObject response, String contentType,
								byte[] bytes) {
							// We could handle conflicts here more throughfully.
							// Right now conflicts are simply ignored, not fatal, since
							// we synchronize anyway when next time starting the App Center.
							Log.log("Recording app installation failed " + status, ex);
							callCleanupInEventThread();
						}

						public void onCancel() {
							callCleanupInEventThread();
						}
						
						private void callCleanupInEventThread() {
							if (cleanup != null) {
								UIUtil.invokeInEventThread(cleanup);
							}
						}
					},
			   	   	false,
			   	   	false);
		
		return call;
	}
	
	/**
	 * Returns the REST call that records a deinstallation to the server.
	 * @param app The app.
	 * @param progress The progress indicator. At the end of this
	 *    call, the progess indicator will go to max.
	 * @param cleanup The action that needs to be done after all.
	 */
	private static RESTCall getRecordDeinstallationCall(
			AppDetail app,
			final ProgressIndicator progress,
			final Runnable cleanup)
	{	
		RESTCall call = new JSONDeleteCall(
					ConnectionUtil.getUninstallBookkeepingURI(app.getId()),
			   	   	new JSONRESTCallContinuation() {

						public void onSuccess(JSONObject response,
								String contentType, byte[] bytes) {
							Log.log("Recording app removal done");
							if (progress != null) {
								progress.increaseProgressValueTo(Integer.MAX_VALUE);
							}
							callCleanupInEventThread();
						}

						public void onFailure(Exception ex, int status,
								JSONObject response, String contentType,
								byte[] bytes) {
							// We could handle conflicts here more throughfully.
							// Right now conflicts are simply ignored, not fatal, since
							// we synchronize anyway when next time starting the App Center.
							Log.log("Recording app removal failed " + status, ex);
							callCleanupInEventThread();
						}

						public void onCancel() {
							callCleanupInEventThread();
						}
						
						private void callCleanupInEventThread() {
							if (cleanup != null) {
								UIUtil.invokeInEventThread(cleanup);
							}
						}
					},
			   	   	false,
			   	   	false);
		
		return call;
	}
}
