/*
 *  Licensed Materials - Property of IBM
 *  5725-G92 (C) Copyright IBM Corp. 2011, 2013. All Rights Reserved.
 *  US Government Users Restricted Rights - Use, duplication or
 *  disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 */

package com.ibm.appcenter.connection;

import java.io.UnsupportedEncodingException;

import org.json.me.JSONObject;

import com.ibm.appcenter.i18n.I18N;
import com.ibm.appcenter.i18n.I18NResource;
import com.ibm.appcenter.system.BlackBerrySystem;
import com.ibm.appcenter.ui.ConfigurationData;
import com.ibm.appcenter.util.Log;

import net.rim.device.api.i18n.Locale;
import net.rim.device.api.system.CoverageInfo;

/**
 * Common utilities for connection.
 */
public class ConnectionUtil {
	
	private static String _username;
	private static String _password;
	private static String _connectionPrefix;
	private static String _connectionSuffix = ""; // ";deviceside=true"; // + ";interface=wifi";
	private static int _timeout = 10000;
		
	/**
	 * Private constructor since this is a singleton.
	 */
	private ConnectionUtil() {}
	
	/**
	 * Sets the common connection data for all REST requests.
	 * @param server   The server. Cannot be null.
	 * @param ssl      Whether SSL is used.
	 * @param username The user name.
	 * @param password The password.
	 * @param suffix   The blackberry connection suffix.
	 * @param timeout  The timeout.
	 */
	public static void setConnection(
			String server,
			boolean ssl,
			String username,
			String password,
			String suffix,
			int timeout)
	{
		_username = username == null ? null : username.trim();
		_password = password == null ? null : password.trim();
		_timeout = timeout;
		
		server = server.trim();
		if (!server.endsWith("/")) {
			server = server + "/";
		}
		
		if (server.startsWith("http:") || server.startsWith("https:")) {
			_connectionPrefix = server + "service/";
		} else if (ssl) {
			_connectionPrefix = "https://" + server + "service/";
		} else {
			_connectionPrefix = "http://" + server + "service/";
		}
		
		if (suffix == null || suffix.length() == 0)
			suffix = BlackBerrySystem.getAutoConnectionSuffix();
		
		if (suffix == null || suffix.length() == 0) {
			_connectionSuffix = "";
		} else {
			if (suffix.startsWith(";")) {
				_connectionSuffix = suffix;
			} else {
				_connectionSuffix = ";" + suffix;
			}
		}
	}

	/**
	 * Sets the connection from the stored configuration.
	 * @param conf The configuration.
	 */
	public static void setConnection(ConfigurationData conf)
	{
		if (conf.getServer() != null) {
			setConnection(
				conf.getServer(),
				conf.isSSL(),
				conf.getUser(),
				conf.getPassword(),
				conf.getConnectionSuffix(),
				conf.getTimeout());
		}
	}
	
	/**
	 * Returns the user name used in all REST services
	 * unless explicitly specified.
	 */
	public static String getUsername()
	{
		return _username;
	}
	
	/**
	 * Returns the password used in all REST services
	 * unless explicitly specified.
	 */
	public static String getPassword()
	{
		return _password;
	}
	
	/**
	 * Returns the timeout used in all REST services
	 * unless explicitly specified.
	 */
	public static int getTimeout()
	{
		return _timeout;
	}
	
	/**
	 * Returns whether the device has coverage for HTTP.
	 * @return
	 */
	static boolean hasCoverage()
	{
		if (CoverageInfo.isCoverageSufficient(CoverageInfo.COVERAGE_DIRECT))
			return true;
		if (CoverageInfo.isCoverageSufficient(CoverageInfo.COVERAGE_MDS))
			return true;
		if (CoverageInfo.isCoverageSufficient(CoverageInfo.COVERAGE_BIS_B))
			return true;
		// if (DeviceInfo.isSimulator())
		//	 	return true;
		return false;
	}
	
	// ----------------------------------------------------------
	// URI
	
	/**
	 * Returns the URI for some url.
	 */
	public static String getURI(String url)
	{
		if (url == null) {
			Log.log("ConnectionUtil getURI Null URL");
			return "";
		}
		// send time stamp to easier identify on the server which
		// call is which
		long t = System.currentTimeMillis();
		if (url.indexOf("?") > 0)
			return url + "&_t=" + t + _connectionSuffix;
		else
			return url + "?_t=" + t + _connectionSuffix;
	}
		
	/**
	 * Returns the URI for registering the device.
	 */
	public static String getDeviceURI()
	{
		return getURI(_connectionPrefix + "device");
	}
	
	/**
	 * Returns the URI for synchronizing the device installed apps.
	 */
	public static String getSynchronizeURI(String deviceId)
	{
		return getURI(_connectionPrefix + "device/" + toURI(deviceId) + "/synchronization");
	}
	
	/**
	 * Returns the URI for synchronizing the device installed apps.
	 */
	public static String getSynchronizeURI()
	{
		return getSynchronizeURI(BlackBerrySystem.getDeviceId());
	}
	
	/**
	 * Returns the URI for retrieving the device catalog.
	 */
	public static String getCatalogURI(String deviceId)
	{
		return getURI(_connectionPrefix + "device/" + toURI(deviceId) + "/catalog?condense=true&exclude_installed=false");
	}
	
	/**
	 * Returns the URI for retrieving the device catalog.
	 */
	public static String getCatalogURI()
	{
		return getCatalogURI(BlackBerrySystem.getDeviceId());
	}
	
	/**
	 * Returns the URI for retrieving the installed apps list.
	 * Those are the apps that the server thinks are installed.
	 */
	public static String getMyAppsURI(String deviceId)
	{
		return getURI(_connectionPrefix + "device/" + toURI(deviceId) + "/package?condense=true");
	}
	
	/**
	 * Returns the URI for retrieving the installed apps list.
	 * Those are the apps that the server thinks are installed.
	 */
	public static String getMyAppsURI()
	{
		return getMyAppsURI(BlackBerrySystem.getDeviceId());
	}
	
	/**
	 * Returns the URI for retrieving the updated apps list.
	 * Those are the apps that the server has a newer version
	 * than the installed version.
	 */
	public static String getUpdatedAppsURI(String deviceId)
	{
		return getURI(_connectionPrefix + "device/" + toURI(deviceId) + "/updates?condense=true");
	}
	
	/**
	 * Returns the URI for retrieving the updated apps list.
	 * Those are the apps that the server has a newer version
	 * than the installed version.
	 */
	public static String getUpdatedAppsURI()
	{
		return getUpdatedAppsURI(BlackBerrySystem.getDeviceId());
	}
	
	/**
	 * Returns the URI for registering that an application was installed.
	 */
	public static String getInstallBookkeepingURI(String deviceId)
	{
		return getURI(_connectionPrefix + "device/" + toURI(deviceId) + "/package");
	}
	
	/**
	 * Returns the URI for registering that an application was installed.
	 */
	public static String getInstallBookkeepingURI()
	{
		return getInstallBookkeepingURI(BlackBerrySystem.getDeviceId());
	}
	
	/**
	 * Returns the URI for registering that an application was uninstalled.
	 */
	public static String getUninstallBookkeepingURI(String deviceId, String id)
	{
		return getURI(_connectionPrefix + "device/" + toURI(deviceId) + "/package/" + toURI(id));
	}
	
	/**
	 * Returns the URI for registering that an application was uninstalled.
	 */
	public static String getUninstallBookkeepingURI(String id)
	{
		return getUninstallBookkeepingURI(BlackBerrySystem.getDeviceId(), id);
	}
	
	
	/**
	 * Returns the URI for retrieving the list feedbacks for an application.
	 * This retrieves not the full list but only a page of the list.
	 * This is for a specific version.
	 * @param id The id of the application.
	 * @param version The version of the application. Can be null, for all versions.
	 * @param startIndex The start index of the page.
	 * @param pageSize The page size.
	 * @param orderKey The ordering key.
	 * @param ascending Whether the ordering is ascending or descending.
	 */
	public static String getFeedbacksURI(
							String id,
							String version,
							int startIndex,
							int pageSize,
							String orderKey,
							boolean ascending)
	{
		if (version == null) {
			return getFeedbacksURINoVersion(
					id, startIndex, pageSize, orderKey, ascending);
		} else {
			return getFeedbacksURIWithVersion(
					id, version, startIndex, pageSize, orderKey, ascending);
		}
	}
	
	/**
	 * Returns the URI for retrieving the list feedbacks for an application.
	 * This retrieves not the full list but only a page of the list.
	 * This is for a specific version.
	 * @param id The id of the application.
	 * @param version The version of the application.
	 * @param startIndex The start index of the page.
	 * @param pageSize The page size.
	 * @param orderKey The ordering key.
	 * @param ascending Whether the ordering is ascending or descending.
	 */
	public static String getFeedbacksURIWithVersion(
							String id,
							String version,
							int startIndex,
							int pageSize,
							String orderKey,
							boolean ascending)
	{
		Locale locale = Locale.getDefault();
		return getURI(_connectionPrefix + "application/feedback/Blackberry/" + 
				toURI(id) + "/" + toURI(version) +
				"?fromIndex=" + startIndex +
				"&pageSize=" + pageSize +
				"&orderBy=" + (ascending ? "" : "-") + orderKey +
				"&locale=" + locale.toString());
	}
	
	/**
	 * Returns the URI for retrieving the list feedbacks for an application.
	 * This retrieves not the full list but only a page of the list.
	 * This is for all versions.
	 * @param id The id of the application.
	 * @param startIndex The start index of the page.
	 * @param pageSize The page size.
	 * @param orderKey The ordering key.
	 * @param ascending Whether the ordering is ascending or descending.
	 */
	public static String getFeedbacksURINoVersion(
							String id,
							int startIndex,
							int pageSize,
							String orderKey,
							boolean ascending)
	{
		Locale locale = Locale.getDefault();
		return getURI(_connectionPrefix + "application/combinedfeedback/Blackberry/" + 
				toURI(id) +
				"?fromIndex=" + startIndex +
				"&pageSize=" + pageSize +
				"&orderBy=" + (ascending ? "" : "-") + orderKey +
				"&locale=" + locale.toString());
	}
	
	/**
	 * Returns the URI for submitting a feedback.
	 * @param id The id of the application.
	 * @param version The version of the application.
	 */
	public static String getSubmitFeedbackURI(String id, String version)
	{
		return getURI(_connectionPrefix + "application/feedback/Blackberry/" + toURI(id) + "/" + toURI(version));
	}
		
	/**
	 * Returns the test URI. Used for debugging.
	 */
	public static String getTestURI(String id)
	{
		return getURI(_connectionPrefix + "testuriutil/" + toURI(id));
	}
	
	public static void testURI()
	{
		String s = "/\\;$#?";
		Log.log("testURI " + s + " " + toURI(s));
		RESTCallBatch batch = new RESTCallBatch(null);
		RESTCall call = 
				new JSONGetCall(ConnectionUtil.getTestURI(s), 
				                new JSONRESTCallContinuation() {

									public void onSuccess(JSONObject response,
											String contentType, byte[] bytes) {
										// TODO Auto-generated method stub
										Log.log("received: " + response.optString("received"));
										Log.log("decoded: " + response.optString("decoded"));
										Log.log("encoded: " + response.optString("encoded"));
									}

									public void onFailure(Exception ex,
											int status, JSONObject response,
											String contentType, byte[] bytes) {
										Log.log("testURI fail");
									}

									public void onCancel() {
										Log.log("testURI cancel");
									}}, 
				                true,
				                false);
		batch.prepend(call);
		batch.ensureBatchLength(1);
		batch.start();
	}
	
	/**
	 * Encode a string that occurs on and URI.
	 * Corresponds to Javascript's encodeURIComponent.
	 */
	private static String toURI(String s)
	{
		if (s == null)
			return s;
		StringBuffer buf = new StringBuffer();
		byte[] bytes;
		try {
			bytes = s.getBytes("UTF-8");
		} catch (UnsupportedEncodingException e) {
		    bytes = s.getBytes();
		}
		// Note: Wink/Jersey requires us to double escape % for /\;$#?
		for (int i = 0; i < bytes.length; ++i) {
			char b = (char) bytes[i];
			if (b == ' ') {
				// for urlEncode, use '+', for urlEncodeComponent, use "%20"
				buf.append("%20");
			}
			else if (b == '/') {
				// double escape the % (%25 = %)
				buf.append("%252F");
			}
			else if (b == '\\') {
				// double escape the % (%25 = %)
				buf.append("%255C");
			}
			else if (b == ';') {
				// double escape the % (%25 = %)
				buf.append("%253B");
			}
			else if (b == '#') {
				// double escape the % (%25 = %)
				buf.append("%2523");
			}
			else if (b == '?') {
				// double escape the % (%25 = %)
				buf.append("%253F");
			}
			else if (isUnreserved(b)) {
				// normal character
				buf.append(b);
			}
			else {
				// use %XY with XY upper case hex
				buf.append('%');
				buf.append(Integer.toHexString(b).toUpperCase());
			}
		}
		return buf.toString();
	}

	private static boolean isUnreserved(char c)
	{
		return (c >= 'A' && c <= 'Z') || (c >= 'a' && c <= 'z') || Character.isDigit(c) ||
				// for urlEncode, use ".-~_:/;?".indexOf
				".-~_".indexOf(c) >= 0;
	}
	
	/**
	 * Converts a HTTP status into a message.
	 */
	public static String getStatusMessage(int status)
	{
		// we don't handle all possibilities here, only the common errors
		if (status >= 200 && status < 300) {
			return I18N.getString(I18NResource.ConnectionUtilSuccessMsg);
		}
		switch (status) {
			case 0:
				return I18N.getString(I18NResource.ConnectionUtilNullStatusMsg);
			case 401:
				return I18N.getString(I18NResource.ConnectionUtilUnauthorizedMsg);
			case 403:
				return I18N.getString(I18NResource.ConnectionUtilForbiddenMsg);
			case 408:
				return I18N.getString(I18NResource.ConnectionUtilTimeoutMsg);
			case 500:
				return I18N.getString(I18NResource.ConnectionUtilInternalErrorMsg);
			case 503:
				return I18N.getString(I18NResource.ConnectionUtilUnavailableMsg);
		}
		return I18N.getString(I18NResource.ConnectionUtilGenericStatusMsg, "" + status);
	}
}
