/*
 *  Licensed Materials - Property of IBM
 *  5725-G92 (C) Copyright IBM Corp. 2011, 2013. All Rights Reserved.
 *  US Government Users Restricted Rights - Use, duplication or
 *  disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 */

package com.ibm.appcenter.connection;

import javax.microedition.io.HttpConnection;

import org.json.me.JSONArray;
import org.json.me.JSONException;
import org.json.me.JSONObject;

import com.ibm.appcenter.artifacts.AppVersion;
import com.ibm.appcenter.i18n.I18N;
import com.ibm.appcenter.i18n.I18NResource;
import com.ibm.appcenter.system.BlackBerrySystem;
import com.ibm.appcenter.util.Log;

public class DeviceUtil 
implements I18NResource
{
	private static boolean _deviceIsRegistered;
	
	/**
	 * Private constructor since this is a singleton.
	 */
	private DeviceUtil() {}
		
	/**
	 * Returns whether the device is registered.
	 */
	public static boolean isRegistered()
	{
		return _deviceIsRegistered;
	}
	
	/**
	 * Registers this device. Works even when the device was previously registered.
	 * @param batch          The batch this call is executed inside.
	 * @param timeout        The timeout in milliseconds.
	 * @param cont           The continuation.
	 * @throws JSONException
	 */
	public static void registerDevice(
			RESTCallBatch batch,
			int timeout,
			final JSONRESTCallContinuation cont)
	throws JSONException
	{
		Log.log("registerDevice");
		_deviceIsRegistered = false;
		RESTCall call = getRegisterDeviceCall(batch, timeout, cont);		
		batch.prepend(call);
		batch.ensureBatchLength(1);
		batch.start();
	}
	
	/**
	 * Returns the call that can register this device.
	 * @param batch          The batch this call is executed inside.
	 * @param timeout        The timeout in milliseconds.
	 * @param cont           The continuation.
	 * @throws JSONException
	 */
	private static RESTCall getRegisterDeviceCall(
			final RESTCallBatch batch,
			final int timeout,
			final JSONRESTCallContinuation cont)
	throws JSONException
	{
		Log.log("getRegisterDeviceCall");
		String nick = I18N.getString(DeviceDescription, BlackBerrySystem.getDeviceModel(), ConnectionUtil.getUsername());
		
		JSONObject deviceInfo = new JSONObject();
		try {
			deviceInfo.put("device_id", BlackBerrySystem.getDeviceId());
			deviceInfo.put("os", "Blackberry");
			deviceInfo.put("os_description", BlackBerrySystem.getDeviceOsDescription());
			deviceInfo.put("maker", BlackBerrySystem.getDeviceMaker());
			deviceInfo.put("model", BlackBerrySystem.getDeviceModel());
			deviceInfo.put("nickname", nick);
		} catch (JSONException e) {
			throw e;
		}
		
		JSONRESTCallContinuation myCont = new JSONRESTCallContinuation() {

			public void onSuccess(JSONObject response, String contentType,
					byte[] bytes) {
				Log.log("Device registered successfully");
				// mark as registered and schedule synchronization
				_deviceIsRegistered = true;
				try {
					RESTCall next = DeviceUtil.getSynchronizeDeviceCall(batch, timeout, cont);
					batch.appendAtRunning(next);
				} catch (JSONException ex) {
					// should never happen (unless we have a serious bug in our JSON construction code
					Log.log("Error when calling getSynchronizeDeviceCall", ex);
					// clearBusy early since onFailure can pop up dialogs
					batch.clearBusyIndicator();
					batch.callOnFailureInEventThread(cont, ex, -1, null, null, null);
				}
			}

			public void onFailure(Exception ex, int status,
					JSONObject response, String contentType, byte[] bytes) {
				if (status == HttpConnection.HTTP_CONFLICT) {
					Log.log("Device registered successfully with conflict");
					// we ignore conflicts:
					// mark as registered and schedule synchronization
					_deviceIsRegistered = true;
					try {
						RESTCall next = DeviceUtil.getSynchronizeDeviceCall(batch, timeout, cont);
						batch.appendAtRunning(next);
					} catch (JSONException ex1) {
						// should never happen (unless we have a serious bug in our JSON construction code
						Log.log("Error when calling getSynchronizeDeviceCall", ex1);
						// clearBusy early since onFailure can pop up dialogs
						batch.clearBusyIndicator();
						batch.callOnFailureInEventThread(cont, ex, -1, null, null, null);
					}
					// TODO: still suboptimal. In case of conflict, should update the device with new info
				} else {
					Log.log("Failed to register device. Status=" + status);
					// clearBusy early since onFailure can pop up dialogs
					batch.clearBusyIndicator();
					batch.callOnFailureInEventThread(cont, ex, status, response, contentType, bytes);
				}
			}

			public void onCancel() {
				Log.log("Canceled to register device");
				batch.clearBusyIndicator();
				batch.callOnCancelInEventThread(cont);
			}
			
		};
		
		RESTCall call = 
				new JSONPostCall(ConnectionUtil.getDeviceURI(), 
				                 deviceInfo, 
				                 timeout, 
				                 myCont, 
				                 false,
				                 false);
				
		return call;
	}
	
	/**
	 * Returns the call that can synchronize this device.
	 * @param batch          The batch this call is executed inside.
	 * @param timeout        The timeout in milliseconds.
	 * @param cont           The continuation.
	 * @throws JSONException
	 */
	private static RESTCall getSynchronizeDeviceCall(
			final RESTCallBatch batch,
			final int timeout,
			final JSONRESTCallContinuation cont)
	throws JSONException
	{
		Log.log("getSynchronizeDeviceCall");
		
		JSONObject appsInfo = new JSONObject();
		JSONArray apps = new JSONArray();
		try {
			AppVersion[] appvers = BlackBerrySystem.getInstalledApplications();
			int n = appvers.length;
			for (int i = 0; i < n; i++) {
				String id = appvers[i].getId();
				String version = appvers[i].getVersion();
				JSONObject app = new JSONObject();
				app.put("pkg", id);
				app.put("version", version);
				app.put("to_update", "Y");
				apps.put(app);
			}			
			appsInfo.put("apps", apps);
		} catch (JSONException e) {
			// log(e.getMessage() + "Failed to create JSON object for device " + getDeviceIdNumber());
			throw e;
		}
		
		JSONRESTCallContinuation myCont = new JSONRESTCallContinuation() {

			public void onSuccess(JSONObject response, String contentType,
					byte[] bytes) {
				Log.log("Device synchronized sucessfully");
				_deviceIsRegistered = true;
				batch.callOnSuccessInEventThread(cont, response, contentType, bytes);
				// here we intentionally clear the busy indicator late
				// since sometimes the work on success is lengthy, but
				// does not pop up dialogs. Otherwise it would be better
				// to do it early to avoid that the busy indicator interfers
				// with dialogs.
				batch.clearBusyIndicator();
			}

			public void onFailure(Exception ex, int status,
					JSONObject response, String contentType, byte[] bytes) {
				Log.log("Failed to synchronize device. Status=" + status);
				// clearBusy early since onFailure can pop up dialogs
				batch.clearBusyIndicator();
				batch.callOnFailureInEventThread(cont, ex, status, response, contentType, bytes);
			}

			public void onCancel() {
				Log.log("Canceled to synchronize device");
				// clearBusy early since onCancel can pop up dialogs
				batch.clearBusyIndicator();
				batch.callOnCancelInEventThread(cont);
			}
			
		};
		
		RESTCall call = 
				new JSONPutCall(ConnectionUtil.getSynchronizeURI(), 
				                appsInfo, 
				                timeout, 
				                myCont, 
				                false,
				                false);
				
		return call;
	}
}
