/*
 *  Licensed Materials - Property of IBM
 *  5725-G92 (C) Copyright IBM Corp. 2011, 2013. All Rights Reserved.
 *  US Government Users Restricted Rights - Use, duplication or
 *  disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 */
package com.ibm.appcenter.connection;

import org.json.me.JSONException;
import org.json.me.JSONObject;

import com.ibm.appcenter.system.BlackBerrySystem;
import com.ibm.appcenter.util.BasicUtil;
import com.ibm.appcenter.util.Log;

/**
 * Static utility for the REST API calls to handle a feedback.
 */
public class FeedbackUtil 
{
	/**
	 * Retrieves the list of feedbacks for an application version.
	 * This retrieves not the full list but only a page of the list.
	 * @param id The id of the application.
	 * @param version The version of the application. Can be null, for
	 *                  all versions.
	 * @param startIndex The start index of the page.
	 * @param pageSize The page size.
	 * @param orderKey The ordering key.
	 * @param ascending Whether the ordering is ascending or descending.
	 * @param batch          The batch that controls the execution of REST calls.
	 * @param cont           The continuation.
	 * @param contInEvThread Whether to run the continuation in event thread.
	 * @param handleBusyInd  Whether to handle the busy indicator.
	 */
	public static void retrieveFeedbacks(
			String id,
			String version,
			int startIndex,
			int pageSize,
			String orderKey,
			boolean ascending,
			RESTCallBatch batch,
			JSONRESTCallContinuation cont,
			boolean contInEvThread,
			boolean handleBusyInd)
	{
		Log.log("retrieveUpdatedApps");
		RESTCall call = 
				new JSONGetCall(ConnectionUtil.getFeedbacksURI(
							id, version, startIndex, pageSize,
							orderKey, ascending), 
				        cont, 
				        contInEvThread,
				        handleBusyInd);
		batch.prepend(call);
		batch.ensureBatchLength(1);
		batch.start();
	}
	
	/**
	 * Submits a feedback.
	 * @param id             The application id.
	 * @param version        The application version.
	 * @param rating         The rating value (0 .. 5).
	 * @param comment        The comment.
	 * @param batch          The batch this call is executed inside.
	 * @param cont           The continuation.
	 * @throws JSONException
	 */
	public static void submitFeedback(
			String id,
			String version,
			int rating,
			String comment,				
			RESTCallBatch batch,
			JSONRESTCallContinuation cont)
	throws JSONException
	{
		Log.log("submitFeedback");		
		RESTCall call = getSubmitFeedbackCall(id, version, rating, comment, batch, cont);		
		batch.prepend(call);
		batch.ensureBatchLength(1);
		batch.start();
	}
	
	/**
	 * Returns the call that can submit a feedback.
	 * @param id             The application id.
	 * @param version        The application version.
	 * @param rating         The rating value (0 .. 5).
	 * @param comment        The comment.
	 * @param batch          The batch this call is executed inside.
	 * @param cont           The continuation.
	 * @throws JSONException
	 */
	private static RESTCall getSubmitFeedbackCall(
			String id,
			String version,
			int rating,
			String comment,			
			RESTCallBatch batch,
			JSONRESTCallContinuation cont)
	throws JSONException
	{
		Log.log("getSubmitFeedbackCall");
		
		JSONObject feedbackInfo = new JSONObject();
		try {
			feedbackInfo.put("device_id", BlackBerrySystem.getDeviceId());
			feedbackInfo.put("comment", comment);
			feedbackInfo.put("rating", BasicUtil.ratingToServer(rating));
		} catch (JSONException e) {
			throw e;
		}
				
		RESTCall call = 
				new JSONPostCall(ConnectionUtil.getSubmitFeedbackURI(id, version), 
				                 feedbackInfo, 
				                 cont, 
				                 true,
				                 true);
				
		return call;
	}
}
