/*
 *  Licensed Materials - Property of IBM
 *  5725-G92 (C) Copyright IBM Corp. 2011, 2013. All Rights Reserved.
 *  US Government Users Restricted Rights - Use, duplication or
 *  disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 */

package com.ibm.appcenter.connection;

import org.json.me.JSONArray;
import org.json.me.JSONException;
import org.json.me.JSONObject;

/**
 * Base class for all JSON based REST calls.
 * Rest calls are executed asynchronously, that is, they open
 * an own thread. When a valid REST response is received, the
 * REST call executes the success continuation, otherwise the
 * error continuation. Note that the continuation is executed
 * in a different thread than the thread that created or started
 * the REST call. However the continuation can be branched into
 * the event thread for GUI operations.
 * <p>
 * REST calls can be chained with the {@link append} method.
 * If a REST call chain is started, all REST calls of the chain
 * are executed subsequently.
 * <p>
 * If a REST call is started while another REST call is running,
 * the running REST call is interrupted and the continuation
 * cancel method is called.
 */
public class JSONRESTCall extends RESTCall
{	
	/**
	 * Creates a new JSON REST call.
	 * Notice that REST calls are executed in a different thread.
	 * Hence a continuation must be passed that executes the code when
	 * the REST call retrieves the answer.
	 * @param url       	  The URL, including all parameters.
	 * @param requestMethod   The request method (GET, HEAD, PUT, POST, DELETE)
	 * @param sendPayload     The payload on PUT and POST.
	 * @param timeout         The timeout in milliseconds.
	 * @param continuation    The continuation code when the call returns.                       
	 * @param continuationInEventThread  Whether the continuation should be
	 *                        executed in the event thread for GUI operations.
	 *                        Must be false if the continuation appends
	 *                        additional calls otherwise those calls never
	 *                        get executed.
	 * @param handleBusyIndicator Whether this call handles the busy indicator
	 *                        automatically. Should only be used when the
	 *                        continuation does not append any additional
	 *                        call.
	 */
	public JSONRESTCall(String url,
					int requestMethod,
					JSONObject sendPayload,
			        int timeout,
					JSONRESTCallContinuation continuation,
					boolean continuationInEventThread,
					boolean handleBusyIndicator)
	{
		super(url,
			  ConnectionUtil.getUsername(),
			  ConnectionUtil.getPassword(),
			  requestMethod,
			  toString(sendPayload),
			  "application/json",
		      timeout,
		      new MyRESTCallContinuation(continuation),
			  continuationInEventThread,
			  handleBusyIndicator);
	}
	
	/**
	 * Converts the JSON object to string.
	 */
	private static String toString(JSONObject obj)
	{
		if (obj != null) {
			return obj.toString();
		}
		return null;
	}
		
	/**
	 * Bridge between RESTCallContinuation and JSONRESTCallContinuation.
	 */
	static class MyRESTCallContinuation implements RESTCallContinuation
	{
		JSONRESTCallContinuation _cont;
		
		MyRESTCallContinuation(JSONRESTCallContinuation c)
		{
			_cont = c;
		}
		
		public void onSuccess(String contentType, byte[] bytes) {
			JSONObject obj = null;
			if ("application/json".equals(contentType)) {
				obj = parse(bytes, true);
			}
			_cont.onSuccess(obj, contentType, bytes);
		}

		public void onFailure(Exception ex, int status, String contentType,
				byte[] bytes) 
		{
			JSONObject obj = null;
			if ("application/json".equals(contentType)) {
				obj = parse(bytes, false);
			}
			_cont.onFailure(ex, status, obj, contentType, bytes);
		}

		public void onCancel() 
		{
			_cont.onCancel();
		}
		
		/**
		 * Parse the input bytes into a JSON object.
		 * @param bytes        The input bytes.
		 * @param errorIsFatal Whether we strictly expect a JSON object.
		 *             If yes, then JSON parsing errors are logged at
		 *             standard error.
		 * @return The corresponding JSON object.
		 */
		private JSONObject parse(byte[] bytes, boolean errorIsFatal)
		{
			if (bytes == null || bytes.length == 0)
				return null;
			
			try {
				String s = new String(bytes, "UTF-8");
				// check whether it is an array or an object
				boolean isArray = false;
				for (int i = 0; i < s.length(); i++) {
					char c = s.charAt(i);
					if (c == '[') {
						isArray = true;
						break;
					}
					if (c != ' ' && c != '\t' && c != '\n') {
						// must be object
						break;
					}
				}
				// convert to JSON an return
				if (isArray) {
					JSONArray array = new JSONArray(s);
					return wrapArray(array);
				} else {
					return new JSONObject(s);
				}
			} catch (Exception ex) {
				// hide the exception if it is not fatal.
				// Happens for failure responses where we strictly
				// do not expect JSON in the response.
				if (errorIsFatal) {
					ex.printStackTrace();
				}
			}
			return null;
		}
	}
	
	/**
	 * An empty continuation used at various places.
	 */
	public static class EmptyContinuation implements JSONRESTCallContinuation
	{

		public void onSuccess(JSONObject response, 
							  String contentType,
							  byte[] bytes) 
		{
		}

		public void onFailure(Exception ex, 
							  int status, 
							  JSONObject response,
							  String contentType, 
							  byte[] bytes)
		{
		}

		public void onCancel() 
		{
		}		
	}
	
	/**
	 * Wraps a JSON array into a JSON object.
	 * @throws JSONException 
	 */
	public static JSONObject wrapArray(JSONArray a)
	throws JSONException
	{
		return new JSONObject().put("items", a);
	}
	
	/**
	 * Unwrap a JSON array from a JSON object.
	 * This is the inverse of {@link #wrapArray}.
	 */
	public static JSONArray unwrapArray(JSONObject o)
	{
		return o.optJSONArray("items");
	}	
}
