/*
 *  Licensed Materials - Property of IBM
 *  5725-G92 (C) Copyright IBM Corp. 2011, 2013. All Rights Reserved.
 *  US Government Users Restricted Rights - Use, duplication or
 *  disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 */

package com.ibm.appcenter.connection;

import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.io.UnsupportedEncodingException;
import java.util.Enumeration;
import java.util.Timer;
import java.util.TimerTask;
import java.util.Vector;

import javax.microedition.io.Connector;
import javax.microedition.io.HttpConnection;

import com.ibm.appcenter.ui.UIUtil;
import com.ibm.appcenter.util.Log;

import net.rim.device.api.io.Base64OutputStream;
import net.rim.device.api.io.IOUtilities;
import net.rim.device.api.io.SocketConnectionEnhanced;
import net.rim.device.api.ui.component.Dialog;

/**
 * Base class for REST calls.
 * Rest calls are executed asynchronously, that is, they open
 * an own thread. When a valid REST response is received, the
 * REST call executes the success continuation, otherwise the
 * error continuation. Note that the continuation is executed
 * in a different thread than the thread that created or started
 * the REST call. However the continuation can be branched into
 * the event thread for GUI operations.
 * <p>
 * REST calls can be chained with the {@link append} method.
 * If a REST call chain is started, all REST calls of the chain
 * are executed subsequently.
 * <p>
 * If a REST call is started while another REST call is running,
 * the running REST call is interrupted and the continuation
 * cancel method is called.
 */
public class RESTCall {

	// no Java 5 in Blackberry, cannot use enums
	
	/**
	 * The REST request method GET.
	 */
	public static final int GET = 0;
	
	/**
	 * The REST request method POST.
	 */
	public static final int POST = 1;

	/**
	 * The REST request method HEAD.
	 */
	public static final int HEAD = 2;
	
	/**
	 * The REST request method PUT.
	 */
	public static final int PUT = 3;
	
	/**
	 * The REST request method DELETE.
	 */
	public static final int DELETE = 4;
	
	// increase if adding more request methods
	private static final int MAX_REQUEST_METHOD = DELETE;
	
	// -----------------------------------------------------
	
	/**
	 * If true, show dialogs when a REST call succeeded or failed.
	 */
	private static boolean DEBUG_SHOW_CONNECTION_DIALOG = false;
	
	/**
	 * If true, prints the connection calls to System.err.
	 * When debugging, it can be found in Eclipse in the Console.
	 * It can also be found in the log screen of the application
	 * itself.
	 */
	private static boolean DEBUG_LOG_CONNECTION_CALLS = true;
	
	/**
	 * If true, prints the connection response to System.err.
	 * The status code is anyway always logged, the question is
	 * merely whether the response content should be logged.
	 * By default better not, since this can be lengthy.
	 * When debugging, it can be found in Eclipse in the Console.
	 * It can also be found in the log screen of the application
	 * itself.
	 */
	private static boolean DEBUG_LOG_CONNECTION_RESPONSE = false;
	
	// -----------------------------------------------------
		
	// the URL
	private String _url;
	
	// the request method;
	private int _requestMethod;
	
	// the post data
	private String _sendPayload;
	
	// the content type of the post data
	private String _sendContentType;
	
	// timeout in milliseconds
	private int _timeout;
		
	// user name for basic authentication
	private String _username;
	
	// password for basic authentication
	private String _password;
	
	// continuation on success or error
	private RESTCallContinuation _continuation;

	// whether all continuation code should run in the even thread
	private boolean _continuationInEventThread;
	
	// additional header parameters
	private Vector _headerParams;
		
	// for chaining, to allow to handle a bunch of REST calls at once 
	private RESTCall _next;
	
	// whether the REST call itself handles the busy indicator of the batch
	private boolean _handleBusyIndicator;
	
	// -----------------------------------------------------
	
	/**
	 * Creates a new REST call.
	 * Notice that REST calls are executed in a different thread.
	 * Hence a continuation must be passed that executes the code when
	 * the REST call retrieves the answer.
	 * @param url       The URL, including all parameters.
	 * @param username  The user name, if authentication is needed.
	 * @param password  The password, if authentication is needed.
	 * @param requestMethod   The request method (GET, HEAD, PUT, POST, DELETE)
	 * @param sendPayload     The payload on PUT and POST.
	 * @param sendContentType The content type of the payload.
	 * @param timeout         The timeout in milliseconds.
	 * @param continuation    The continuation code when the call returns.
	 * @param continuationInEventThread  Whether the continuation should be
	 *                        executed in the event thread for GUI operations.
	 *                        Must be false if the continuation appends
	 *                        additional calls otherwise those calls never
	 *                        get executed.
	 * @param handleBusyIndicator Whether this call handles the busy indicator
	 *                        automatically. Should only be used when the
	 *                        continuation does not append any additional
	 *                        call.
	 */
	public RESTCall(String url,
					String username,
					String password,
					int requestMethod,
					String sendPayload,
					String sendContentType,
			        int timeout,
					RESTCallContinuation continuation,
					boolean continuationInEventThread,
					boolean handleBusyIndicator)
	{
		_url = url;
		_username = username;
		_password = password;
		_timeout = timeout;
		_continuation = continuation;
		_continuationInEventThread = continuationInEventThread;
		_requestMethod = checkRequestMethod(requestMethod);
		_sendPayload = sendPayload;
		_sendContentType = sendContentType;
		_handleBusyIndicator = handleBusyIndicator;
	}
		
	/**
	 * Adds an additional header parameter. 
	 * @param key   The parameter key.
	 * @param value The parameter value.
	 */
	public void addHeader(String key, String value)
	{
		if (_headerParams == null) {
			_headerParams = new Vector();
		}
		_headerParams.addElement(new Tuple(key, value));
	}
		
	/**
	 * Process the response as bytes.
	 * @param batch The batch this call is running inside.
	 * @param status The response status, e.g. HttpConnection.HTTP_OK.
	 * @param contentType The content type. Can be null.
	 * @param bytes The bytes of the response. Can be null.
	 */
	protected void processResponse(RESTCallBatch batch, int status, String contentType, byte[] bytes)
	{
		if (DEBUG_LOG_CONNECTION_CALLS) {
			Log.log("processResponse " + status);
		}
		if (status == HttpConnection.HTTP_CLIENT_TIMEOUT) {
			onTimeout(batch);
		} else if (isStatusSuccess(status)) {
			if (bytes == null) {
				bytes = new byte[] {};
			}
			if (contentType == null) {
				contentType = "";
			}
			onSuccess(batch, contentType, bytes);
		} else {
			onError(batch, null, status, contentType, bytes);
		}
	}
	
	/**
	 * Returns true if the response status is considered a success.
	 * Overwritten when registering device, since in that case
	 * conflict is also success.
	 */
	protected boolean isStatusSuccess(int status)
	{
		return status == HttpConnection.HTTP_OK;
	}
	
	/**
	 * Called on a successful response as bytes.
	 * @param batch The batch inside which this call is executed.
	 * @param contentType The content type.
	 * @param bytes The bytes of the response. Never null.
	 */
	protected void onSuccess(final RESTCallBatch batch,
							 final String contentType,
							 final byte[] bytes)
	{
		if (DEBUG_LOG_CONNECTION_CALLS) {
			String content = "";
			String contentTp = contentType != null ? contentType : "";
			try {
				if (bytes != null)
					content = new String(bytes, "UTF-8");
			} catch (UnsupportedEncodingException e) {
			}
		
			if (DEBUG_LOG_CONNECTION_RESPONSE) {
				Log.log("SUCCESS " + contentTp + ": " + content);
			} else {
				Log.log("SUCCESS " + contentTp);
			}
		}
		if (_continuationInEventThread) {
			final RESTCallContinuation c = _continuation;
			UIUtil.invokeInEventThread(new Runnable() {  
                public void run() {
                	if (DEBUG_SHOW_CONNECTION_DIALOG) {
                		Dialog.alert("Success");  // debugging only, no i18n
                	}
                	// set indicator non-busy after c.onSuccess because in
                	// success case, we often have lots of work, e.g.
                	// JSON parsing to do.
                	c.onSuccess(contentType, bytes);
                	clearBusyIndicator(batch);
                }
			});
		} else {
           	// set indicator non-busy after c.onSuccess because in
        	// success case, we often have lots of work, e.g.
        	// JSON parsing to do.
			_continuation.onSuccess(contentType, bytes);
			clearBusyIndicator(batch);
		}
	}
		
	/**
	 * Called on a general error.
	 * @param batch The batch inside which this call is executed.
	 * @param ex The exception, if an exception caused this error.
	 * @param status The HTTP status if no exception caused this error.
	 * @param contentType The content type. Can be null.
	 * @param bytes The bytes of the response. Can be null.
	 */
	protected void onError(final RESTCallBatch batch,
						   final Exception ex, 
						   final int status,
						   final String contentType,
						   final byte[] bytes)
	{
		if (DEBUG_LOG_CONNECTION_CALLS) {
			if (ex != null) {
				Log.log(ex);
			}
			String content = "";
			String contentTp = contentType != null ? contentType : "";
			try {
				if (bytes != null)
					content = new String(bytes, "UTF-8");
			} catch (UnsupportedEncodingException e) {
				Log.log(e);
			}
		
			if (DEBUG_LOG_CONNECTION_RESPONSE) {
				Log.log("FAIL with status=" + status + " " + contentTp + ": " + content);
			} else {
				Log.log("FAIL with status=" + status + " " + contentTp);
			}
		}
		if (_continuationInEventThread) {
			final RESTCallContinuation c = _continuation;
			UIUtil.invokeInEventThread(new Runnable() {  
                public void run() {
                	if (DEBUG_SHOW_CONNECTION_DIALOG) {
                		String suffix = "";
                		if (ex != null)
                			suffix = ex.getMessage();
                		Dialog.alert("Fail " + status + " " + suffix);  // debugging only, no i18n
                	}
                	// set indicator non-busy early, so that we can pop up
                	// error dialogs
                	clearBusyIndicator(batch);
                	c.onFailure(ex, status, contentType, bytes);
                }
			});
		} else {
			// set indicator non-busy early, so that we can pop up
        	// error dialogs
			clearBusyIndicator(batch);
			_continuation.onFailure(ex, status, contentType, bytes);
		}
	}
		
	/**
	 * Called on timeout.
	 * @param batch The batch inside which this call is executed.
	 */
	protected void onTimeout(final RESTCallBatch batch)
	{
		if (DEBUG_LOG_CONNECTION_CALLS) {		
			Log.log("FAIL with status=timeout");
		}
		if (_continuationInEventThread) {
			final RESTCallContinuation c = _continuation;
			UIUtil.invokeInEventThread(new Runnable() {  
                public void run() {
                	if (DEBUG_SHOW_CONNECTION_DIALOG) {
                		Dialog.alert("Timeout");  // debugging only, no i18n
                	}
                	clearBusyIndicator(batch);
                	c.onFailure(null, HttpConnection.HTTP_CLIENT_TIMEOUT, "", new byte[] {});
                }
			});
		} else {
			clearBusyIndicator(batch);
			_continuation.onFailure(null, HttpConnection.HTTP_CLIENT_TIMEOUT, "", new byte[] {});
		}
	}
	
	/**
	 * Called on cancel during running.
	 * @param batch The batch inside which this call is executed.
	 */
	protected void onCancel(final RESTCallBatch batch)
	{
		if (DEBUG_LOG_CONNECTION_CALLS) {		
			Log.log("CANCELLED REST CALL " + _url);
		}
		if (_continuationInEventThread) {
			final RESTCallContinuation c = _continuation;
			UIUtil.invokeInEventThread(new Runnable() {  
                public void run() {
                	clearBusyIndicator(batch);
                	c.onCancel();
                }
			});
		} else {
			clearBusyIndicator(batch);
			_continuation.onCancel();
		}
	}
	
	private void clearBusyIndicator(RESTCallBatch batch)
	{
		if (_handleBusyIndicator) {
			batch.clearBusyIndicator();
		}
	}
	
	/**
	 * Checks the range of the request method.
	 * @param method The request method.
	 * @return A valid method.
	 */
	private int checkRequestMethod(int method)
	{
		if (method < 0 || method > MAX_REQUEST_METHOD)
			throw new IllegalArgumentException("Illegal request method " + method);
		return method;
	}
	
	/**
	 * Returns the current request method as string.
	 */
	private String getRequestMethodAsString()
	{
		switch (_requestMethod) {
		
		case GET:    return HttpConnection.GET;
		case POST:   return HttpConnection.POST;
		case HEAD:   return HttpConnection.HEAD;
		case PUT:    return "PUT";
		case DELETE: return "DELETE";
		
		}
		
		return HttpConnection.GET;
	}
	
	/**
	 * Appends a REST call to a chain of calls.
	 */
	public void append(RESTCall nextCall)
	{
		RESTCall current = this;
		while (current.getNext() != null) {
			current = current.getNext();;
		}
		current._next = nextCall;
	}
	
	/**
	 * Returns the next REST call in a chain of calls.
	 */
	public RESTCall getNext()
	{
		return _next;
	}
	
	// -----------------------------------------------------
	
	/**
	 * Thread that executes the HTTP Request.
	 * Belongs logically more to the batch than to the call,
	 * since we have one thread per batch, but can handle
	 * multiple calls in one batch.
	 */
	static class ExecutionThread extends Thread
	{
		private RESTCall _call;
		private boolean _running = false;
		private HttpConnection _connection = null;
		private Timer _timeoutTimer = null;
		private HttpTimerTask _currentTimerTask = null;
		private boolean _interrupted;
		private RESTCallBatch _batch;
		
		/**
		 * Constructor.
		 */
		ExecutionThread(RESTCallBatch batch)
		{
			_batch = batch;
		}
		
		/**
		 * Runs the thread.
		 */
		public void run()
		{
			_call = _batch.popCurrent();
	        _timeoutTimer = new Timer();
			try {
				// We handle not just one REST call, but an entire chain of REST calls
				// here.
				while (_call != null) {
					RESTCall call = getCall();
					if (call == null) break;
					if (DEBUG_LOG_CONNECTION_CALLS) {		
						Log.log("CONNECT " + call._username + " " + Log.hidePassword(call._password) + " " + call.getRequestMethodAsString() + " " + call._url);
					}
					try {
						_running = true;
						_interrupted = false;
						// start a timer that kills the connection after approximately the timeout time
						_currentTimerTask = new HttpTimerTask();
						_timeoutTimer.schedule(_currentTimerTask, call._timeout + 2000);
						// open the connection
						_connection = (HttpConnection)Connector.open(call._url);
						_connection.setRequestMethod(call.getRequestMethodAsString());
						handleHeader();
						handleTimeout();
						handlePayload();
								
						// this will block and wait for the response
						if (!_interrupted) {
							int status = _connection.getResponseCode();
							// Log.log("Response code: " + status);
							String contentType = _connection.getHeaderField("content-type");
							byte[] bytes = handleResponse();
				
							synchronized (_batch) {
								if (cleanup(true)) {
									call.processResponse(_batch, status, contentType, bytes);
								}
							}
						}
					} catch (Exception ex) {
						synchronized (_batch) {
							if (cleanup(true)) {
								call.onError(_batch, ex, 0, null, null);
							}
						}
					}
					call = getCall();
					if (call == null) break;
					_call = call.getNext();
				}
			} finally {
				_timeoutTimer.cancel();
				_timeoutTimer = null;
				_call = null;
				_batch.clear();
			}
		}
		
		/**
		 * Handles the header of the connection.
		 * @throws Exception
		 */
		private void handleHeader() throws IOException
		{
			if (_interrupted) return;
			RESTCall call = getCall();
			if (call == null) return;
			
			if (call._username != null) {
				String upw = call._username + ":" + (call._password != null ? call._password : "");
				byte[] cred = Base64OutputStream.encode((upw).getBytes("UTF-8"), 0, upw.length(), false, false);
				// _connection.setRequestProperty("Authorization", "Basic " + new String(cred).toString());
				call.addHeader("Authorization", "Basic " + new String(cred).toString());
			}
			
			if (call._headerParams != null) {
				Enumeration e = call._headerParams.elements();
				while (e.hasMoreElements()) {
					Tuple t = (Tuple)e.nextElement();
					_connection.setRequestProperty(t.key, t.value);
				}
			}
		}
		
		/**
		 * Handles the timeout of the connection.
		 */
		private void handleTimeout() throws IOException
		{
			if (_interrupted) return;
			RESTCall call = getCall();
			if (call == null) return;
			if (_connection instanceof SocketConnectionEnhanced) {
				SocketConnectionEnhanced sce = (SocketConnectionEnhanced) _connection;
				short timeoutKey = SocketConnectionEnhanced.READ_TIMEOUT;
				sce.setSocketOptionEx(timeoutKey, call._timeout);
			}
		}
		
		/**
		 * Handles the payload on PUT and POST.
		 */
		private void handlePayload() throws IOException
		{
			if (_interrupted) return;
			RESTCall call = getCall();
			if (call == null) return;
			if (call._sendPayload != null && call._sendPayload.length() > 0) {
				String contentType = call._sendContentType != null ? call._sendContentType : "text/plain";
				byte[] bytes = call._sendPayload.getBytes("UTF-8");			
				_connection.setRequestProperty("Content-Type", contentType);
				_connection.setRequestProperty("Content-Length", String.valueOf(bytes.length));
				OutputStream os = _connection.openOutputStream();
				os.write(bytes);
				os.flush();
				os.close();
			}	
		}
		
		/**
		 * Handles the response if any.
		 * @return The bytes of the response.
		 */
		private byte[] handleResponse()
		{
			if (_interrupted) return new byte[] {};
			byte[] bytes = null;

			try {
				InputStream input = _connection.openInputStream();
				try {
					bytes = IOUtilities.streamToBytes(input);
				} finally {
					if (input != null) input.close();
				}
			} catch (IOException ex) {
			}
            return bytes;
		}
		
		/**
		 * Stop the thread.
		 * Called from the main thread.
		 */
		public void stop()
		{
			synchronized (_batch) {
				Log.log("ExecutionThread stop " + _call + " running=" + _running);
				if (_call == null) return;
				if (cleanup(true)) {
					_call.onCancel(_batch);
				}
				_call = null;
			}
		}
		
		/**
		 * Timeout the thread.
		 * Called from the HttpTimerThread.
		 */
		public void timeout()
		{
			synchronized (_batch) {
				// Do not cancel the timeout timer
				// since this is called from inside the
				// timeout timer.
				if (cleanup(false)) {
					Log.log("Timeout from TimeoutTimerTask");
					RESTCall call = getCall();
					if (call != null) {
						call.onTimeout(_batch);
					}
				}
			}
		}
		
		/**
		 * Stop running and cleanup.
		 * Stops the timeout timer task if necessary.
		 * Interrupts the execution thread if called from extern thread.
		 * Closes the connection.
		 * @param stopTimerTask Whether the timeout timer task needs stopped.
		 * @return Whether it was previously running.
		 */
		private boolean cleanup(boolean stopTimerTask)
		{
			synchronized (_batch) {
				Log.log("ExecutionThread cleanup " + stopTimerTask + " running=" + _running);
				// if necessary, stop the timeout timer task
				if (stopTimerTask && _currentTimerTask != null) {
					_currentTimerTask.cancel();
					_currentTimerTask = null;
				}
				// interrupt the HTTP execution thread when called from
				// extern
				if (Thread.currentThread() != this) {
					_interrupted = true;
					// In a chain, we cannot do this. But closeing
					// the connection is already enough to stop
					// the HTTP call.
					// interrupt();
				}
				// mark as non-running
				boolean wasRunning = _running;
				_running = false;
				// close the connection
				if (_connection != null) {
					HttpConnection c = _connection;
					_connection = null;
					try {
						c.close();
					} catch (Exception ex) {
					}
				}
				// return whether it was running
				return wasRunning;
			}
		}
		
		private RESTCall getCall()
		{
			synchronized (_batch) {
				return _call;
			}
		}
		
		public void appendAtRunning(RESTCall call)
		{
			synchronized (_batch) {
				if (_call == null) {
					_call = call;
				} else {
					_call.append(call);
				}
			}
		}
	}
	
	// -----------------------------------------------------
	
	/**
	 * Task that helps to stop the request on timeout.
	 * Blackberry's timeout mechanism is not really reliable
	 * and this task improves the situation somewhat.
	 */
	static class HttpTimerTask extends TimerTask {
	    ExecutionThread _ownerThread = null;
	    
	    /**
	     * Constructor.
	     * @param connection The associated connection.
	     */
	    public HttpTimerTask()
	    {
	        _ownerThread = (ExecutionThread)Thread.currentThread();
	    }
	    	    
	    /**
	     * Runs the task.
	     */
	    public void run() {
	    	if (_ownerThread != null) {
	    		_ownerThread.timeout();
	    	}
	    }
	}
	
	// -----------------------------------------------------
	
	/**
	 * Simple class for key value entries.
	 */
	static class Tuple {
		String key;
		String value;
		
		Tuple(String k, String v)
		{
			key = k;
			value = v;
		}
	}
}
