/*
 *  Licensed Materials - Property of IBM
 *  5725-G92 (C) Copyright IBM Corp. 2011, 2013. All Rights Reserved.
 *  US Government Users Restricted Rights - Use, duplication or
 *  disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 */

package com.ibm.appcenter.connection;

import org.json.me.JSONObject;

import com.ibm.appcenter.connection.RESTCall.ExecutionThread;
import com.ibm.appcenter.i18n.I18N;
import com.ibm.appcenter.i18n.I18NResource;
import com.ibm.appcenter.ui.UIUtil;
import com.ibm.appcenter.util.BusyIndicator;
import com.ibm.appcenter.util.Log;

/**
 * A sequence of REST calls.
 * REST calls can only be executed in a REST Call batch.
 * First you add REST calls to the batch. They are executed
 * sequentially. Then you start them. They are executed in one
 * separate thread. If the batch is cancelled, all calls
 * currently in the batch are canceled.
 * Each REST call that got executed is removed from the batch.
 */
public class RESTCallBatch 
{
	// The call chain that is executed.
	private RESTCall _call = null;
	
	// the busy indicator, can be null
	private BusyIndicator _indicator;
	
	// the execution thread
	private ExecutionThread _thread;
	
	// whether we can stop the thread
	private boolean _canStopThread = true;
	
	/**
	 * Constructor.
	 * @param indicator The indicator that displays that the app is busy.
	 *                  Can be null.
	 */
	public RESTCallBatch(BusyIndicator indicator)
	{
		_indicator = indicator;
		_call = null;
	}
	
	/**
	 * Starts the batch of REST calls.
	 */
	public synchronized void start()
	{
		if (_call == null) {
			return;
		}
		if (!ConnectionUtil.hasCoverage()) {
			Log.log("No coverage for HTTP");
			UIUtil.showAlertDialog(I18N.getString(I18NResource.ConnectionUtilNoCoverageMsg));
			return;
		}
		// stop running calls
		stop();
		// indicate that we are busy
		if (_indicator != null) {
			_indicator.setBusy(true);
		}
		// and start a new thread
		_canStopThread = true;
		_thread = new ExecutionThread(this);
		_thread.start();
	}
	
	/**
	 * Stops batch of the REST calls.
	 */
	public synchronized void stop()
	{
		Log.log("RESTCallBatch stop, canStop=" + canStop() + ", running" + isRunning());
		if (!canStop()) {
			return;
		}
		clearBusyIndicator();
		if (isRunning()) {
			// ensure that if t.stop calls this,
			// we don't have an endless loop
			ExecutionThread t = _thread;
			_thread = null;
			t.stop();
			// the above ensures also that
			// _currentCall.onCancel is called
			// when needed.
		}
	}
	
	/**
	 * Returns whether we can stop the execution thread.
	 */
	public synchronized boolean canStop()
	{
		return _canStopThread;
	}
	
	/**
	 * Sets whether we can stop the execution thread.
	 */
	public synchronized void setCanStop(boolean can)
	{
		_canStopThread = can;
	}
	
	/**
	 * Called before the batch of REST calls end.
	 */
	protected synchronized void clearBusyIndicator()
	{
		if (_indicator != null) {
			_indicator.setBusy(false);
		}
	}
	
	/**
	 * Returns the current REST Call. Returns null if no REST call is active.
	 */
	public synchronized RESTCall getCurrent()
	{
		return _call;
	}
		
	/**
	 * Prepends a REST call but does not execute it yet.
	 * This is fast.
	 */
	public synchronized void prepend(RESTCall call)
	{
		call.append(_call);
		_call = call;
	}
	
	/**
	 * Appends a REST call but does not execute it yet.
	 */
	public synchronized void append(RESTCall call)
	{
		if (_call == null) {
			_call = call;
		} else {
			_call.append(call);
		}
	}
	
	/**
	 * Appends a REST call to the currently excecuted calls.
	 */
	public synchronized void appendAtRunning(RESTCall call)
	{
		if (_thread == null) return;
		_thread.appendAtRunning(call);
	}
	
	/**
	 * Clean up after running the calls.
	 * This is called exactly when the execution thread exits.
	 */
	public synchronized void clear()
	{
		clearBusyIndicator();
		_call = null;
		_thread = null;
	}
	
	/**
	 * Returns the current call chain, and removes it from the
	 * batch. This is called when we start to execute that call chain.
	 * @return
	 */
	synchronized RESTCall popCurrent()
	{
		RESTCall call = _call;
		_call = null;
		return call;
	}
	
	/**
	 * This is mainly for debugging. It ensures that the batch
	 * length is as expected and pops a dialog otherwise.
	 * @param n The expected number of calls in the batch.
	 */
	public synchronized void ensureBatchLength(int n)
	{
		int count = 0;
		RESTCall current = getCurrent();
		while (current != null) {
			count++;
			current = current.getNext();
		}
		if (count != n) {
			// this is only a debugging message, should never happen ... no I18N
			UIUtil.showAlertDialog("DEBUG: Batch size wrong; expected: " + n + ", detected: " + count);
		}
	}
	
	/**
	 * Returns whether the batch is currently executed.
	 */
	public boolean isRunning()
	{
		return _thread != null;
	}
	
	/**
	 * Returns whether the batch contains anything.
	 */
	public boolean isEmpty()
	{
		return _call == null;
	}
	
	// ------------------------------------------------------------------
	// Utilities to invoke continuations in event thread.

	/**
	 * Call {@link JSONRESTCallContinuation.onSuccess} in the event thread.
	 *
	public void callOnSuccessInEventThread(final RESTCallContinuation cont,
										   final String contentType,
										   final byte[] bytes)
	{
		UIUtil.invokeInEventThread(new Runnable() {
			public void run() {
				cont.onSuccess(contentType, bytes);
			}});
	}
	
	/**
	 * Call {@link JSONRESTCallContinuation.onFailure} in the event thread.
	 *
	public void callOnFailureInEventThread(final RESTCallContinuation cont,
										   final Exception ex,
										   final int status,
										   final String contentType,
										   final byte[] bytes)
	{
		UIUtil.invokeInEventThread(new Runnable() {
			public void run() {
				cont.onFailure(ex, status, contentType, bytes);
			}});
	}
	
	/**
	 * Call {@link JSONRESTCallContinuation.onCancel} in the event thread.
	 *
	public void callOnCancelInEventThread(final RESTCallContinuation cont)
	{
		UIUtil.invokeInEventThread(new Runnable() {
			public void run() {
				cont.onCancel();
			}});
	}
	*/
	
	/**
	 * Call {@link JSONRESTCallContinuation.onSuccess} in the event thread.
	 */
	public void callOnSuccessInEventThread(final JSONRESTCallContinuation cont,
										   final JSONObject response,
										   final String contentType,
										   final byte[] bytes)
	{
		UIUtil.invokeInEventThread(new Runnable() {
			public void run() {
				cont.onSuccess(response, contentType, bytes);
			}});
	}
	
	/**
	 * Call {@link JSONRESTCallContinuation.onFailure} in the event thread.
	 */
	public void callOnFailureInEventThread(final JSONRESTCallContinuation cont,
										   final Exception ex,
										   final int status,
										   final JSONObject response,
										   final String contentType,
										   final byte[] bytes)
	{
		UIUtil.invokeInEventThread(new Runnable() {
			public void run() {
				cont.onFailure(ex, status, response, contentType, bytes);
			}});
	}
	
	/**
	 * Call {@link JSONRESTCallContinuation.onCancel} in the event thread.
	 */
	public void callOnCancelInEventThread(final JSONRESTCallContinuation cont)
	{
		UIUtil.invokeInEventThread(new Runnable() {
			public void run() {
				cont.onCancel();
			}});
	}
}
