/*
 *  Licensed Materials - Property of IBM
 *  5725-G92 (C) Copyright IBM Corp. 2011, 2013. All Rights Reserved.
 *  US Government Users Restricted Rights - Use, duplication or
 *  disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 */

package com.ibm.appcenter.system;

import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Vector;

import net.rim.device.api.servicebook.ServiceBook;
import net.rim.device.api.servicebook.ServiceRecord;
import net.rim.device.api.system.CodeModuleGroup;
import net.rim.device.api.system.CodeModuleGroupManager;
import net.rim.device.api.system.CodeModuleManager;
import net.rim.device.api.system.DeviceInfo;
import net.rim.device.api.system.RadioInfo;
import net.rim.device.api.system.WLANInfo;

import com.ibm.appcenter.artifacts.AppDetail;
import com.ibm.appcenter.artifacts.AppVersion;
import com.ibm.appcenter.i18n.I18N;
import com.ibm.appcenter.i18n.I18NResource;
import com.ibm.appcenter.util.BasicUtil;
import com.ibm.appcenter.util.Log;
import com.ibm.appcenter.util.ProgressIndicator;

/**
 * Utilities to access the underlying BlackBerry system
 * (net.rim.device.api).
 * @author Sander
 *
 */
public class BlackBerrySystem {
	
	// no java 5 => no enums. Need to simulate
	
	/**
	 * Return code to indicate that an application is
	 * installed.
	 */
	public static final int APPLICATION_INSTALLED = 0;
	
	/**
	 * Return code to indicate that an application is installed
	 * but it is not the expected newest version.
	 */
	public static final int APPLICATION_NEEDS_UPGRADE = 1;
	
	/**
	 * Return code to indicate that an application is installed
	 * but it is not the expected older version.
	 */
	public static final int APPLICATION_NEEDS_DOWNGRADE = 2;
	
	/**
	 * Return code to indicate that an application is not installed.
	 */
	public static final int APPLICATION_NOT_INSTALLED = 3;
	
	/**
	 * Return the BIS-B Connection suffix if you want to distribute
	 * this application as BlackBerry Alliance member. In that case,
	 * BlackBerry assigned a BIS-B secret to you. By default, 
	 * connection via BIS-B is disabled since IBM is not allowed to
	 * make its BIS-B secret public. 
	 */
	public static final String BIS_B_SECRET = null;
	
	/**
	 * Flag that tells the unexposed application list whether it needs
	 * refresh on expose, i.e. if some apps have changed.
	 */
	public static boolean applicationsChanged = false;
	
	/**
	 * Device info.
	 */
	private static String _deviceId;
	private static int _deviceIdNumber;
	private static String _deviceMaker;
	private static String _deviceModel;
	private static String _deviceOs;
	private static String _deviceOsDescr;
	
	// -------------------------------------------------------------------
	
	/**
	 * Private constructor since this is a singleton.
	 */
	private BlackBerrySystem() {}
	
	// -------------------------------------------------------------------
	
	/**
	 * Tries to detect the appropriate connection suffix automatically.
	 * The return value omits the leading semicolon.
	 */
	public static String getAutoConnectionSuffix()
	{
		String connSuffix = "";
		if (DeviceInfo.isSimulator()) {
			connSuffix = "deviceside=true";
		}
		else if (BIS_B_SECRET != null) {
			// use BIS-B if a secret is known
			connSuffix = BIS_B_SECRET;
		} else if ((WLANInfo.getWLANState() == WLANInfo.WLAN_STATE_CONNECTED) &&
					RadioInfo.areWAFsSupported(RadioInfo.WAF_WLAN)) {
			// use Wifi whenever possible
			connSuffix="interface=wifi";
		} else {
			// search user id
			String uid = null;
			ServiceBook sb = ServiceBook.getSB();
			ServiceRecord[] records = sb.findRecordsByCid("WPTCP");
			for (int i = 0; i < records.length; i++) {
				if (records[i].isValid() && !records[i].isDisabled()) {
					if (records[i].getUid() != null && records[i].getUid().length() != 0) {
						if ((records[i].getCid().toLowerCase().indexOf("wptcp") != -1) &&
								(records[i].getUid().toLowerCase().indexOf("wifi") == -1) &&
								(records[i].getUid().toLowerCase().indexOf("mms") == -1)   ) {
							uid = records[i].getUid();
							break;
						}
					}
				}
			}
			if (uid != null) {
				// WAP2 Connection
				connSuffix = "deviceside=true;ConnectionUID="+uid;
			} else {
				// fallback direct TCP (assuming it can find 
				// an APN in the service book)
				connSuffix = "deviceside=true";
			}
		}
		
		return connSuffix;
	}
	
	// -------------------------------------------------------------------
	// Device API
	
	/**
	 * Returns the device id as number.
	 *
	public static int getDeviceIdNumber()
	{
		validateDevice();
		return _deviceIdNumber;
	}
	 */
	
	/**
	 * Returns the device id as string.
	 */
	public static String getDeviceId()
	{
		validateDevice();
		return _deviceId;
	}
	
	/**
	 * Returns the device maker.
	 */
	public static String getDeviceMaker()
	{
		validateDevice();
		return _deviceMaker == null ? "BB" : _deviceMaker;
	}
	
	/**
	 * Returns the device model.
	 */
	public static String getDeviceModel()
	{
		validateDevice();
		return _deviceModel == null ? "Simulator" : _deviceModel;
	}
	
	/**
	 * Returns the detailed device OS.
	 * This is not the OS how the device gets registered in the data base,
	 * because on the server, we use simply "Blackberry" as OS.
	 *
	public static String getDeviceOs()
	{
		validateDevice();
		return _deviceOs == null ? "Unknown" : _deviceOs;
	}
	 */
	
	/**
	 * Returns the device OS description.
	 */
	public static String getDeviceOsDescription()
	{
		validateDevice();
		return _deviceOsDescr == null ? "Unknown" : _deviceOsDescr;
	}
		
	/**
	 * Ensure that the device id is valid.
	 */
	private static void validateDevice()
	{
		if (_deviceId != null) {
			return;
		}
		
		_deviceIdNumber = DeviceInfo.getDeviceId();
		_deviceMaker = DeviceInfo.getManufacturerName();
		_deviceModel = DeviceInfo.getDeviceName();
		_deviceOs = DeviceInfo.getSoftwareVersion();
		_deviceOsDescr = I18N.getString(I18NResource.DeviceOSDescription, DeviceInfo.getPlatformVersion(), _deviceOs);
		_deviceId = "BB-" + _deviceIdNumber + "-" + convertForDevId(_deviceModel);
	}
	
	/**
	 * Convert a string so that it can be contained in a device id.
	 * Device ids allow only A-Z, 0-9, dash.
	 */
	private static String convertForDevId(String s)
	{
		StringBuffer buf = new StringBuffer();
		for (int i = 0; i < s.length(); i++) {
			char c = s.charAt(i);
			if (c >= 'A' && c <= 'Z') {
				buf.append(c);
			}
			else if (c >= 'a' && c <= 'z') {
				buf.append(c);
			}
			else if (c >= '0' && c <= '9') {
				buf.append(c);
			}
			else if (c == '-') {
				buf.append(c);
			}
		}
		return buf.toString();
	}
	
	// -------------------------------------------------------------------
	// Application API
	
	/**
	 * Installs an application.
	 * @param indicator The progress indicator.
	 * @param groupName The group name of the application.
	 * @param groupFriendlyName The readable group label of the application.
	 * @param vendor The vendor of the application.
	 * @param version The version of the application.
	 * @param descr The description of the application.
	 * @param data The list of data from cod modules.
	 * @return true on success.
	 */
	public static boolean installApplication(
			ProgressIndicator indicator, 
			String groupName, 
			String groupFriendlyName, 
			String vendor, 
			String version, 
			String descr, 
			byte[][] data)
	{
		log("installApplication " + groupName + " " + groupFriendlyName + " " + vendor + " " + version);
		int n = data.length;
		if (n <= 0) {
			return false;
		}
		int[] moduleHandles = new int[n];
		boolean rc = true;
				
		if (isModuleGroupInstalled(groupName)) {
			deleteApplication(null, true, groupName);
		}
		
		if (true) {
			try {
				// need to work with code groups
				CodeModuleGroup cmGroup = new CodeModuleGroup(groupName);
				log("New group handle " + cmGroup.getHandle());
				if (groupFriendlyName != null)
					cmGroup.setFriendlyName(groupFriendlyName);
				if (vendor != null)
					cmGroup.setVendor(vendor);
				if (version != null)
					cmGroup.setVersion(version);
				if (descr != null)
					cmGroup.setDescription(descr);
				for (int i = 0; i < n; i++) {
					int handle = moduleHandles[i] = installCodModule(data[i]);
					if (handle != 0) {
						cmGroup.addModule(CodeModuleManager.getModuleName(handle));
						if (indicator != null) {
							indicator.increaseProgressValueTo(n + i + 1);
						}
					} else {
						log("No handle for module " + i);
						throw new Exception("No handle for module " + i);
					}
				}
			
				if (!cmGroup.store()) {
					log("Failed to store module group " + groupName);
					throw new Exception("Failed to store module group " + groupName);
				}
			} catch (Throwable th1) {
				log("Exception during install of " + groupName + " " + th1.toString());
				
				// Delete in reverse order, this is faster
				for (int i = n - 1; n >= 0; n--) {
					// A non 0 value indicates the module was created.
					try {
						if (moduleHandles[i] != 0) {
							CodeModuleManager.deleteModuleEx(moduleHandles[i], true);
						}
					} catch (Throwable th2) {
						log("Cleanup after failed install caused throwable", th2);
					}
				}
				log("The application failed to install.");
				rc = false;				
			}
		}
		if (rc) {
			applicationsChanged = true;
		}
		// done by caller: 
		// if (CodeModuleManager.isResetRequired()) {
		//     CodeModuleManager.promptForResetIfRequired();
		// }
		return rc;
	}

	/**
	 * Install a single cod module.
	 * @param data The data of the module.
	 * @return The new module handle.
	 */
	private static int installCodModule(byte[] data)
	{
    	log("installCodModule " + (data == null ? "no data" : ""+ data.length + " bytes"));
    	
    	int moduleHandle = 0;
    	int transactionHandle = CodeModuleManager.beginTransaction();
    	if (transactionHandle != 0) {

        	moduleHandle = CodeModuleManager.createNewModule(data.length, data, data.length);

    		if (moduleHandle == 0) {
    			// couldn't get a module handle! Waaaaaa!
    			log("Could not get a module handle");
    		} else {
    			int rc1 = CodeModuleManager.saveNewModule(moduleHandle, true, transactionHandle);
    			switch (rc1) {
    				case CodeModuleManager.CMM_OK:
    					// fall through
    				case CodeModuleManager.CMM_OK_MODULE_OVERWRITTEN:
    					int rc2 = CodeModuleManager.endTransaction(transactionHandle);
    					switch (rc2) {
    						case CodeModuleManager.CMM_TRANSACTION_COMPLETE:
    							// addition finished successfully
    							log("Added module sucessfully");
    							break;
    						case CodeModuleManager.CMM_TRANSACTION_RESET_REQUIRED:
    							// device needs to reset to finish the transaction
    							// ... do the reset work here ...
    							log("Reset required");
    							break;
    						default:
    							// transaction failed for some reason; since transactions are all
    							// or nothing, it will cancel itself and clean up the module handle.
    							// See below for the rcs.
    							log("Transaction failed " + rc2);
    							break;
    					}
    					break;
    				default:
    					// couldn't save the module for some reason, and so the CMM hasn't built an
    					// an association between the transaction handle and the module handle:
    					// we must clean up the module handle ourselves.
    					CodeModuleManager.deleteNewModule(moduleHandle);
    					log("Transaction failed");
    					moduleHandle = 0;
    					break;
    			}
    		}
    	} else {
    		log("Could not get a transaction handle");
    	}
		return moduleHandle;
	}
	
	/**
	 * Deletes an application.
	 * @param indicator The progress indicator.
	 * @param fast Whether to use the fast or the slow variant.
	 * @param groupName The application id.
	 * @return true on success.
	 */
	public static boolean deleteApplication(ProgressIndicator progress, boolean fast, String groupName)
	{
		// TODO: How do I delete the app if it has no group name?
		// According to the docu:
		//   A group is NOT created if you use javaloader to load the application by specifying the COD file names.
		//   A group is NOT created when a COD file is copied into the BlackBerry Simulator directory (this is what occurs when you test/debug your application).
		//   A group IS created if the application is installed using any other means listed here.
		// Right now we consider apps without group name as nondeletable.
		
		log("deleteApplication " + groupName);
		boolean rc = true;
		try {
			// CodeModuleGroup[] allGroups = CodeModuleGroupManager.loadAll();
			// for (int i = 0; i < allGroups.length; i++) {
			// 	log(allGroups[i].getName() + " " + allGroups[i].getFriendlyName());
			// }
				
			CodeModuleGroup cmGroup = CodeModuleGroupManager.load(groupName);
			if (cmGroup == null) {
				log("Did not find a module group for " + groupName + ", stop delete.");
				return false;
			}
			// first count
			int n = 0;
			Enumeration modules = cmGroup.getModules();
			while (modules.hasMoreElements()) {  
				modules.nextElement();
			    n++;    
			} 
			// then delete the modules
			if (n > 0) {
				int[] retCodes = new int[n];
				int[] handles = new int[n];
				n = 0;
				modules = cmGroup.getModules();
				while (modules.hasMoreElements()) {  
					String moduleName = (String) modules.nextElement();  
					int handle = CodeModuleManager.getModuleHandle(moduleName);
					if (handle != 0) {
						handles[n++] = handle;
						if (!fast) {
							try {
								log("Try delete " + handle);
								CodeModuleManager.deleteModuleEx(handle, true);
							} catch (Throwable th) {
								log("Delete caused throwable", th);
							}
							if (progress != null) {
								progress.increaseProgressValueTo(n);
							}
						}	
					}
				}
				if (fast) {
					int[] tmphandles = new int[n];
					System.arraycopy(handles, 0, tmphandles, 0, n);
					try {
						retCodes = CodeModuleManager.deleteModules(tmphandles, true);
					} catch (Throwable th) {
						log("Delete caused throwable", th);
					}
					if (progress != null) {
						progress.increaseProgressValueTo(n);
					}
				}
				if (retCodes != null) {
					// I don't know exactly why, but it seems for me that with bigger
					// multipart apps, some deletes always fail. Maybe it is because
					// depending modules have already deletes that module. Therefore
					// we return only false when all return codes are wrong.
					boolean allWrong = true;
					n = retCodes.length;
					for (int i = 0; i < n; i++) {
						if (!deleteReturnCodeOK(retCodes[i])) {
							// see below for the rc's (CMM_ list)
							log("Failed to delete module " + i + ", rc=" + retCodes[i]);
							if (i < handles.length) {
								log("The handle was " + handles[i] + ". This is not yet fatal.");
							} 
						} else {
							allWrong = false;
						}
						if (allWrong) {
							log("Fatal: All modules could not be deleted!");
							rc = false;
						}
					}
				}
			}
			// finally delete the group
			cmGroup.delete();  
		} catch (Exception ex) {
			log("Exception when trying to delete application " + groupName, ex);
		}
		if (progress != null) {
			progress.increaseProgressValueTo(Integer.MAX_VALUE);
		}
		
		// done by caller:
	    // if (CodeModuleManager.isResetRequired()) {  
		//       CodeModuleManager.promptForResetIfRequired(); 
	    // }
	    	
		if (rc) {
			applicationsChanged = true;
		}
		
		return rc;
	}
	
	/**
	 * Returns the number of code modules in a group.
	 * @param groupName The group name.
	 */
	public static int getNumberOfModulesOfGroup(String groupName)
	{
		log("getNumberOfModulesOfGroup " + groupName);
		CodeModuleGroup cmGroup = CodeModuleGroupManager.load(groupName);
		if (cmGroup == null) {
			log("Did not find a module group for " + groupName + ", stop counting.");
			return 0;
		}
		// now count
		int n = 0;
		Enumeration modules = cmGroup.getModules();
		while (modules.hasMoreElements()) {  
			modules.nextElement();
		    n++;    
		}
		return n;
	}
	
	/**
	 * Returns whether the return code from a delete operation indicates success.
	 * @param code The code.
	 */
	private static boolean deleteReturnCodeOK(int code)
	{
		return
				code == CodeModuleManager.CMM_OK ||
				code == CodeModuleManager.CMM_OK_MODULE_MARKED_FOR_DELETION ||
				code == CodeModuleManager.CMM_OK_MODULE_OVERWRITTEN;
	}
	
	// For debugging, the following info is helpful:
	//
	// CMM_HANDLE_INVALID = 10
	// CMM_HASH_INVALID = 2
	// CMM_MODULE_EXCLUDED = 12
	// CMM_MODULE_IN_USE = 7
	// CMM_MODULE_IN_USE_BY_PERSISTENT_STORE = 8
	// CMM_MODULE_INCOMPATIBLE = 5
	// CMM_MODULE_INVALID = 4
	// CMM_MODULE_REQUIRED = 9
	// CMM_OK = 0
	// CMM_OK_MODULE_MARKED_FOR_DELETION = 6
	// CMM_OK_MODULE_OVERWRITTEN = 1
	// CMM_OUT_OF_MEMORY = 11
	// CMM_SIGNATURE_INVALID = 3
	// CMM_TRANSACTION_COMPLETE = 14
	// CMM_TRANSACTION_INVALID = 13
	// CMM_TRANSACTION_RESET_REQUIRED = 15
	
	
	/**
	 * Determines whether an application is installed as group.
	 * @param groupName The group name (essentially the application name).
	 */
	public static boolean isModuleGroupInstalled(String groupName)
	{
		log("isModuleGroupInstalled " + groupName);
		try {
			CodeModuleGroup group = CodeModuleGroupManager.load(groupName);
			return group != null;
		} catch (Throwable th) {
			log("Throwable in isModuleGroupInstalled " + groupName, th);
			return false;
		}
	}
	
	/**
	 * Determines whether an application module is installed.
	 * @param moduleName The module name.
	 *
	public static boolean isModuleInstalled(String moduleName)
	{
		log("isModuleInstalled " + moduleName);
		try {
			int moduleHandle = CodeModuleManager.getModuleHandle(moduleName);
			return moduleHandle != 0;
		} catch (Throwable th) {
			log("Throwable in isApplicationInstalled " + moduleName, th);
			return false;
		}
	}
	 */
	 
	/**
	 * Determines whether an application is installed.
	 * @param groupName groupName The group name (essentially the application name).
	 * @param version   The version.
	 * @return Once of the choices APPLICATION_INSTALLED, ...
	 */
	public static int isModuleGroupInstalled(String groupName, String version)
	{
		log("isModuleGroupInstalled " + groupName + " " + version);
		try {
			CodeModuleGroup group = CodeModuleGroupManager.load(groupName);
		
			if (group == null) {
				return APPLICATION_NOT_INSTALLED;
			}
			else {
				return getApplicationInstallStatus(group.getVersion(), version);
			}
		} catch (Throwable th) {
			log("Throwable in isModuleGroupInstalled " + groupName + " " + version, th);
			return APPLICATION_NOT_INSTALLED;
		}
	}
	
	/**
	 * Determines whether an application is installed.
	 * @param moduleName The module name.
	 * @param version    The version.
	 * @return Once of the choices APPLICATION_INSTALLED, ...
	 *
	public static int isModuleInstalled(String moduleName, String version)
	{
		log("isModuleInstalled " + moduleName + " " + version);
		try {
			int moduleHandle = CodeModuleManager.getModuleHandle(moduleName);
		
			if (moduleHandle == 0) {
				return APPLICATION_NOT_INSTALLED;
			}
			else {
				return getApplicationInstallStatus(CodeModuleManager.getModuleVersion(moduleHandle), version);
			}
		} catch (Throwable th) {
			log("Throwable in isModuleInstalled " + moduleName + " " + version, th);
			return APPLICATION_NOT_INSTALLED;
		}
	}
	 */

	/**
	 * Returns the status of a version compared to the installed version.
	 * @param installedVersion The installed version.
	 * @param testVersion    The version to be tested.
	 * @return Once of the choices APPLICATION_INSTALLED, ...
	 */
	public static int getApplicationInstallStatus(String installedVersion, String testVersion)
	{
		if (installedVersion == null) {
			return APPLICATION_NOT_INSTALLED;
		}
		int rc = BasicUtil.compareVersion(installedVersion, testVersion);
		if (rc == 0) {
			return APPLICATION_INSTALLED;
		} else if (rc < 0) {
			return APPLICATION_NEEDS_UPGRADE;
		} else {
			return APPLICATION_NEEDS_DOWNGRADE;
		}
	}
	
	/**
	 * Returns the list of applications installed on the device.
	 */
	public static AppVersion[] getInstalledApplications()
	{
		Vector result = new Vector();
		
		// Problem here is this: sometimes apps have groups, sometimes
		// they don't have groups. If an app has more than one module,
		// e.g. a lib for i18n, then it must be a group. But if an app
		// consists only of one COD module, it does not need to be a 
		// group. The CodeModuleGroupManager finds it when searching its
		// name, but it does not list it as a group.
		// Hence we have no choice: we simply assume the group and module
		// name space is the same and we collect both groups and modules.
		// Sometimes it will list an app twice, but that's life. I did
		// not find a better way.
		
		// First the groups
		
		Hashtable handledApps = new Hashtable();
		CodeModuleGroup[] groups = CodeModuleGroupManager.loadAll();
		if (groups != null) {
			for (int i = 0; i < groups.length; i++) {
				String vendor = groups[i].getVendor();
				String jsonid = groupIdToJSON(groups[i].getName(), vendor);
				AppVersion app = new AppVersion(
						jsonid,
						groups[i].getVersion(), 0);
				result.addElement(app);
				handledApps.put(jsonid, Boolean.TRUE);
				Enumeration e = groups[i].getModules();
				while (e.hasMoreElements()) {
				     String name = (String)e.nextElement();
				     handledApps.put(name, Boolean.TRUE);
				}
			}
		}

		// then the cod modules that are no groups
		
		int[] handles = CodeModuleManager.getModuleHandles(true);
		if (handles != null) {
			for (int i = 0; i < handles.length; i++) {
				if (!CodeModuleManager.isLibrary(handles[i])) {
					String id = CodeModuleManager.getModuleName(handles[i]);
					if (handledApps.get(id) == null) {
						String version = CodeModuleManager.getModuleVersion(handles[i]);
						AppVersion app = new AppVersion(id, version, 0);
						result.addElement(app);
					}
				}
			}
		}

		AppVersion[] r = new AppVersion[result.size()];
		result.copyInto(r);
		return r;
	}
	
	/**
	 * Extends the app with information from the code module manager.
	 * @param app The app.
	 */
	public static void updateFromCodeModuleManager(AppDetail app, boolean forAllVersions)
	{
		CodeModuleGroup group = CodeModuleGroupManager.load(app.getModuleGroupName());
		if (group == null) {
			return;
		}
		String vendor = group.getVendor();
		String installedVersion = group.getVersion();
		long downloaded = -1;
		long created = -1;
		Enumeration e = group.getModules();
		while (e.hasMoreElements()) {
			String moduleName = (String)e.nextElement();
			int handle = CodeModuleManager.getModuleHandle(moduleName);
			if (handle == 0) continue;
			if (!CodeModuleManager.isLibrary(handle)) {
				downloaded = CodeModuleManager.getModuleDownloadTimestamp(handle);
				created = CodeModuleManager.getModuleTimestamp(handle);
				if (vendor == null || vendor.length() == 0) {
					vendor = CodeModuleManager.getModuleVendor(handle);
				}
			}
		}
		
		// if we have a vendor, that one should dominate
		if (app.getVendor() != null && app.getVendor().length() > 0) {
			vendor = app.getVendor();
		}
		// if the installed version is not the display version, don't update
		// downloaded and created, as it would be for the wrong version
		if (!forAllVersions && !installedVersion.equals(app.getVersion())) {
			downloaded = -1;
			created = -1;
		}
		app.update(vendor, installedVersion, downloaded, created);
	}
	
	/**
	 * Converts a CodeModuleManager id to a JSON pkg.
	 * @param id The id.
	 * @param vendor The vendor.
	 * @return The pkg.
	 */
	public static String groupIdToJSON(String id, String vendor)
	{
		// if an app was installed OTA, BlackBerry always appends the vendor.
		if (id != null && vendor != null && id.endsWith(":" + vendor)) {
			return id.substring(0, id.length() - vendor.length() - 1);
		}
		return id;
	}
	
	/**
	 * Converts a JSON pkg to a CodeModuleManager id.
	 * @param pkg The pkg.
	 * @param vendor The vendor.
	 * @return The id.
	 */
	public static String groupIdFromJSON(String pkg, String vendor)
	{
		if (pkg != null && vendor != null)
			return pkg + ":" + vendor;
		return pkg;
	}
	
	/**
	 * Logging utility.
	 */
	private static void log(String s)
	{
		Log.log("BlackBerrySystem " + s);
	}
	
	private static void log(String s, Throwable th)
	{
		Log.log("BlackBerrySystem " + s, th);
	}
}