/*
 *  Licensed Materials - Property of IBM
 *  5725-G92 (C) Copyright IBM Corp. 2011, 2013. All Rights Reserved.
 *  US Government Users Restricted Rights - Use, duplication or
 *  disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 */

package com.ibm.appcenter.ui;

import net.rim.device.api.i18n.ResourceBundle;
import net.rim.device.api.system.Bitmap;
import net.rim.device.api.ui.MenuItem;
import net.rim.device.api.ui.Screen;
import net.rim.device.api.ui.UiApplication;
import net.rim.device.api.ui.component.StandardTitleBar;
import net.rim.device.api.ui.container.MainScreen;
import net.rim.device.api.ui.decor.Background;
import net.rim.device.api.ui.image.Image;
import net.rim.device.api.ui.image.ImageFactory;

import com.ibm.appcenter.connection.RESTCallBatch;
import com.ibm.appcenter.i18n.I18N;
import com.ibm.appcenter.i18n.I18NResource;
import com.ibm.appcenter.ui.widget.WaitPopupManager;
import com.ibm.appcenter.util.BusyIndicator;
import com.ibm.appcenter.util.Log;

/**
 * Base class for all screens of the application center.
 * Contains common functionality such as the menu handling.
 */
public abstract class AppCenterScreen
extends MainScreen
implements I18NResource
{	
	private boolean _busy;
	private BusyIndicator _indicator;
	private StandardTitleBar _titlebar;
	private RESTCallBatch _batch;
	
	
	/**
	 * Menu to go to the start screen.
	 * Placement ordinal: 110 (0 = top).
	 * Priority 10.
	 */
	private MenuItem _homeScreenItem = new PushScreenMenuItem(HOME_SCREEN, I18N.getBundle(), "homescreen92light.png", AppCenterScreenHomeMenuItem, 110, 10);
	
	/**
	 * Menu to go to the catalog screen.
	 * Placement ordinal: 111 (0 = top).
	 * Priority 10.
	 */
	private MenuItem _catalogScreenItem = new PushScreenMenuItem(CATALOG_SCREEN, I18N.getBundle(), "appcatalogscreen92light.png", AppCenterScreenCatalogMenuItem, 111, 10);
	
	/**
	 * Menu to go to the my applications screen.
	 * Placement ordinal: 112 (0 = top).
	 * Priority 10.
	 */
	private MenuItem _myAppsScreenItem = new PushScreenMenuItem(MYAPPS_SCREEN, I18N.getBundle(), "mymobilescreen92light.png", AppCenterScreenMyAppsMenuItem, 112, 10);

	/**
	 * Menu to go to the installed applications screen.
	 * Placement ordinal: 113 (0 = top).
	 * Priority 10.
	 */
	private MenuItem _updatesScreenItem = new PushScreenMenuItem(UPDATES_SCREEN, I18N.getBundle(), "updatesscreen92light.png", AppCenterScreenUpdatesMenuItem, 113, 10);
	
	/**
	 * Menu to go to the configuration screen.
	 * Placement ordinal: 114 (0 = top).
	 * Priority 10.
	 */
	private MenuItem _configScreenItem = new PushScreenMenuItem(CONFIG_SCREEN, I18N.getBundle(), "settingsscreen92light.png", AppCenterScreenConfigMenuItem, 114, 10);
	
	
	/**
	 * Constructor.
	 */
	public AppCenterScreen()
	{
		super();
		init();
	}
	
	/**
	 * Constructor.
	 */
	public AppCenterScreen(long style)
	{
		super(style | MainScreen.NO_SYSTEM_MENU_ITEMS);
		init();
	}
	
	/**
	 * Sets whether the screen is busy.
	 * Normally this should disable interactions.
	 */
	public void setBusy(boolean busy)
	{
		_busy = busy;
	}
	
	/**
	 * Returns whether the screen is busy.
	 */
	public boolean isBusy()
	{
		return _busy;
	}
	
	private void init()
	{
    	log("Constructor");
    	
		Background bg = UIUtil.getNormalBackground();
		if (bg != null) {
			getMainManager().setBackground(bg);
		}
    	_titlebar = new StandardTitleBar();
    	_titlebar.addSignalIndicator();
    	setTitleBar(_titlebar);
    	_batch = new RESTCallBatch(getBusyIndicator());
	}

	public void setTitle(String s)
	{
		_titlebar.addTitle(s);
	}
	
	/**
	 * Adds the view menu to the view.
	 */
	protected void addViewMenu()
	{
		addMenuItem(_homeScreenItem);
		if (!(this instanceof CatalogScreen)) {
			addMenuItem(_catalogScreenItem);
		}
		if (!(this instanceof MyAppsScreen)) {
			addMenuItem(_myAppsScreenItem);
		}
		if (!(this instanceof UpdatesScreen)) {
			addMenuItem(_updatesScreenItem);
		}
		if (!(this instanceof ConfigurationScreen)) {
			addMenuItem(_configScreenItem);
		}
	}
	
	/**
	 * Returns the busy indicator used for this screen.
	 */
	protected BusyIndicator getBusyIndicator()
	{
		if (_indicator == null)
			_indicator = new WaitPopupManager(this, 1000);
		return _indicator;
	}
	
	/**
	 * Returns the batch of REST calls normally used to
	 * update this screen.
	 */
	protected RESTCallBatch getRESTCallBatch()
	{
		return _batch;
	}
	
    /**
     * Called when the screen becomes visible or invisible.
     */
    protected void onUiEngineAttached(boolean attached)
    {
    	log("onUiEngineAttached " + attached);
    	super.onUiEngineAttached(attached);
    	
    	if (!attached) {
    		// Screen becomes invisible
    		// cancel pending REST operations that are
    		// relevant for the UI
    		getRESTCallBatch().stop();
    		if (_indicator != null) {
        		_indicator.setBusy(false);
    		}
    	}
    }
    
    /**
     * Called when the screen becomes exposed again after being
     * hidden under a different screen.
     */
    protected void onExposed()
    {
    	log("onExposed");
    	super.onExposed();
    }
    
    /**
     * Called when the screen becomes obscured by another screen.
     */
    protected void onObscured()
    {
    	log("onObscured");
    	super.onObscured();
    }
    
    /**
     * This is called when this screen is closed.
     */
    public void close()
    {
    	log("close");
        super.close();
    }
    
	// --------------------------------------------------------
	
	/**
	 * Returns the screen name for logging.
	 */
	public abstract String getScreenName();
	
	private boolean _logEnabled = true;
	
	/**
	 * Enable or disable logging for this screen.
	 */
	public void setLogEnabled(boolean enable)
	{
		_logEnabled = enable;
	}
	
	/**
	 * Returns whether logging for this screen is enabled.
	 */
	public boolean isLogEnabled()
	{
		return _logEnabled;
	}
		
	/**
	 * Logs a throwable with a message.
	 * @param msg The message.
	 * @param th  The throwable.
	 */
	public void log(String msg, Throwable th)
	{
		if (_logEnabled)
			Log.log(getScreenName() + " " + msg, th);
	}
	
	/**
	 * Logs a message.
	 * @param msg The message.
	 */
	public void log(String msg)
	{
		if (_logEnabled)
			Log.log(getScreenName() + " " + msg);
	}
	
	/**
	 * Logs a message with arguments.
	 * @param msg The message.
	 * @param args The arguments.
	 *
	public void log(String msg, Object[] args)
	{
		if (_logEnabled)
			Log.log(getScreenName() + " " + msg, args);
	}
	 */
	
	// --------------------------------------------------------
	
	private static final int CONFIG_SCREEN   = 0;
	private static final int CATALOG_SCREEN  = 1;
	private static final int MYAPPS_SCREEN   = 2;
	private static final int UPDATES_SCREEN  = 3;
	private static final int HOME_SCREEN     = 4;
	
	/**
	 * A menu item that can push a screen.
	 */
	private class PushScreenMenuItem extends MenuItem
	{
		private int _mode;
		
		public PushScreenMenuItem(int mode, ResourceBundle bundle, String iconName, int textKey, int ordinal, int priority)
		{
			super(bundle, textKey, ordinal, priority);
			_mode = mode;
			
			if (iconName != null) {
				Bitmap bm = UIUtil.getScaledBitmap(iconName, 92, 92);
				Image im = ImageFactory.createImage(bm);
				this.setIcon(im);
			}
		}
		
		public void run() 
		{
			log("PushMenuItem " + _mode + " run");
			UiApplication app = UiApplication.getUiApplication();
			// pop all screens until we are at the start screen
			Screen screen = app.getActiveScreen();
			while (!(screen instanceof StartScreen)) {
				app.popScreen(screen);
				screen = app.getActiveScreen();
			}
			// then add the screen
			switch (_mode) {
				case CONFIG_SCREEN:
					app.pushScreen(new ConfigurationScreen());
					break;
				case CATALOG_SCREEN:
					app.pushScreen(new CatalogScreen());
					break;
				case MYAPPS_SCREEN:
					app.pushScreen(new MyAppsScreen());
					break;
				case UPDATES_SCREEN:
					app.pushScreen(new UpdatesScreen());
					break;
				case HOME_SCREEN:
					// nothing to do
					break;
			}
		}
	};
}
