/*
 *  Licensed Materials - Property of IBM
 *  5725-G92 (C) Copyright IBM Corp. 2011, 2013. All Rights Reserved.
 *  US Government Users Restricted Rights - Use, duplication or
 *  disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 */

package com.ibm.appcenter.ui;

import net.rim.device.api.system.CodeModuleManager;
import net.rim.device.api.system.Display;
import net.rim.device.api.ui.Field;
import net.rim.device.api.ui.FieldChangeListener;
import net.rim.device.api.ui.Manager;
import net.rim.device.api.ui.component.ButtonField;
import net.rim.device.api.ui.component.Dialog;
import net.rim.device.api.ui.component.LabelField;
import net.rim.device.api.ui.component.SeparatorField;
import net.rim.device.api.ui.container.VerticalFieldManager;

import com.ibm.appcenter.artifacts.AppDetail;
import com.ibm.appcenter.artifacts.AppSummary;
import com.ibm.appcenter.artifacts.AppVersion;
import com.ibm.appcenter.artifacts.CodModule;
import com.ibm.appcenter.connection.ApplicationUtil;
import com.ibm.appcenter.connection.RESTCall;
import com.ibm.appcenter.connection.RESTCallBatch;
import com.ibm.appcenter.connection.RESTCallContinuation;
import com.ibm.appcenter.i18n.I18N;
import com.ibm.appcenter.system.BlackBerrySystem;
import com.ibm.appcenter.ui.widget.EnhancedBitmapButtonField;
import com.ibm.appcenter.ui.widget.EnhancedLabelField;
import com.ibm.appcenter.ui.widget.EnhancedTwoColumnField;
import com.ibm.appcenter.ui.widget.EnhancedTwoColumnFieldManager;
import com.ibm.appcenter.ui.widget.LeadingTrailingManager;
import com.ibm.appcenter.ui.widget.ProgressBarAndButtonSet;
import com.ibm.appcenter.util.BasicUtil;

/**
 * The detail pane for an application.
 */
public class AppPropertyDetailPane
extends AppPropertyPane
{
	private ProgressBarAndButtonSet _progressSet;
	private ButtonField _installButton;
	private ButtonField _deleteButton;
	private int _appInstallStatus;
	
	private EnhancedBitmapButtonField _nf1 = null;
	
	private boolean _mayNeedDeviceRestart = false;
	private Runnable _cleanupInstall = null;
	
	// during installation, this batch is used
	private RESTCallBatch _installBatch = new RESTCallBatch(null) {
		public synchronized void stop() {
			super.stop();
			// ensure that _cleanupInstall runs
			if (_cleanupInstall != null) {
				_cleanupInstall.run();
				_cleanupInstall = null;
			}
		}
	};
	
	/**
	 * Creates the pane.
	 * @param title The title of the pane.
	 * @param parent The parent screen.
	 */
	public AppPropertyDetailPane(String title, AppPropertyScreen parent)
	{
		super(title, parent);
	}
	
	/**
	 * Rebuilds the pane. This is called whenever
	 * the data about the application is refreshed.
	 */
	public void build()
	{
		log("build");
		
		VerticalFieldManager content = getContent();
		
    	if (content == null) return;
    	
    	getParent().setSafeFocus();
    	
        AppVersion vapp = getAppVersion();
        AppSummary sapp = (vapp instanceof AppSummary ? (AppSummary)vapp : null);
        AppDetail  dapp = (sapp instanceof AppDetail  ? (AppDetail)sapp : null);
        
    	_appInstallStatus = BlackBerrySystem.isModuleGroupInstalled(vapp.getModuleGroupName(), vapp.getVersion());
    	
    	log("hasFocus" + checkFocus());
    	content.deleteAll();
    	
        int margin = UIUtil.getNormalXAdvanceInPxl() / 2;
        content.setMargin(margin, margin, margin, margin);
        int topmargin = margin;
        
        // artificial focus field for trackpad-only devices.
        // Touch devices don't have this problem but trackpad-only devices
        // need a focussable button at the top and bottom of the pane otherwise
        // the pane does not scroll well.
        int nfsize = Math.min(5, margin / 2);
        _nf1 = new EnhancedBitmapButtonField("emptydark", nfsize, Field.FIELD_HCENTER);
        content.add(_nf1);
        
        FieldChangeListener installAppListener = new FieldChangeListener() {
            public void fieldChanged(Field field, int context) {
                installApp();
            }
        };
        FieldChangeListener deleteAppListener = new FieldChangeListener() {
            public void fieldChanged(Field field, int context) {
                deleteApp();
            }
        };
        
        _installButton = new ButtonField(I18N.getString(AppPropertyInstallButtonTitle), Field.FIELD_HCENTER | ButtonField.CONSUME_CLICK);
        _installButton.setChangeListener(installAppListener);
        _installButton.setEnabled(canInstall());
        
        _deleteButton = new ButtonField(I18N.getString(AppPropertyUninstallButtonTitle), Field.FIELD_HCENTER | ButtonField.CONSUME_CLICK);
        _deleteButton.setChangeListener(deleteAppListener);
        _deleteButton.setEnabled(canDelete());
        
        _progressSet = new ProgressBarAndButtonSet(Manager.USE_ALL_WIDTH) {
        	protected void onCancel() {
        		onCancelInstallation();
        	}
        };
        _progressSet.setMargin(2 * margin, margin, 2 * margin, margin);
        updateInstallButtons();
        
        content.add(_progressSet);
        
        SeparatorField separator = new SeparatorField(SeparatorField.LINE_HORIZONTAL);
        content.add(separator);
        
        // description field 
        if (dapp != null) {
        	String s = dapp.getDescription();
        	if (s != null && s.length() > 0 && !s.equals("null")) {
        		LabelField descrField = new EnhancedLabelField(s);
        		descrField.setMargin(topmargin, margin, margin, margin); // TRBL
        		descrField.setFont(UIUtil.getNormalFont());
        		content.add(descrField);
        		// separate the remaining a bit from the description
        		topmargin = margin;
        		
        		separator = new SeparatorField(SeparatorField.LINE_HORIZONTAL);
                content.add(separator);
        	}
        }

        // version
        LabelField versionField = null;
        LabelField sizeField = null;
        if (sapp != null) {
        	String s = sapp.getVersion();
        	if (s != null && s.length() > 0 && !s.equals("null")) {
        		if (getParent().isForAllVersions()) {
        			s = I18N.getString(AppArtifactLatestVersionInfo, s);
        		} else {
        			s = I18N.getString(AppArtifactVersionInfo, s);
        		}
        		versionField = new EnhancedLabelField(s, Field.FIELD_LEADING);
        		versionField.setFont(UIUtil.getMiniFont());
        	}
        }
        
        // size
        if (dapp != null) {
        	String s = getAppSize(dapp.getModulesTotalLength());
        	if (s != null && s.length() > 0) {
        		if (versionField == null) {
        			// at the leading side, add label
        			sizeField = new EnhancedLabelField(
        					I18N.getString(AppArtifactSizeInfo, s), Field.FIELD_LEADING);
        		} else {
        			// at the trailing side, no label needed
        			sizeField = new EnhancedLabelField(s, Field.FIELD_TRAILING);
        		}
        		sizeField.setFont(UIUtil.getMiniFont());
        	}
        }
    	
        // put version and size together in the same line
        if (versionField != null || sizeField != null) {
        	LeadingTrailingManager ltmgr = new LeadingTrailingManager(Manager.USE_ALL_WIDTH);
        	ltmgr.setMargin(topmargin, margin, 0, margin);
        	if (versionField != null)
        		ltmgr.add(versionField);
        	if (sizeField != null)
        		ltmgr.add(sizeField);
        	content.add(ltmgr);
    		topmargin = 0;
        }
        
        // installed version
        if (dapp != null) {
        	String s = dapp.getInstalledVersion();
        	if (s != null && s.length() > 0 && !s.equals("null")) {
        		LabelField field = new EnhancedLabelField(
        				I18N.getString(AppArtifactInstalledVersionInfo, s));
        		field.setMargin(topmargin, margin, 0, margin);
        		field.setFont(UIUtil.getMiniFont());
        		content.add(field);
        		topmargin = 0;
        	}
        }
        
        // next few is a group
        EnhancedTwoColumnFieldManager group =
        		new EnhancedTwoColumnFieldManager(Display.getWidth() / 2, 0);
        boolean needGroup = false;
        
        // vendor
        if (dapp != null) {
        	String s = dapp.getVendor();
        	if (s != null && s.length() > 0 && !s.equals("null")) {
        		LabelField field1 = new EnhancedLabelField(
        				I18N.getString(AppArtifactVendorLabel));
        		LabelField field2 = new EnhancedLabelField(s);
        		EnhancedTwoColumnField field = new EnhancedTwoColumnField(field1, field2, 0);
        		field.setMargin(topmargin, margin, 0, margin);
        		field1.setFont(UIUtil.getMiniFont());
        		field2.setFont(UIUtil.getMiniFont());
        		group.add(field);
        		needGroup = true;
        		topmargin = 0;
        	}
        }
        
        // author
        if (dapp != null) {
        	String s1 = dapp.getAuthorDisplay();
        	String s2 = dapp.getAuthorLogin();
        	String s = (s1 != null && s1.length() > 0 && !s1.equals("null")) ? s1 : s2;
        	if (s != null && s.length() > 0 && !s.equals("null")) {
        		LabelField field1 = new EnhancedLabelField(
        				I18N.getString(AppArtifactAuthorLabel));
        		LabelField field2 = new EnhancedLabelField(s);
        		EnhancedTwoColumnField field = new EnhancedTwoColumnField(field1, field2, 0);
        		field.setMargin(topmargin, margin, 0, margin);
        		field1.setFont(UIUtil.getMiniFont());
        		field2.setFont(UIUtil.getMiniFont());
        		group.add(field);
        		needGroup = true;
        		topmargin = 0;
        	}
        }
        
        // popularity
        if (dapp != null) {
        	double d = dapp.getPopularity();
        	if (d > 0) {
        		String s = BasicUtil.formatNumber(d, 0);
        		LabelField field1 = new EnhancedLabelField(
        				I18N.getString(AppArtifactPopularityLabel));
        		LabelField field2 = new EnhancedLabelField(s);
        		EnhancedTwoColumnField field = new EnhancedTwoColumnField(field1, field2, 0);
        		field.setMargin(topmargin, margin, 0, margin);
        		field1.setFont(UIUtil.getMiniFont());
        		field2.setFont(UIUtil.getMiniFont());
        		group.add(field);
        		needGroup = true;
        		topmargin = 0;
        	}
        }
        
        // time stamps
        
        /* For now we decided that the created timestamp is not useful
        if (dapp != null) {
        	String s = dapp.getCreatedAsString();
        	if (s != null && s.length() > 0) {
        		LabelField field1 = new EnhancedLabelField(
        				I18N.getString(AppArtifactCreatedTimeLabel));
        		LabelField field2 = new EnhancedLabelField(s);
        		EnhancedTwoColumnField field = new EnhancedTwoColumnField(field1, field2, 0);
        		field.setMargin(topmargin, margin, 0, margin);
        		field1.setFont(UIUtil.getMiniFont());
        		field2.setFont(UIUtil.getMiniFont());
        		group.add(field);
        		needGroup = true;
        		topmargin = 0;
        	}
        }
        */
        
        if (dapp != null) {
        	String s = dapp.getUpdatedAsString();
        	if (s != null && s.length() > 0) {
        		LabelField field1 = new EnhancedLabelField(
        				I18N.getString(AppArtifactUploadedTimeLabel));
        		LabelField field2 = new EnhancedLabelField(s);
        		EnhancedTwoColumnField field = new EnhancedTwoColumnField(field1, field2, 0);
        		field.setMargin(topmargin, margin, 0, margin);
        		field1.setFont(UIUtil.getMiniFont());
        		field2.setFont(UIUtil.getMiniFont());
        		group.add(field);
        		needGroup = true;
        		topmargin = 0;
        	}
        }
        if (dapp != null) {
        	String s = dapp.getDownloadedAsString();
        	if (s != null && s.length() > 0) {
        		LabelField field1 = new EnhancedLabelField(
        				I18N.getString(AppArtifactInstalledTimeLabel));
        		LabelField field2 = new EnhancedLabelField(s);
        		EnhancedTwoColumnField field = new EnhancedTwoColumnField(field1, field2, 0);
        		field.setMargin(topmargin, margin, 0, margin);
        		field1.setFont(UIUtil.getMiniFont());
        		field2.setFont(UIUtil.getMiniFont());
        		group.add(field);
        		needGroup = true;
        		topmargin = 0;
        	}
        }
        
        if (needGroup) {
        	content.add(group);
        }
                
        // dummy button just so that the scrolling behaves better
        EnhancedBitmapButtonField nf2 = new EnhancedBitmapButtonField("emptydark", nfsize, Field.FIELD_HCENTER);
        content.add(nf2);
        
        log("build finished " + content.getFieldCount());
	}
	
    /**
     * Returns a string describing the application size.
     * @param lengthInBytes The number of bytes of the application.
     * @return A string such as "x KB" or "y.zz MB"
     */
    private static String getAppSize(int lengthInBytes)
    {
    	int kbSize = (lengthInBytes + 1023) / 1024;
    	if (kbSize < 1024) {
    		return I18N.getString(AppArtifactSizeInKiloByteInfo, "" + kbSize);
    	} else {
    		double mbSize = ((double)kbSize) / 1024;
    		return I18N.getString(AppArtifactSizeInMegaByteInfo, BasicUtil.formatNumber(mbSize, 2));
    	}
    }
    
	/**
	 * Enable or disable the interactive parts of the UI.
	 */
	public void setEnabled(boolean enable)
	{
		log("setEnabled " + enable);
    	if (enable) {
    		if (_installButton != null) {
    			_installButton.setEnabled(canInstall());
    		}
    		if (_deleteButton != null) {
    			_deleteButton.setEnabled(canDelete());
    		}
    	} else {
    		if (_installButton != null) {
    			_installButton.setEnabled(false);
    		}
    		if (_deleteButton != null) {
    			_deleteButton.setEnabled(false);
    		}
    	}
	}
	
	
	/**
	 * Returns the pane name for logging.
	 */
	public String getPaneName()
	{
		return "ADetPane";
	}
	
	// ------------------------------------------------------------------------
	// Everything related to installing
	
    private void updateInstallButtons()
    {
    	if (_progressSet == null) return;
    	
    	if (canInstall()) {
    		if (canDelete()) {
    			_progressSet.deleteAll();
    			_progressSet.add(_deleteButton);
    			_progressSet.add(_installButton);
    			_installButton.setEnabled(true);
    			_deleteButton.setEnabled(true);
    		} else {
    			_progressSet.deleteAll();
    			_progressSet.add(new LabelField(" "));
    			_progressSet.add(_installButton);
    			_installButton.setEnabled(true);
    			_deleteButton.setEnabled(false);
    		}
    		_installButton.setLabel(getInstallButtonLabel(_appInstallStatus));
    	} else {
    		if (canDelete()) {
    			_progressSet.deleteAll();
    			_progressSet.add(_deleteButton);
    			_progressSet.add(new LabelField(" "));
    			_installButton.setEnabled(false);
    			_deleteButton.setEnabled(true);
    		} else {
    			_progressSet.deleteAll();
    			_progressSet.add(new LabelField(" "));
    			_installButton.setEnabled(false);
    			_deleteButton.setEnabled(false);
    		}
    	}
    }
    
    /**
     * Returns the label for an install button.
     * @param installStatus The status of the version to be installed
     *            compared to the version that is installed.
     * @return The label "Install", "Upgrade", ...
     */
    String getInstallButtonLabel(int installStatus)
    {
		switch (installStatus) {
			case BlackBerrySystem.APPLICATION_INSTALLED:
				return I18N.getString(AppPropertyReinstallButtonTitle);
			case BlackBerrySystem.APPLICATION_NEEDS_DOWNGRADE:
				return I18N.getString(AppPropertyRevertToLowerVersionButtonTitle);
			case BlackBerrySystem.APPLICATION_NEEDS_UPGRADE:
				return I18N.getString(AppPropertyUpdateToHigherVersionTitle);
			case BlackBerrySystem.APPLICATION_NOT_INSTALLED:
				return I18N.getString(AppPropertyInstallButtonTitle);
		}
		return "";
    }
    
    /**
     * Called when the cancel button of the progress indicator is pressed.
     */
    void onCancelInstallation()
    {
    	log("onCancelInstallation");
    	_installBatch.stop();
    }
      
    /**
     * Returns true if installation is possible.
     */
    private boolean canInstall()
    {
    	if (!getAppVersion().allowInstall()) {
    		return false;
    	}
    	
    	if (!(getAppVersion() instanceof AppDetail)) {
    		return false;
    	}
    	AppDetail app = (AppDetail)getAppVersion();
    	if (app.getModules() == null || app.getModules().length == 0) {
    		return false;
    	}
    	return true;
    }
    
    /**
     * Returns true if deletion is possible.
     */
    private boolean canDelete()
    {
    	if (!getAppVersion().allowDelete()) {
    		return false;
    	}
    	
    	if (!(getAppVersion() instanceof AppDetail)) {
    		return false;
    	}

    	// we can only delete if the cod module is installed
    	// and if the group is installed as well ...
    	// In theory, modules without group are deletable
    	// but the installer creates always groups and we
    	// want to delete only what we did install ourself.
    	
    	if (_appInstallStatus == BlackBerrySystem.APPLICATION_NOT_INSTALLED)
    		return false;
    	
    	AppDetail dapp = (AppDetail)getAppVersion();
    	if (!getParent().isForAllVersions() && !dapp.getVersion().equals(dapp.getInstalledVersion()))
    		return false;
    	    	
    	return true;
    }
        
    /**
     * Installs the application.
     */
    private void installApp()
    {
    	// we use our own batch, in order not to interfere with the normal
    	// screen refresh batch.
    	if (getAppVersion() instanceof AppDetail) {
    		_installBatch = new RESTCallBatch(null) {
    			public synchronized void stop() {
    				super.stop();
    				// ensure that _cleanupInstall runs
    				if (_cleanupInstall != null) {
    					_cleanupInstall.run();
    					_cleanupInstall = null;
    				}
    			}
    		};
    		installApp((AppDetail)getAppVersion(), _progressSet);
    	}
    }

    /**
     * This does the work.
     * @param app
     * @param progressIndicator
     */
    private void installApp(
    		final AppDetail app, 
    		final ProgressBarAndButtonSet progressIndicator)
    {
    	int rc = Dialog.ask(Dialog.D_YES_NO, 
    				I18N.getString(AppPropertyConfirmInstallDialog, app.getName(), app.getVersion()));
    	if (rc != Dialog.YES) {
    		// UIUtil.showStatus(I18N.getString(AppPropertyConfirmInstallCancelledMsg));
    		setBusy(false);
    		return;
    	}
    	
    	log("Install " + app.getName());
    	
    	_nf1.setFocus();
    	
    	final CodModule[] modules = app.getModules();
    	if (modules == null) {
    		log("No modules");
    		setBusy(false);
    		return;
    	}
    	final int n = modules.length;
    	if (n == 0) {
    		log("No modules");
    		setBusy(false);
    		return;
    	} else {
    		log("First Cod Module is " + modules[0].getURL());
    	}
    	    	
    	// ok, we need to think backward: this is what we need to do at the
    	// very end in any case. This is guaranteed.
    	_mayNeedDeviceRestart = false;
		final Runnable cleanup = new Runnable() {
			public void run() {
				// refresh the UI
				setBusy(false);
				progressIndicator.setProgressMode(false);
				BlackBerrySystem.updateFromCodeModuleManager((AppDetail)getAppVersion(), getParent().isForAllVersions());
				getParent().updateUI();
				if (CodeModuleManager.isResetRequired() && _mayNeedDeviceRestart) {
					CodeModuleManager.promptForResetIfRequired();
				}
				_cleanupInstall = null;
			}};
    				
		// now start
		setBusy(true);
    	progressIndicator.setProgressMaxValue(2 * n);
    	progressIndicator.setProgressMode(true);
    	// 1. Call REST chain to get all data of all cod modules
    	ApplicationUtil.retrieveCodModulesData(app,
    			_installBatch,
    			new RESTCallContinuation() {

					public void onSuccess(String contentType, byte[] bytes) {
						boolean wasInstalled = BlackBerrySystem.isModuleGroupInstalled(app.getModuleGroupName());
						// ok, from now on, we don't want to stop this call
						_installBatch.setCanStop(false);
						_mayNeedDeviceRestart = true;
						// this does the real installation on the device
						if (finishInstall(app, progressIndicator)) {
							UIUtil.showStatus(I18N.getString(AppActionInstallSuccessMsg, app.getName(), app.getVersion()));
						} else {
							UIUtil.showStatus(I18N.getString(AppActionInstallFailureMsg, app.getName(), app.getVersion()));
						}
						RESTCall next = ApplicationUtil.getRecordInstallationResultCall(app, wasInstalled, progressIndicator, cleanup);
						if (next != null) {
							_installBatch.appendAtRunning(next);
						} else {
							UIUtil.invokeInEventThread(cleanup);
						}
					}

					public void onFailure(Exception ex, int status,
							String contentType, byte[] bytes) {
						UIUtil.showStatus(I18N.getString(AppActionInstallDownloadFailureMsg, app.getName(), app.getVersion()));
						UIUtil.invokeInEventThread(cleanup);
					}

					public void onCancel() {
						progressIndicator.setProgressMode(false);
						UIUtil.invokeInEventThread(cleanup);
					}},
				progressIndicator);
    	
    	// do this after the batch has started!!! Otherwise it would cleanup on start.
    	_cleanupInstall = cleanup;
    }
    
    private boolean finishInstall(
    					AppDetail app,
    					ProgressBarAndButtonSet progressIndicator)
    {
    	log("start finishInstall");
    	progressIndicator.setCancelEnabled(false);
    	CodModule[] modules = app.getModules();
    	if (modules == null) {
    		log("No modules");
    		return false;
    	}
    	int n = modules.length;
    	if (n == 0) {
    		log("No modules");
    		return false;
    	}
    	
    	// verify that all are loaded and have data
    	for (int i = 0; i < n; i++) {
    		if (modules[i].getData() == null) {
    			log("Module " + i + " has no bytes");
    			return false;
    		}
    		if (modules[i].getData().length == 0) {
    			log("Module " + i + " has no bytes");
    			return false;
    		}
    		if (modules[i].getData().length != modules[i].getSize()) {
    			log("Module " + i + " has wrong size");
    			return false;
    		}
    	}
    	
    	log("Start writing modules");
    	boolean rc = BlackBerrySystem.installApplication(
    					progressIndicator,
    					app.getModuleGroupName(),
    					app.getModuleGroupFriendlyName(), 
    					app.getVendor(),
    					app.getVersion(),
    					app.getDescription(),
    					app.getData());
    	return rc;
    }
    
    /**
     * Deletes the application.
     */
    private void deleteApp()
    {
    	if (getAppVersion() instanceof AppDetail) {
    		deleteApp((AppDetail)getAppVersion());
    	}
    }
    
    private void deleteApp(final AppDetail app)
    {
    	int rc = Dialog.ask(Dialog.D_YES_NO, 
				I18N.getString(AppPropertyConfirmUninstallDialog, app.getName(), app.getVersion()));
    	if (rc != Dialog.YES) {
    		// UIUtil.showStatus(I18N.getString(AppPropertyConfirmUninstallCancelledMsg));
    		setBusy(false);
    		return;
    	}
	
    	_nf1.setFocus();
    	
    	int n = BlackBerrySystem.getNumberOfModulesOfGroup(app.getModuleGroupName());
    	log("deleteApp" + app.getModuleGroupName() + " found " + n + " modules");
    	setBusy(true);
    	_progressSet.setCancelEnabled(false);
    	_progressSet.setProgressMaxValue(n + 1);
    	_progressSet.setProgressMode(true);
    	
    	// ok, we need to think backward: this is what we need to do at the
    	// very end in any case. This is guaranteed.
		final Runnable cleanup = new Runnable() {
			public void run() {
				// refresh the UI
				setBusy(false);
				_progressSet.setProgressMode(false);
				getParent().updateUI();
				if (CodeModuleManager.isResetRequired()) {
					CodeModuleManager.promptForResetIfRequired();
				}
			}};
			
    	// now do it
		Thread thread = new Thread() {
			public void run() {
				if (BlackBerrySystem.deleteApplication(_progressSet, false, app.getModuleGroupName())) {
		    		RESTCall call = ApplicationUtil.getRecordInstallationResultCall(app, true, _progressSet, cleanup);
		    		_installBatch.append(call);
		    		_installBatch.start();
		    		UIUtil.showStatus(I18N.getString(AppActionUninstallSuccessMsg, app.getName()));
				} else {
					UIUtil.showStatus(I18N.getString(AppActionUninstallFailureMsg, app.getName()));
					UIUtil.invokeInEventThread(cleanup);
				}
			}
		};
    	thread.start();
    }
    
    /**
     * Sets the screen busy.
     */
    private void setBusy(boolean busy)
    {
    	getParent().setBusy(busy);
    }

}
