/*
 *  Licensed Materials - Property of IBM
 *  5725-G92 (C) Copyright IBM Corp. 2011, 2013. All Rights Reserved.
 *  US Government Users Restricted Rights - Use, duplication or
 *  disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 */

package com.ibm.appcenter.ui;

import net.rim.device.api.system.PersistentObject;
import net.rim.device.api.system.PersistentStore;
import net.rim.device.api.util.Persistable;

/**
 * The configuration data for the application.
 * The contains the user, password and server info.
 */
public class ConfigurationData 
implements Persistable 
{
    private String _user;
    private String _password;
    private String _server;
    private boolean _ssl;
    // if empty, we use automated connection suffix
    private String _connectionSuffix;
    private int    _timeout;

    // Some hash for the persistence mechanism 
    private static final long PERSISTENCE_ID = 0x6eaf8c5691e4217dL;

    // Persistent object wrapping the effective properties instance
    private static PersistentObject store;

    // Singleton for more conveniant access
    private static ConfigurationData _instance;
    
    // Ensure that an effective properties set exists on startup.
    static {
        store = PersistentStore.getPersistentObject(PERSISTENCE_ID);
        synchronized (store) {
            if (store.getContents() == null) {
                store.setContents(new ConfigurationData());
                store.commit();
            }
        }
    }

    /**
     * Constructor.
     */
    private ConfigurationData()
    {
        _user = "";
        _password = "";
        _server = "";
        _ssl = false;
        _connectionSuffix = "";
        _timeout = 10000;
    }        

    /**
     * Copy constructor.
     */
    private ConfigurationData(ConfigurationData other)
    {
        _user = other._user;
        _password = other._password;
        _server = other._server;
        _ssl = other._ssl;
        _connectionSuffix = other._connectionSuffix;
        _timeout = other._timeout;
    }

    /**
     * Retrieve the configuration data from the persistent storage.
     */
    public static ConfigurationData get()
    {
    	if (_instance != null) {
    		return _instance;
    	}
    	// store should always be there, see static initializer
        synchronized (store)
        {
            ConfigurationData savedProps =
              (ConfigurationData) store.getContents();
            _instance = new ConfigurationData(savedProps);
            return _instance;
        }
    }

    /**
     * Save the configuration into the persistent storage.
     */
    public void save()
    {
    	// store should always be there, see static initializer
        synchronized (store)
        {
        	if (!UIUtil.isSavePasswordOnDevice()) {
        		_password = "";
        	}
        	store.setContents(this);
        	store.commit();
        	_instance = null;
        }
    }

    /**
     * Returns the user of the server connection.
     */
    public String getUser()
    {
    	return _user;
    }
    
    /**
     * Sets the user of the server connection.
     * @param user The user
     */
    public void setUser(String user)
    {
    	_user = user;
    }
    
    /**
     * Returns the password of the server connection.
     */
    public String getPassword()
    {
    	return decode(_password);
    }
    
    /**
     * Sets the password of the server connection.
     * @param password The password
     */
    public void setPassword(String password)
    {
    	_password = encode(password);
    }
    
    /**
     * Returns the URL of the server connection.
     */
    public String getServer()
    {
    	return _server;
    }
    
    /**
     * Sets the URL of the server connection.
     * @param server The URL of the server connection
     */
    public void setServer(String server)
    {
    	_server = server;
    }
    
    /**
     * Returns whether the server connection uses SSL.
     */
    public boolean isSSL()
    {
    	return _ssl;
    }
    
    /**
     * Sets whether the server connection uses SSL.
     * @param ssl Flag whether ssl is used.
     */
    public void setSSL(boolean ssl)
    {
    	_ssl = ssl;
    }
    
    /**
	 * Returns the timeout used in all REST services
	 * unless explicitly specified.
	 */
	public int getTimeout()
	{
		return _timeout;
	}
	
	/**
	 * Sets the timeout used in all REST services,
	 * unless explicitly specified.
	 * @param timeout The time in milliseconds.
	 */
	public void setTimeout(int timeout)
	{
		if (timeout < 100) {
			timeout = 100;
		}
		_timeout = timeout;
	}
	   
    // Documentation concerning the connection suffix:
    // This is needed because Blackberry has many different ways how
    // to create a HTTP connection. Some of the ways are for public
    // users, other ways are for enterprise users who are forced to
    // route the traffic through an Business Enterprise Server (BES).
    // The related documentation can be found here:
    //
    // http://supportforums.blackberry.com/t5/Java-Development/Network-Transports/ta-p/482457
    //
    // We do automatic detection of the connection suffix only in some
    // cases, in the following order:
    // 1) if we are in the simulator, we use direct HTTP
    // 2) if Wifi is available, we prefer to use Wifi
    // 3) if WAP2 is available, we use WAP2
    // 4) otherwise we try to use direct TCP
    //
    // Nowadays, nearly all devices have Wifi, so in most cases 2) is
    // sufficient. But the user might have turned Wifi off, or might
    // have no Wifi access point nearby, in which case we fall back to
    // WAP2. Nearly all newer devices have WAP2, hence WAP1 does not
    // really play a role here. Enterprise devices usually have a 
    // service book. Enterprise administrators can specify in the service
    // book which kind of connections are allowed; for instance that can
    // forbid wifi except through BES.
    // We do not attempt to use WAP1, since we have no way to figure out
    // the credentials automatically.
    // We do not attempt to use BES or BIS, since we have no way to
    // figure out the BIS-B secret or the BES credentials.
    // If those connection modes are needed, the connection suffix must
    // be specified.
    //
    // What are the possible connection suffixes:
    //
    // BES/MDS (Mobile device service):
    // deviceside=false
    // The connection goes through either through a wifi hotspot or through
    // the mobile carrier (GPRS, 3G etc.) to the Blackberry Infrastructure 
    // Network Operation Center (NOC). This path is encrypted and compressed.
    // The NOC then sends the data to your own Blackberry Enterprise Server
    // (BES), from where it is routed to the final destination. This option 
    // is only available if your company has set up a BES and if the device
    // is registered for MDS through that BES inside the device service book.
    // If no connection suffix would be specified, this would be the default.
    // However note that the BlackBerry AppCenter Client requires to specify
    // deviceside=false, since no connection suffix means automatic connection
    // detection.
    //
    // BES/MDS with explicit connection UID:
    // deviceside=false;ConnectionUID=XYZ
    // Essentially the same as MDS. Instead of looking up the default 
    // connection UID from the service book, it uses the specified one.
    // This may be useful if your company has set up multiple BES.
    //
    // Explicit Wifi (without MDS):
    // interface=wifi
    // The connection goes through a wifi hotspot to the internet
    // to the server. The automatic connection suffix detection will
    // try explicit wifi whenever it is available. This works for 
    // consumer and enterprise customers since it does not require any BES.
    // The connection is not secure (except when adding SSL options).
    //
    // WAP 2.0:
    // deviceside=true;ConnectionUID=XYZ
    // where XYZ is your id connection ID from the service book.
    // The connection goes through the mobile carrier (GPRS, 3G etc.)
    // to the mobile carrier's WAP gateway to the internet to the server. 
    // If Wifi is not available, the automatic connection suffix detection will 
    // find the connection UID automatically.
    //
    // WAP 1.0/1.1:
    // WapGatewayIP=127.0.0.1;WapGatewayPort:9201; WapGatewayAPN=XYZ
    // This is similar to WAP 2.0 however WAP 1.x is unable to get
    // gateway and port from the service book, hence you have to 
    // specify it explicitly.
    // The connection goes through the mobile carrier's WAP gateway
    // to the internet to the server.
    //
    // Direct TCP
    // deviceside=true
    // deviceside=true;apn=XYZ
    // deviceside=true;apn=XYZ;TunnelAuthUsername=ABC;TunnelAuthPassword=DEF
    // This is like WAP but avoids the WAP gateway.
    // The connection goes through the mobile carrier (GPRS, 3G etc.)
    // to the internet to the server. 
    // The first variant requires you to set up the default APN of your 
    // carrier in the service book. The second variant specifies the APN
    // explicitely (XYZ), and the third variant specifies the authentication
    // (ABC, DEF) if the carrier's network requires authentication.
    //
    // BIS-B
    // some secret connection suffix
    // This is similar to MDS, except that it does not need a
    // corporate owned BES. Instead, it connects to a BIS-B server
    // provided by BlackBerry. Hence, different to MDS, this is available
    // to consumer customers as well, but only to people who have registered
    // to the Blackberry alliance program. Alliance members get their own
    // secret connection suffix.
    // The connection path goes through either through a wifi hotspot or through
    // the mobile carrier (GPRS, 3G etc.) to the Blackberry Infrastructure 
    // Network Operation Center (NOC). This path is encrypted and compressed.
    // The NOC then sends the data to BlackBerry's BIS-B Server
    // from where it is routed to the final destination.
    //
    // Additional connection suffixes to be appended for SSL:
    //
    // EndToEndRequired
    // The SSL connection goes from end server to device. This is secure.
    //
    // EndToEndDesired
    // The SSL connection goes from end server either to the device or
    // to the BES or BIS-B Server. The connection between those servers
    // and the device may be secured in a different way, not through SSL.
    
    
    /**
     * Returns the BlackBerry connection suffix for internet
     * connections. If null or empty string, the connection
     * suffix is automatically calculated on the fly.
     * The returned string excluded the leading semicolon.
     */
    public String getConnectionSuffix()
    {
    	return _connectionSuffix;
    }
    
    /**
     * Sets the connection suffix.
     * @param suffix The connection suffix.
     */
    public void setConnectionSuffix(String suffix)
    {
    	_connectionSuffix = suffix;
    }
        
    // --------------------------------------------------------------
    // Simple password encryption/decryption, bidirectional
    
    static final String encodeSequence = "0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz";
    static final String decodeSequence = "ACBDEFHGIJMZKLNOPQRSXTUVWYabdcxfghyjklemtnopiqrsuvwz3456789012";
    
    /**
     * Encode the password string.
     */
    private static String encode(String s)
    {
    	return code(s, encodeSequence, decodeSequence);
    }
    
    /**
     * Decode the password string.
     */
    private static String decode(String s)
    {
    	return code(s, decodeSequence, encodeSequence);
    }
    
    /**
     * Encode or decode a string.
     * @param src The source string.
     * @param seq1 The character sequence for the source string.
     * @param seq2 The character sequence for the target string.
     * @return The en/decoded string.
     */
    private static String code(String src, String seq1, String seq2)
    {
    	if (src == null) return null;
    	StringBuffer buf = new StringBuffer();
    	for (int i = 0; i < src.length(); i++) {
    		char c = src.charAt(i);
    		int idx = seq1.indexOf(c);
    		if (idx >= 0) {
    			buf.append(seq2.charAt(idx));
    		} else {
    			buf.append(c);
    		}
    	}
    	return buf.toString();
    }
}
