/*
 *  Licensed Materials - Property of IBM
 *  5725-G92 (C) Copyright IBM Corp. 2011, 2013. All Rights Reserved.
 *  US Government Users Restricted Rights - Use, duplication or
 *  disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 */

package com.ibm.appcenter.ui;

import com.ibm.appcenter.i18n.I18N;
import com.ibm.appcenter.i18n.I18NResource;
import com.ibm.appcenter.system.BlackBerrySystem;
import com.ibm.appcenter.ui.widget.EnhancedCheckboxField;
import com.ibm.appcenter.ui.widget.EnhancedEditField;
import com.ibm.appcenter.ui.widget.EnhancedLabelField;
import com.ibm.appcenter.ui.widget.EnhancedPasswordEditField;

import net.rim.blackberry.api.options.OptionsProvider;
import net.rim.device.api.ui.Field;
import net.rim.device.api.ui.FieldChangeListener;
import net.rim.device.api.ui.Graphics;
import net.rim.device.api.ui.Manager;
import net.rim.device.api.ui.component.BasicEditField;
import net.rim.device.api.ui.component.CheckboxField;
import net.rim.device.api.ui.component.EditField;
import net.rim.device.api.ui.component.LabelField;
import net.rim.device.api.ui.component.PasswordEditField;
import net.rim.device.api.ui.component.TextField;
import net.rim.device.api.ui.container.MainScreen;
import net.rim.device.api.ui.container.VerticalFieldManager;

/**
 * The options provider for the configuration.
 * This mechanism allows to see the application options
 * in the global blackberry options menu.
 * Note that this instance will be kept alive forever,
 * therefore it is important that it does not store big data.
 */
public class ConfigurationOptionsProvider 
implements OptionsProvider, I18NResource
{
	private ConfigurationData _conf;
	private TextField _userField;
	private TextField _passwordField;
	private TextField _serverField;
	private TextField _timeoutField;
	private TextField _connectionSuffixField;
	private CheckboxField _sslField;
	private CheckboxField _autoConnSuffixField;
	
	/**
	 * Constructor.
	 */
	public ConfigurationOptionsProvider()
	{		
	}
	
	public String getTitle()
	{
		return I18N.getString(ConfigurationOptionsTitle);
	}

	/**
	 * Populates the options screen visible in the options menu.
	 * @param mainScreen The main screen.
	 */
	public void populateMainScreen(MainScreen mainScreen)
	{	        
        mainScreen.add(getVerticalFieldManager(true));
	}

	/**
	 * Returns the vertical field manager that contains all options.
	 * It is scrollable.
	 * @param insideOptions If true, this is used inside the Blackberry Options
	 * panel. If false, it is used inside the application.
	 */
	public VerticalFieldManager getVerticalFieldManager(final boolean insideOptions)
	{	
		 // Get the current configuration.
        _conf = ConfigurationData.get();

        // create all important fields
        final long style = Field.FIELD_LEADING;
        
        // turn off all the "good" stuff such as spell checking, learning,
        // complex input, since that autocorrects the input in an unpleasent
        // manner
        final long editStyle = style | 
        		EditField.EDITABLE | 
        		EditField.NO_NEWLINE |
        		EditField.NON_SPELLCHECKABLE | 
        		EditField.NO_LEARNING |
    			EditField.NO_COMPLEX_INPUT; 
        
        final VerticalFieldManager vfm =
        		new VerticalFieldManager(Manager.VERTICAL_SCROLL) {
            protected void paintBackground( Graphics g )
            {
            	if (insideOptions) {
            		int oldColor = g.getColor();
            		try {
            			g.setColor( 0xDDDDDD );
            			g.fillRect( 0, getVerticalScroll(), getWidth(), getHeight() );
            		} finally {
            			g.setColor( oldColor );
            		}
            	} else {
            		super.paintBackground(g);
            	}
            }
        };
        
        _userField = new EnhancedEditField(
        		"",
        		_conf.getUser(),
        		I18N.getString(ConfigurationOptionsUserHint),
        		BasicEditField.DEFAULT_MAXCHARS,
        		editStyle);
        customize(_userField, insideOptions);

        _passwordField = new EnhancedPasswordEditField(
        		"",
        		_conf.getPassword(),
        		I18N.getString(ConfigurationOptionsPasswordHint),
        		BasicEditField.DEFAULT_MAXCHARS,
        		editStyle);
        customize(_passwordField, insideOptions);
        
        _serverField = new EnhancedEditField(
        		"",
        		_conf.getServer(),
        		I18N.getString(ConfigurationOptionsServerHint),
        		BasicEditField.DEFAULT_MAXCHARS,
        		editStyle);
        customize(_serverField, insideOptions);

        _timeoutField = new EnhancedEditField(
        		"",
        		"" + _conf.getTimeout(),
        		I18N.getString(ConfigurationOptionsTimeoutHint),
        		BasicEditField.DEFAULT_MAXCHARS,
        		editStyle | BasicEditField.FILTER_INTEGER);
        customize(_timeoutField, insideOptions);
        
        _connectionSuffixField = new EnhancedEditField(
        		"",
        		getConnectionSuffix(),
        		I18N.getString(ConfigurationOptionsConnectionSuffixHint),
        		BasicEditField.DEFAULT_MAXCHARS,
        		editStyle);
        customize(_connectionSuffixField, insideOptions);
        _connectionSuffixField.setEnabled(!isAutoConnectionSuffix());
        
        _autoConnSuffixField = getCheckboxField(
        		I18N.getString(ConfigurationOptionsAutoConnSuffix),
        		isAutoConnectionSuffix(),
        		insideOptions);
        
        _sslField = getCheckboxField(
        		I18N.getString(ConfigurationOptionsSSL),
        		_conf.isSSL(),
        		insideOptions);
        
        final CheckboxField showPasswordField = getCheckboxField(
        		I18N.getString(ConfigurationOptionsShowPassword), false, insideOptions);
        
        // add listener that replaces the password field if necessary
        // when switching "show password" on or off
        showPasswordField.setChangeListener(new FieldChangeListener() {

			public void fieldChanged(Field field, int context) {
				if (showPasswordField.getChecked() == true) {
					if (_passwordField instanceof PasswordEditField) {
						TextField newPasswordField =
							new EnhancedEditField(
				        		"",
				        		_passwordField.getText(),
				        		I18N.getString(ConfigurationOptionsPasswordHint),
				        		BasicEditField.DEFAULT_MAXCHARS,
				        		editStyle);
						customize(newPasswordField, insideOptions);
						vfm.replace(_passwordField, newPasswordField);
						_passwordField = newPasswordField;
					}
				} else {
					if (!(_passwordField instanceof PasswordEditField)) {
						TextField newPasswordField =
							new EnhancedPasswordEditField(
						        "",
						        _passwordField.getText(),
						        I18N.getString(ConfigurationOptionsPasswordHint),
						        BasicEditField.DEFAULT_MAXCHARS,
						        editStyle);
						customize(newPasswordField, insideOptions);
						vfm.replace(_passwordField, newPasswordField);
						_passwordField = newPasswordField;
					}
				}
			}
        	
        });
        
        // add listener that enables or disables the connection suffix field
        _autoConnSuffixField.setChangeListener(new FieldChangeListener() {

			public void fieldChanged(Field field, int context) {
				if (_autoConnSuffixField.getChecked() == true) {
					_connectionSuffixField.setEnabled(false);
					_connectionSuffixField.setText("");
					_connectionSuffixField.setText(BlackBerrySystem.getAutoConnectionSuffix());
					_conf.setConnectionSuffix("");
				} else {
					_connectionSuffixField.setEnabled(true);
					String s = _connectionSuffixField.getText().trim();
					if (s == null || s.length() == 0)
						s = "interface=wifi";
					_connectionSuffixField.setText("");
					_connectionSuffixField.setText(s);
					_conf.setConnectionSuffix(s);
				}
				
			}});
        
        // Put all together.
        // No Grid layout, since then we avoid running in i18n layout problems.
        // Most other Blackberry apps also don't use grid layout
        vfm.add(getLabelField(I18N.getString(ConfigurationOptionsUser), style, insideOptions));
        vfm.add(_userField);
        if (!insideOptions || UIUtil.isSavePasswordOnDevice()) {
        	vfm.add(getLabelField(I18N.getString(ConfigurationOptionsPassword), style, insideOptions));
        	vfm.add(_passwordField);
        	vfm.add(showPasswordField);
        }
        vfm.add(getLabelField("", 0, insideOptions));
        vfm.add(getLabelField(I18N.getString(ConfigurationOptionsServer), style, insideOptions));
        vfm.add(_serverField);
        if (insideOptions) {
        	vfm.add(getLabelField(
        			I18N.getString(ConfigurationOptionsTimeout), style, insideOptions));
        	vfm.add(_timeoutField);
        	vfm.add(getLabelField(
        			I18N.getString(ConfigurationOptionsConnectionSuffix), style, insideOptions));
        	vfm.add(_connectionSuffixField);
        	vfm.add(_autoConnSuffixField);
        } else {
        	_timeoutField = null;
        	_connectionSuffixField = null;
        }
        vfm.add(_sslField);
        
        return vfm;
	}
	
	private void customize(Field f, boolean insideOptions)
	{
		if (insideOptions) return;
		int p = UIUtil.getNormalFontHeightInPxl() / 4;
        // TRBL
        f.setPadding(p, p, p, p);
	}
	
	/**
	 * Returns a label field.
	 */
	private LabelField getLabelField(String s, long style, boolean insideOptions)
	{
		if (insideOptions)
			return new LabelField(s, style);
		else
			return new EnhancedLabelField(s, style);
	}
	
	/**
	 * Returns a checkbox field.
	 */
	private CheckboxField getCheckboxField(String s, boolean b, boolean insideOptions)
	{
		if (insideOptions)
			return new CheckboxField(s, b);
		else
			return new EnhancedCheckboxField(s, b);
	}
	
	/**
	 * Returns the effective connection suffix.
	 */
	private String getConnectionSuffix()
	{
		if (isAutoConnectionSuffix()) {
			return BlackBerrySystem.getAutoConnectionSuffix();
		} else {
			return _conf.getConnectionSuffix();
		}
	}
	
	/**
	 * Returns whether we are in automatic connection suffix mode.
	 */
	private boolean isAutoConnectionSuffix()
	{
		if (_conf.getConnectionSuffix() == null || _conf.getConnectionSuffix().length() == 0) {
			return true;
		} else {
			return false;
		}
	}
	
	/**
	 * Saves the options when the options screen is exited.
	 */
	public void save()
	{
		if (_conf == null) return;
		
		// Transfer the values from the UI controls to the configuration
		updateDataFromUI();

        // Write our changes back to the persistent store.
        _conf.save();

        // For garbage collecton.

        _userField = null;
        _passwordField = null;
        _serverField = null;
        _sslField = null;
        _connectionSuffixField = null;
        _conf = null;
	}
	
	/**
	 * Saves the data. This can be used when the
	 * options screen is kept.
	 */
	void saveAndKeep()
	{
		if (_conf == null) return;
		
		// Transfer the values from the UI controls to the configuration
		updateDataFromUI();

        // Write our changes back to the persistent store.
        _conf.save();
	}
	
	/**
	 * Returns the configuration data if any.
	 * Must be called before {@link #save}, otherwise
	 * it returns always null.
	 */
	ConfigurationData getData()
	{
		updateDataFromUI();
		return _conf;
	}
	
	/**
	 * Transfer the data from the UI controls to the configuration.
	 */
	void updateDataFromUI()
	{
		if (_conf == null) return;
		
		if (_userField != null) {
			_conf.setUser(_userField.getText().trim());
		}
		if (_passwordField != null) {
			_conf.setPassword(_passwordField.getText().trim());
		}
		if (_serverField != null) {
			_conf.setServer(_serverField.getText().trim());
		}
		if (_timeoutField != null) {
			try {
			_conf.setTimeout(Integer.parseInt(_timeoutField.getText().trim()));
			} catch (Exception ex) {
			}
		}
		if (_connectionSuffixField != null) {
			if (_autoConnSuffixField.getChecked()) {
				_conf.setConnectionSuffix("");
			} else {
				_conf.setConnectionSuffix(_connectionSuffixField.getText().trim());
			}
		}
		if (_sslField != null) {
			_conf.setSSL(_sslField.getChecked());
		}	
	}
}
