/*
 *  Licensed Materials - Property of IBM
 *  5725-G92 (C) Copyright IBM Corp. 2011, 2013. All Rights Reserved.
 *  US Government Users Restricted Rights - Use, duplication or
 *  disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 */

package com.ibm.appcenter.ui;

import org.json.me.JSONException;
import org.json.me.JSONObject;

import net.rim.device.api.system.Display;
import net.rim.device.api.ui.Field;
import net.rim.device.api.ui.FieldChangeListener;
import net.rim.device.api.ui.Manager;
import net.rim.device.api.ui.UiApplication;
import net.rim.device.api.ui.component.ButtonField;
import net.rim.device.api.ui.component.Dialog;
import net.rim.device.api.ui.component.LabelField;
import net.rim.device.api.ui.component.StandardTitleBar;
import net.rim.device.api.ui.component.TitleBar;
import net.rim.device.api.ui.container.MainScreen;
import net.rim.device.api.ui.container.VerticalFieldManager;

import com.ibm.appcenter.artifacts.AppVersion;
import com.ibm.appcenter.connection.FeedbackUtil;
import com.ibm.appcenter.connection.JSONRESTCallContinuation;
import com.ibm.appcenter.i18n.I18N;
import com.ibm.appcenter.ui.widget.EnhancedAutoTextEditField;
import com.ibm.appcenter.ui.widget.EnhancedLabelField;
import com.ibm.appcenter.ui.widget.RatingEditField;
import com.ibm.appcenter.ui.widget.advancedui.HorizontalButtonFieldSet;

/**
 * This screen allows the submission of a rating with comments.
 */
public class FeedbackSubmissionScreen extends AppCenterScreen
{
	private AppPropertyScreen _parent;
	private AppVersion _app;
	private ButtonField _cancelButton;
    private ButtonField _submitButton;
    private RatingEditField _ratingField;
    private EnhancedAutoTextEditField _commentField;
    
	private String[] _ratingNames = new String[] {
		"", // for 0 star rating
		I18N.getString(FeedbackSubmissionScreen1StarExplanation),
		I18N.getString(FeedbackSubmissionScreen2StarExplanation),
		I18N.getString(FeedbackSubmissionScreen3StarExplanation),
		I18N.getString(FeedbackSubmissionScreen4StarExplanation),
		I18N.getString(FeedbackSubmissionScreen5StarExplanation)
	};
	
	/**
	 * Constructor.
	 * @param app The app that can be rated.
	 */
	public FeedbackSubmissionScreen(AppVersion app, AppPropertyScreen parent)
	{
    	super(MainScreen.NO_SYSTEM_MENU_ITEMS | Manager.NO_VERTICAL_SCROLL);

    	log("Constructor, for " + app.getId() + " " + app.getVersion());
    	
    	_app = app;
    	_parent = parent;
    	
    	// set the title bar
        TitleBar bar = getTitleBar();
        if (bar instanceof StandardTitleBar) {
        	StandardTitleBar sbar = (StandardTitleBar)bar;
        	sbar.addSignalIndicator();
        	sbar.addTitle(I18N.getString(FeedbackSubmissionScreenTitle));
        } else {
        	StandardTitleBar sbar = new StandardTitleBar();
        	sbar.addSignalIndicator();
        	sbar.addTitle(I18N.getString(FeedbackSubmissionScreenTitle));
        	setTitleBar(sbar);
        }
        
        // allocate a scrollable manager
    	VerticalFieldManager mgr = new VerticalFieldManager(Manager.VERTICAL_SCROLL);
    	add(mgr);
    	
    	// allocate the submit and cancel button
        _cancelButton = new ButtonField(I18N.getString(FeedbackSubmissionScreenCancelButtonTitle));
        _submitButton = new ButtonField(I18N.getString(FeedbackSubmissionScreenSubmitButtonTitle));
        
        // add explanation
    	int margin = 3 * UIUtil.getNormalXAdvanceInPxl() / 2;
    	EnhancedLabelField label = new EnhancedLabelField(I18N.getString(FeedbackSubmissionScreenExplanation));
    	label.setMargin(margin, margin, margin, margin);
    	mgr.add(label);
    	
    	// add rating and label of rating
    	int size = 3 * UIUtil.getLargeBoldFontHeightInPxl()/2;
    	
    	if (size * 7 + 14 > Display.getWidth()) {
    		size = (Display.getWidth() - 14) / 7;
    	}
    	
    	
        final LabelField label1 = new EnhancedLabelField("", Field.FIELD_HCENTER);
        
        _ratingField = new RatingEditField(size, Field.FIELD_HCENTER) {
        	protected void onRating(int val) {
        		if (val < 0 || val > 5) return;
        		String r = _ratingNames[val];
        		label1.setText(r);
        		validateSubmitButton();
        	}
        };
        
        mgr.add(_ratingField);
        mgr.add(label1);
        
        // add the comment field
        String hint = I18N.getString(FeedbackSubmissionScreenCommentHint);
        _commentField = new EnhancedAutoTextEditField("", "", hint, 2000, 0);
        _commentField.setMinHeight(UIUtil.getNormalFontHeightInPxl() * 4);
        int padding = UIUtil.getNormalFontHeightInPxl() / 4;
        _commentField.setPadding(padding, padding, padding, padding);
        _commentField.setMargin(margin, margin, margin, margin);
        _commentField.setChangeListener(new FieldChangeListener() {
			public void fieldChanged(Field field, int context) {
				validateSubmitButton();
			}});
        mgr.add(_commentField);
        
        // configure and add the cancel and submit button
        HorizontalButtonFieldSet buttonSet = new HorizontalButtonFieldSet();
        buttonSet.add(_cancelButton);
        buttonSet.add(_submitButton);
        buttonSet.setMargin(margin, margin, margin, margin);
        _cancelButton.setMargin(0, margin/2, 0, 0);
        _submitButton.setMargin(0, 0, 0, margin/2);
        mgr.add(buttonSet);
        
        _cancelButton.setChangeListener(new FieldChangeListener() {
			public void fieldChanged(Field field, int context) {
				onBack();				
			}});
        _submitButton.setChangeListener(new FieldChangeListener() {
			public void fieldChanged(Field field, int context) {
				FeedbackSubmissionScreen.this.setDirty(false);
				onSubmit(_ratingField.getRating(), _commentField.getText());				
			}});
        
        // check whether the submit button must be enabled
        validateSubmitButton();
	}

	/**
	 * Disables or enabled the submit button as needed. 
	 */
	private void validateSubmitButton()
	{
		_submitButton.setEnabled(
				_ratingField.getRating() > 0 &&
				_commentField.getText() != null &&
				_commentField.getText().length() > 0);
	}
	
	/**
	 * Called when the back or cancel button is pressed.
	 */
	public void onBack()
	{
		if (!isDirty() || onSavePrompt()) {
			setDirty(false);
			getRESTCallBatch().stop();
			UiApplication.getUiApplication().popScreen(this);
		}
	}
	
	/**
	 * Called when the submit button is pressed.
	 * @param rating  The rating.
	 * @param comment The comment.
	 */
	public void onSubmit(final int rating, String comment)
	{
		log("onSubmit " + rating + " " + comment);
		try {
			FeedbackUtil.submitFeedback(
					_app.getId(),
					_app.getVersion(),
					rating,
					comment,				
					getRESTCallBatch(),
					new JSONRESTCallContinuation() {

						public void onSuccess(JSONObject response,
								String contentType, byte[] bytes) {
							log("Submitted feedback successfully!");
							_app.recalcRatingOnSubmission(rating);
							UIUtil.showStatus(I18N.getString(FeedbackSubmissionScreenSubmissionSuccess));
							_parent.setFeedbacksNeedRefresh(true);
							onBack();
						}

						public void onFailure(Exception ex, int status,
								JSONObject response, String contentType,
								byte[] bytes) {
							log("Failed to submit feedback! Status=" + status);
							UIUtil.showConnectionStatus(status, ex);
						}

						public void onCancel() {
						}});
		} catch (JSONException e) {
			// this should never happen, since our JSON is so simple
			log("Exception in onSubmit", e);
		}
	}

	/**
	 * Returns the screen name for logging.
	 */
	public String getScreenName()
	{
		return "FeeSubSCR"; // no I18N
	}
	
    /**
     * Called when the screen was edited and closed.
     */
    protected boolean onSave()
    {
    	// nothing to save
    	return true;
    }
    
    /**
     * Called when the screen was edited, to query whether to close
     * the screen.
     * @return true when we can close the screen, false otherwise.
     */
    protected boolean onSavePrompt()
    {
    	if (_commentField.getText() == null || _commentField.getText().length() == 0) {
    		return true;
    	}
    	int res = Dialog.ask(
    			Dialog.D_YES_NO,
    			I18N.getString(FeedbackSubmissionScreenWarnAndAskYesOrNoWhenClosing),
    			Dialog.NO);
    	if (res == Dialog.NO)
    		return false;
    	else
    		return true;
    }
}
