/*
 *  Licensed Materials - Property of IBM
 *  5725-G92 (C) Copyright IBM Corp. 2011, 2013. All Rights Reserved.
 *  US Government Users Restricted Rights - Use, duplication or
 *  disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 */

package com.ibm.appcenter.ui;

import net.rim.device.api.system.Bitmap;


/**
 * The cache for the application icons.
 */
public class IconCache
{
	private static IconCache _instance = null;
	
	/**
	 * Return value when the bitmap does not exist.
	 */
    public static final Bitmap NO_BITMAP = new Bitmap(1,1);
    
    private int _cacheSize = 40;
    private IconBitmapCacheEntry[] _iconBitmapCache;
    
    
	/**
	 * Constructor.
	 */
	private IconCache()
	{
		this(40);
	}
	
	/**
	 * Constructor.
	 * @param size The number of entries in the cache.
	 */
	public IconCache(int size)
	{
		_cacheSize = size;
		_iconBitmapCache = new IconBitmapCacheEntry[_cacheSize];
	}
	
	/**
	 * Returns the default instance of the icon cache.
	 */
	public static IconCache getInstance()
	{
		if (_instance == null) {
			_instance = new IconCache();
		}
		return _instance;
	}

	/**
	 * Returns the width and height of square icons
	 * in the default inctance of the icon cache.
	 */
	public static int getDefaultIconDimension()
	{
		return UIUtil.getBoldFontHeightInPxl() + 2 * UIUtil.getMicroFontHeightInPxl();
	}
	
    /**
     * Clear the cache.
     */
    public void clear()
    {
    	for (int i = 0; i < _cacheSize; i++) {
    		_iconBitmapCache[i] = null;
    	}
    }
    
    /**
     * Returns the cache entry for an icon.
     * @param url The url of the icon.
     * @return The cache entry, or null.
     */
    private IconBitmapCacheEntry getIconBitmapCacheEntry(String url)
    {
    	if (url == null)
    		return null;
    	for (int i = 0; i < _cacheSize; i++) {
    		if (_iconBitmapCache[i] == null) continue;
    		if (url.equals(_iconBitmapCache[i].url)) {
    			_iconBitmapCache[i].timestamp = System.currentTimeMillis();
    			return _iconBitmapCache[i];
    		}
    	}
    	return null;
    }
    
    /**
     * Returns the bitmap for an icon from the cache.
     * @param url The url of the icon.
     * @return The bitmap. Returns NO_BITMAP if the entry exists
     *    in the cache but the bitmap is null.
     */
    public Bitmap get(String url)
    {
    	if (url == null)
    		return NO_BITMAP;
    	IconBitmapCacheEntry entry = getIconBitmapCacheEntry(url);
    	if (entry != null)
    		return entry.bitmap;
    	return null;
    }
    
    /**
     * Stores a bitmap for an icon in the cache.
     * @param url The url of the icon.
     * @param bm The bitmap to be stored.
     */
    public void put(String url, Bitmap bm)
    {
    	IconBitmapCacheEntry entry = getIconBitmapCacheEntry(url);
    	if (entry != null) {
    		entry.bitmap = (bm == null ? NO_BITMAP : bm);
    	} else {
    		// search free entry, or oldest entry
    		long oldestTimeStamp = Long.MAX_VALUE;
    		int oldestIndex = 0;
    		for (int i = 0; i < _cacheSize; i++) {
    			entry = _iconBitmapCache[i];
    			if (entry == null) {
    				entry = new IconBitmapCacheEntry();
    				entry.url = url;
    				entry.bitmap = (bm == null ? NO_BITMAP : bm);
    				entry.timestamp = System.currentTimeMillis();
    				_iconBitmapCache[i] = entry;
    				return;
    			} else {
    				if (entry.timestamp < oldestTimeStamp) {
    					oldestTimeStamp = entry.timestamp;
    					oldestIndex = i;
    				}
    			}
    		}
    		entry = _iconBitmapCache[oldestIndex];
    		entry.url = url;
    		entry.bitmap = (bm == null ? NO_BITMAP : bm);
    		entry.timestamp = System.currentTimeMillis();
    	}
    }
    
    // -------------------------------------------------------------------
    
    /**
     * Class for the cache entry.
     */
    private static class IconBitmapCacheEntry
    {
    	String url;
    	Bitmap bitmap;
    	long timestamp;
    }
}
