/*
 *  Licensed Materials - Property of IBM
 *  5725-G92 (C) Copyright IBM Corp. 2011, 2013. All Rights Reserved.
 *  US Government Users Restricted Rights - Use, duplication or
 *  disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 */

package com.ibm.appcenter.ui;

import com.ibm.appcenter.connection.ConnectionUtil;
import com.ibm.appcenter.i18n.I18N;
import com.ibm.appcenter.i18n.I18NResource;
import com.ibm.appcenter.util.Log;
import com.ibm.appcenter.util.Properties;

import net.rim.device.api.system.Bitmap;
import net.rim.device.api.system.Display;
import net.rim.device.api.ui.Color;
import net.rim.device.api.ui.Field;
import net.rim.device.api.ui.Font;
import net.rim.device.api.ui.Graphics;
import net.rim.device.api.ui.Screen;
import net.rim.device.api.ui.Ui;
import net.rim.device.api.ui.UiApplication;
import net.rim.device.api.ui.component.Dialog;
import net.rim.device.api.ui.component.Status;
import net.rim.device.api.ui.decor.Background;
import net.rim.device.api.ui.decor.BackgroundFactory;
import net.rim.device.cldc.io.ssl.TLSIOException;

/**
 * Common static utilities for the UI.
 */
public class UIUtil {

	// the properties
	private static Properties _properties = null;
	
	// left to right or right to left layout
	private static boolean _isRTL = false;
	private static boolean _isRTLInitialized = false;
	
	// various font factors
	private static double _largeFontFactor = 1.0;
	private static double _boldFontFactor = 1.0;
	private static double _miniFontFactor = 0.9;
	private static double _microFontFactor = 0.9;
	private static double _toolbarIconFactor = 1.1;
	
	// various fonts that we commonly use
    private static Font _normalFont = Font.getDefault();
    private static Font _italicFont = _normalFont.derive(Font.ITALIC);
    private static Font _boldFont = _normalFont.derive(Font.EXTRA_BOLD,
										getBoldFontSize(_normalFont),
										Ui.UNITS_pt);
    private static Font _largeBoldFont = _normalFont.derive(Font.BOLD,
										getLargeFontSize(_normalFont),
										Ui.UNITS_pt);
    private static Font _miniFont = _normalFont.derive(_normalFont.getStyle(),
            							getMiniFontSize(_normalFont),
            							Ui.UNITS_pt);
    private static Font _microFont = _normalFont.derive(_normalFont.getStyle(),
    						             getMicroFontSize(_normalFont),
    						             Ui.UNITS_pt);
    
    // various spacing parameters that we commonly use
    private static int _normalFontHeightInPxl = _normalFont.getHeight();
    private static int _italicFontHeightInPxl = _italicFont.getHeight();
    private static int _boldFontHeightInPxl = _boldFont.getHeight();
    private static int _largeBoldFontHeightInPxl = _largeBoldFont.getHeight();
    private static int _miniFontHeightInPxl = _miniFont.getHeight();
    private static int _microFontHeightInPxl = _microFont.getHeight();
    private static int _normalXAdvanceInPxl = _normalFont.getAdvance("x");
    
    // various colors
    private static int _normalTextColor = Color.BLACK;
    private static int _editTextColor = Color.BLACK;
    private static int _disabledTextColor = 0x00808080;
    private static int _hintTextColor = 0x00a0a0a0; 
    private static int _editBackgroundColor = 0xf6f6f6;
    private static int _buttonSetBackgroundColor = 0xa0a0a0;
    private static int _buttonSetBorderColor = 0x000000;
    
    // various backgrounds
    private static Background _normalBackground =
    		BackgroundFactory.createSolidBackground(0xffffff);
	private static Background _focusBackground = 
		BackgroundFactory.createLinearGradientBackground(
				0x00207CFE, 0x00207CFE, 0x00074CAC, 0x00074CAC);
	private static Background _paneHeaderBackground =
		BackgroundFactory.createSolidBackground(0x00a0a0a0);
	private static Background _topHeaderBackground =
			BackgroundFactory.createSolidBackground(0x00c0c0c0);
	
	// whether the normal background is darker than the normal text color
	private static boolean _isLightOnDark = false;
	
	// whether the top header background is dark and needs light icons
	private static boolean _topHeaderNeedsLightIcons = false;
	
	// Whether we allow searching for ratings, versions etc.
	// (besides application name, which is always there).
	//In this case, we show a button next to the search field
	// that allows to set the search scope.
	private static boolean _searchScopeEnabled = false;

	// Whether ascending and descending sort is allowed.
	// It is only useful to enable this when also 
	// _adaptAppCatalogInfoLineToSorting is enabled, since
	// otherwise there is no visual feedback whether the sort
	// is ascending or descending, and that can be confusing.
	private static boolean _allowAscendingDescendingSort = false;
	
	// Whether the info line in the application catalog (that is
	// normally only showing the version) can show popularity 
	// and updates. If enabled, when you change the sorting of
	// the application catalog, the version info gets replaced
	// by the popularity or by the update date. 
	private static boolean _adaptAppCatalogInfoLineToSorting = false;
	
	// Whether selection of a specific version is allowed.
	// If disabled, it is only possible to install the latest
	// version of an application. If enabled, it is possible
	// to revert to any older version of an application, that is,
	// to specify precisely which version to install.
	private static boolean _allowVersionSelection = true;
	
	// Whether the application is allowed to save the password
	// on the device. When not saved, restarting the App Center
	// will always ask for the password.
	private static boolean _savePasswordOnDevice = true;
	
	// ------------------------------------------------------------------------

	/**
	 * Private constructor since this is a singleton.
	 */
	private UIUtil() {}
	
	/**
	 * Evaluates the properties.
	 */
	public static void evaluateUIProperties(Class clazz)
	{
		if (_properties != null) return;
		_properties = new Properties(clazz, "/appcenter.properties");
		
		if (Display.getNumColors() <= 65536) {
			// display color numbers to small for filling large backgrounds with
			// gradient. Better use uniform background
			_normalBackground =
					_properties.getBackground("normalBackgroundLQ", _normalBackground);
		} else {
			_normalBackground =
					_properties.getBackground("normalBackgroundHQ", _normalBackground);
		}
		
		_paneHeaderBackground =
				_properties.getBackground("scrollPaneHeaderBackground", _paneHeaderBackground);
		_topHeaderBackground =
				_properties.getBackground("topHeaderBackground", _topHeaderBackground);

		_topHeaderNeedsLightIcons = _properties.getBoolean("topHeaderLightIcons", _topHeaderNeedsLightIcons);
		
	    _normalTextColor =
	    		_properties.getColor("normalTextColor", _normalTextColor);
	    _editTextColor =
	    		_properties.getColor("editTextColor", _editTextColor);
	    _disabledTextColor =
	    		_properties.getColor("disabledTextColor", _disabledTextColor);
	    _hintTextColor =
	    		_properties.getColor("hintTextColor", _hintTextColor);
	    _editBackgroundColor =
	    		_properties.getColor("editBackgroundColor", _editBackgroundColor);
	    _buttonSetBackgroundColor =
	    		_properties.getColor("buttonSetBackgroundColor", _buttonSetBackgroundColor);
	    _buttonSetBorderColor =
	    		_properties.getColor("buttonSetBorderColor", _buttonSetBorderColor);
	    
	    _isLightOnDark = _properties.getBoolean("lightTextOnDarkBackground", false);
	    
		_largeFontFactor   = _properties.getDouble("largeFontFactor",   _largeFontFactor);
		_boldFontFactor    = _properties.getDouble("boldFontFactor",    _boldFontFactor);
		_miniFontFactor    = _properties.getDouble("miniFontFactor",    _miniFontFactor);
		_microFontFactor   = _properties.getDouble("microFontFactor",   _microFontFactor);
		_toolbarIconFactor = _properties.getDouble("toolbarIconFactor", _toolbarIconFactor);
		
	    _boldFont = _normalFont.derive(Font.EXTRA_BOLD, 
	    							   getBoldFontSize(_normalFont),
	    							   Ui.UNITS_pt);
	    _largeBoldFont = _normalFont.derive(Font.BOLD,
											getLargeFontSize(_normalFont),
											Ui.UNITS_pt);
	    _miniFont = _normalFont.derive(_normalFont.getStyle(),
									   getMiniFontSize(_normalFont),
									   Ui.UNITS_pt);
	    _microFont = _normalFont.derive(_normalFont.getStyle(),
	             						getMicroFontSize(_normalFont),
	             						Ui.UNITS_pt);
	    
	    _boldFontHeightInPxl = _boldFont.getHeight();
	    _largeBoldFontHeightInPxl = _largeBoldFont.getHeight();
	    _miniFontHeightInPxl = _miniFont.getHeight();
	    _microFontHeightInPxl = _microFont.getHeight();
	    
		_searchScopeEnabled = 
				_properties.getBoolean("searchScopeEnabled", _searchScopeEnabled);
		_allowAscendingDescendingSort =
				_properties.getBoolean("allowAscendingDescendingSort", _allowAscendingDescendingSort);
 		_adaptAppCatalogInfoLineToSorting = 
 				_properties.getBoolean("adaptAppCatalogInfoLineToSorting", _adaptAppCatalogInfoLineToSorting);
 		_allowVersionSelection =
 				_properties.getBoolean("allowVersionSelection", _allowVersionSelection);
 		_savePasswordOnDevice =
 				_properties.getBoolean("savePasswordOnDevice", _savePasswordOnDevice);
	}
	
	/**
	 * Returns whether the background is dark and the text is light.
	 */
	public static boolean isLightOnDark()
	{
		return _isLightOnDark;
	}
	
	/**
	 * Returns the standard normal font.
	 */
	public static Font getNormalFont()
	{
		return _normalFont;
	}
	
	/**
	 * Returns the standard italic font.
	 */
	public static Font getItalicFont()
	{
		return _italicFont;
	}
	
	/**
	 * Returns the standard bold font.
	 */
	public static Font getBoldFont()
	{
		return _normalFont;
	}
	
	/**
	 * Returns the large bold font.
	 */
	public static Font getLargeBoldFont()
	{
		return _largeBoldFont;
	}
	
	/**
	 * Returns the standard mini font.
	 */
	public static Font getMiniFont()
	{
		return _miniFont;
	}
	
	/**
	 * Returns the standard micro font.
	 */
	public static Font getMicroFont()
	{
		return _microFont;
	}
	
	/**
	 * Returns the font height (the high of a character M)
	 * for the normal font, in screen pixel.
	 */
	public static int getNormalFontHeightInPxl()
	{
		return _normalFontHeightInPxl;
	}
	
	/**
	 * Returns the font height (the high of a character M)
	 * for the italic font, in screen pixel.
	 */
	public static int getItalicFontHeightInPxl()
	{
		return _italicFontHeightInPxl;
	}
	
	/**
	 * Returns the font height (the high of a character M)
	 * for the bold font, in screen pixel.
	 */
	public static int getBoldFontHeightInPxl()
	{
		return _boldFontHeightInPxl;
	}

	/**
	 * Returns the font height (the high of a character M)
	 * for the large bold font, in screen pixel.
	 */
	public static int getLargeBoldFontHeightInPxl()
	{
		return _largeBoldFontHeightInPxl;
	}
	
	/**
	 * Returns the font height (the high of a character M)
	 * for the mini font, in screen pixel.
	 */
	public static int getMiniFontHeightInPxl()
	{
		return _miniFontHeightInPxl;
	}
	
	/**
	 * Returns the font height (the high of a character M)
	 * for the micro font, in screen pixel.
	 */
	public static int getMicroFontHeightInPxl()
	{
		return _microFontHeightInPxl;
	}
	
	/**
	 * Returns the horizontal space of the character x
	 * for the normal font, in screen pixel.
	 * Many horizontal spacings use this value as base.
	 */
	public static int getNormalXAdvanceInPxl()
	{
		return _normalXAdvanceInPxl;
	}
	
	/**
	 * Returns the height of icons in the toolbat in screen pixel.
	 */
	public static int getToolbarIconSizeInPxl()
	{
		return (int)(_toolbarIconFactor * _normalFontHeightInPxl);
	}
	
	/**
	 * Returns the color for normal text.
	 */
	public static int getNormalTextColor()
	{
		return _normalTextColor;
	}
	
	/**
	 * Returns the color for edit field text.
	 */
	public static int getEditTextColor()
	{
		return _editTextColor;
	}
	
	/**
	 * Returns the color for text in disabled edit fields.
	 */
	public static int getDisabledTextColor()
	{
		return _disabledTextColor;
	}
	
	/**
	 * Returns the color for hints in edit fields.
	 */
	public static int getHintTextColor()
	{
		return _hintTextColor;
	}
	
	/**
	 * Returns the background color of editable text fields.
	 */
	public static int getEditBackgroundColor()
	{
		return _editBackgroundColor;
	}
	
	/**
	 * Returns the background color of the button set on the start screen.
	 */
	public static int getButtonSetBackgroundColor()
	{
		return _buttonSetBackgroundColor;
	}
	
	/**
	 * Returns the border color of the button set on the start screen.
	 */
	public static int getButtonSetBorderColor()
	{
		return _buttonSetBorderColor;
	}
    
	/**
	 * Returns the normal background.
	 */
	public static Background getNormalBackground()
	{
		return _normalBackground;
	}
	
	/**
	 * Returns the standard focus background (bluish).
	 * This is fixed, since it is set by the OS.
	 */
	public static Background getFocusBackground()
	{
		return _focusBackground;
	}
		
	/**
	 * Returns the background for the non-focussed horizontal
	 * scroll pane header.
	 */
	public static Background getScrollPaneHeaderBackground()
	{
		return _paneHeaderBackground;
	}
		
	/**
	 * Returns the background for the top header which has
	 * the search field and the refresh icon.
	 */
	public static Background getTopHeaderBackground()
	{
		return _topHeaderBackground;
	}

	/**
	 * Returns whether the top header background is dark
	 * and the icons are light.
	 */
	public static boolean isTopHeaderNeedsLightIcons()
	{
		return _topHeaderNeedsLightIcons;
	}
	
    /**
     * Returns the font size in pts of the large font relative to
     * the input font.
     * @param font The input font, which is assumed to have normal size.
     * @return The corresponding large font size in pt.
     */
    private static int getLargeFontSize(Font font)
    {
    	int size = font.getHeight(Ui.UNITS_pt);
    	return (int)(size * _largeFontFactor);
    }
    
    /**
     * Returns the font size in pts of the bold font relative to
     * the input font.
     * @param font The input font, which is assumed to have normal size.
     * @return The corresponding bold font size in pt.
     */
    private static int getBoldFontSize(Font font)
    {
    	int size = font.getHeight(Ui.UNITS_pt);
    	return (int)(size * _boldFontFactor);
    }
    
    /**
     * Returns the font size in pts of the small font relative to
     * the input font.
     * @param font The input font, which is assumed to have normal size.
     * @return The corresponding small font size in pt.
     */
    private static int getMiniFontSize(Font font)
    {
    	int size = font.getHeight(Ui.UNITS_pt);
    	return (int)(size * _miniFontFactor);
    }
    
    /**
     * Returns the font size in pts of the smallest font relative to
     * the input font.
     * @param font The input font, which is assumed to have normal size.
     * @return The corresponding smallest font size in pt.
     */
    private static int getMicroFontSize(Font font)
    {
    	int size = font.getHeight(Ui.UNITS_pt);
    	return (int)(size * _microFontFactor);
    }
    
	/**
	 * Returns a scaled bitmap from a resource name.
	 * @param bitmapName The name of the bitmap resource.
	 * @param w The desired width;
	 * @param h The desired height;
	 * @return The scaled bitmap;
	 */
	public static Bitmap getScaledBitmap(String bitmapName, int w, int h)
	{
		return getScaledBitmap(Bitmap.getBitmapResource(bitmapName), w, h);
	}

    /**
     * Returns a scaled bitmap if the height of the original 
     * bitmap is too small or too large. It keeps the aspect
     * ratio of the bitmap.
	 * @param bitmapName The name of the bitmap resource.
	 * @param h The desired height;
	 * @return The scaled bitmap;
	 */
	public static Bitmap getScaledBitmap(String bitmapName, int h)
	{
		return getScaledBitmap(Bitmap.getBitmapResource(bitmapName), h);
	}
	    
    /**
     * Returns a scaled bitmap if the height of the original 
     * bitmap is too small or too large. It keeps the aspect
     * ratio of the bitmap.
     * @param orig The original bitmap.
     * @param h The desired height;
     * @return The scaled bitmap;
     */
    public static Bitmap getScaledBitmap(Bitmap orig, int h)
    {
		if (orig == null)
			return null;
		if (orig.getHeight() == h)
			return orig;
		int w = orig.getWidth() * h / orig.getHeight();
		return getScaledBitmap(orig, w, h);
    }
    
    /**
     * Returns a scaled bitmap if the original bitmap is
     * too small or too large.
     * @param orig The original bitmap.
     * @param w The desired width.
     * @param h The desired height.
     * @return The scaled bitmap.
     */
    public static Bitmap getScaledBitmap(Bitmap orig, int w, int h)
    {
    	if (orig == null)
    		return null;
    	if (orig.getWidth() == w && orig.getHeight() == h)
    		return orig;
    	// In 7.0, this would work, but in 6.0, this loses transparency
    	// Bitmap bitmap = new Bitmap(orig.getType(), w, h);
    	// orig.scaleInto(bitmap, Bitmap.FILTER_LANCZOS);
    	
    	Bitmap bitmap = scaleTransparentBitmap(orig, w, h, Bitmap.FILTER_LANCZOS, Bitmap.SCALE_TO_FIT);
    	
    	return bitmap;
    }
    
    /**
     * Resizes a bitmap with an alpha channel (transparency) without the artifacts introduced
     *   by <code>scaleInto()</code>.
     *
     * @param bmpSrc        Source Bitmap
     * @param nWidth        New Width
     * @param nHeight       New Height
     * @param nFilterType   Filter quality to use. Can be <code>Bitmap.FILTER_LANCZOS</code>,
     *                           <code>Bitmap.FILTER_BILINEAR</code> or
     *                           <code>Bitmap.FILTER_BOX</code>.
     * @param nAspectRatio  Specifies how the picture is resized. Can be
     *                           <code>Bitmap.SCALE_TO_FIT</code>,
     *                           <code>Bitmap.SCALE_TO_FILL</code> or
     *                           <code>Bitmap.SCALE_STRETCH</code>.
     * @return              The resized Bitmap in a new object.
     */
    public static Bitmap scaleTransparentBitmap(Bitmap bmpSrc, int nWidth, int nHeight, int nFilterType, int nAspectRatio)
    {
        if(bmpSrc== null)
            return null;

        //Get the original dimensions of the bitmap
        int nOriginWidth = bmpSrc.getWidth();
        int nOriginHeight = bmpSrc.getHeight();
        if(nWidth == nOriginWidth && nHeight == nOriginHeight)
            return bmpSrc;

        //Prepare a drawing bitmap and graphic object
        Bitmap bmpOrigin = new Bitmap(nOriginWidth, nOriginHeight);
        Graphics graph = Graphics.create(bmpOrigin);

        //Create a line of transparent pixels for later use
        int[] aEmptyLine = new int[nWidth];
        for(int x = 0; x < nWidth; x++)
            aEmptyLine[x] = 0x00000000;
        //Create two scaled bitmaps
        Bitmap[] bmpScaled = new Bitmap[2];
        for(int i = 0; i < 2; i++)
        {
            //Draw the bitmap on a white background first, then on a black background
            graph.setColor((i == 0) ? Color.WHITE : Color.BLACK);
            graph.fillRect(0, 0, nOriginWidth, nOriginHeight);
            graph.drawBitmap(0, 0, nOriginWidth, nOriginHeight, bmpSrc, 0, 0);

            //Create a new bitmap with the desired size
            bmpScaled[i] = new Bitmap(nWidth, nHeight);
            if(nAspectRatio == Bitmap.SCALE_TO_FIT)
            {
                //Set the alpha channel of all pixels to 0 to ensure transparency is
                //applied around the picture, if needed by the transformation
                for(int y = 0; y < nHeight; y++)
                    bmpScaled[i].setARGB(aEmptyLine, 0, nWidth, 0, y, nWidth, 1);
            }

            //Scale the bitmap
            bmpOrigin.scaleInto(bmpScaled[i], nFilterType, nAspectRatio);
        }

        //Prepare objects for final iteration
        Bitmap bmpFinal = bmpScaled[0];
        int[][] aPixelLine = new int[2][nWidth];

        //Iterate every line of the two scaled bitmaps
        for(int y = 0; y < nHeight; y++)
        {
            bmpScaled[0].getARGB(aPixelLine[0], 0, nWidth, 0, y, nWidth, 1);
            bmpScaled[1].getARGB(aPixelLine[1], 0, nWidth, 0, y, nWidth, 1);

            //Check every pixel one by one
            for(int x = 0; x < nWidth; x++)
            {
                //If the pixel was untouched (alpha channel still at 0), keep it transparent
                if(((aPixelLine[0][x] >> 24) & 0xff) == 0)
                    aPixelLine[0][x] = 0x00000000;
                else
                {
                    //Compute the alpha value based on the difference of intensity
                    //in the red channel
                    int nAlpha = ((aPixelLine[1][x] >> 16) & 0xff) -
                                    ((aPixelLine[0][x] >> 16) & 0xff) + 255;
                    if(nAlpha == 0)
                        aPixelLine[0][x] = 0x00000000; //Completely transparent
                    else if(nAlpha >= 255)
                        aPixelLine[0][x] |= 0xff000000; //Completely opaque
                    else
                    {
                        //Compute the value of the each channel one by one
                        int nRed = ((aPixelLine[0][x] >> 16 ) & 0xff);
                        int nGreen = ((aPixelLine[0][x] >> 8 ) & 0xff);
                        int nBlue = (aPixelLine[0][x] & 0xff);

                        nRed = (int)(255 + (255.0 * ((double)(nRed-255)/(double)nAlpha)));
                        nGreen = (int)(255 + (255.0 * ((double)(nGreen-255)/(double)nAlpha)));
                        nBlue = (int)(255 + (255.0 * ((double)(nBlue-255)/(double)nAlpha)));

                        if(nRed < 0) nRed = 0;
                        if(nGreen < 0) nGreen = 0;
                        if(nBlue < 0) nBlue = 0;
                        aPixelLine[0][x] = nBlue | (nGreen<<8) | (nRed<<16) | (nAlpha<<24);
                    }
                }
            }

            //Change the pixels of this line to their final value
            bmpFinal.setARGB(aPixelLine[0], 0, nWidth, 0, y, nWidth, 1);
        }
        return bmpFinal;
    }
    
	/**
	 * Shows the alert dialog.
	 * Can be called outside the event thread.
	 * @param msg The message to show.
	 */
	public static void showAlertDialog(final String msg)
	{
		invokeInEventThread(
			new Runnable() {
				public void run() {
					Dialog.alert(msg);
			}});
	}
	
	/**
	 * Shows the status popup.
	 * Can be called outside the event thread.
	 * @param msg The message to show.
	 */
	public static void showStatus(final String msg)
	{
		invokeInEventThread(
			new Runnable() {
				public void run() {
					Status.show(msg);
			}});
	}
	
	/**
	 * Shows the status popup when a connection to the server
	 * failed.
	 * @param status The status of the response.
	 * @param ex The exception, if any. Can be null.
	 */
	public static void showConnectionStatus(int status, Exception ex)
	{
		String msg = ConnectionUtil.getStatusMessage(status);
		if (ex != null) {
			if (ex instanceof TLSIOException) {
				// this unfortunately often has a null message
				msg += " " + I18N.getString(I18NResource.ConnectionUtilTLSSSLMsg);
			}
			if (ex.getMessage() != null) {
				msg += " " + ex.getMessage();
			}
		}
		UIUtil.showStatus(msg);
	}
	
	/**
	 * Push a screen on the display.
	 * Can be called outside the event thread.
	 * @param screen The screen.
	 */
	public static void pushScreen(final Screen screen)
	{
		invokeInEventThread(
			new Runnable() {
				public void run() {
					UiApplication.getUiApplication().pushScreen( screen );
			}});
	}
	
	/**
	 * Ensure that this is done in the event thread.
	 */
	public static void invokeInEventThread(Runnable runnable)
	{
		if (UiApplication.getApplication().isEventThread()) {
			runnable.run();
		} else {
			UiApplication.getApplication().invokeLater(runnable);
		}
	}
	    
	/**
	 * Returns whether the input field is in left-to-right mode.
	 * In theory, Blackberry 7 can decide it from the field, but we
	 * cannot use introspection to call it (no introspection in 
	 * JavaME). Blackberry 6 returns always true.
	 * @param f The field. Currently unused, but could be used when
	 *     we drop support for Blackberry 6.
	 */
	public static boolean isLeftToRight(Field f)
	{
		if (!_isRTLInitialized) {
			try {
				_isRTL = "true".equalsIgnoreCase(I18N.getString(I18NResource.IsRightToLeftLayout));
			} catch (Throwable th) {
				_isRTL = false;
			}
			_isRTLInitialized = true;
		}
		return !_isRTL;
	}
	
    // --------------------------------------------------------------
    // Features for future versions.
    // Those features work already but we decided to disable them
    // for now.
    
	/**
	 * Whether selection of a specific version is allowed.
	 * If disabled, it is only possible to install the latest
	 * version of an application. If enabled, it is possible
	 * to revert to any older version of an application, that is,
	 * to specify precisely which version to install.
	 */
	public static boolean isVersionSelectionAllowed()
	{
		return _allowVersionSelection;
	}
	
    /**
     * Whether the search box has a search scope.
     */
    public static boolean isSearchScopeEnabled()
    {
    	return _searchScopeEnabled;
    }
    
    /**
	 * Whether ascending and descending sort is allowed.
	 * It is only useful to enable this when also 
	 * {@link #isAdaptAppCatalogInfoLineToSorting} is enabled, since
	 * otherwise there is no visual feedback whether the sort
	 * is ascending or descending, and that can be confusing.
     */
    public static boolean isAscendingDescendingSortAllowed()
    {
    	return _allowAscendingDescendingSort;
    }
    
    /**
     * Whether the info line in the application catalog (that is
     * normally only showing the version) can show popularity 
     * and updates. If enabled, when you change the sorting of
     * the application catalog, the version info gets replaced
     * by the popularity or by the update date.
     */
    public static boolean isAdaptAppCatalogInfoLineToSorting()
    {
    	return _adaptAppCatalogInfoLineToSorting;
    }
    
    /**
	 * Whether the application is allowed to save the password
	 * on the device. When not saved, restarting the App Center
	 * will always ask for the password.
	 */
    public static boolean isSavePasswordOnDevice()
    {
    	return _savePasswordOnDevice;
    }
}
