/*
 *  Licensed Materials - Property of IBM
 *  5725-G92 (C) Copyright IBM Corp. 2011, 2013. All Rights Reserved.
 *  US Government Users Restricted Rights - Use, duplication or
 *  disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 */

package com.ibm.appcenter.ui.model;

import java.util.Timer;
import java.util.TimerTask;

import com.ibm.appcenter.artifacts.AppSummary;
import com.ibm.appcenter.connection.ConnectionUtil;
import com.ibm.appcenter.connection.RESTCall;
import com.ibm.appcenter.connection.RESTCallBatch;
import com.ibm.appcenter.connection.RESTCallContinuation;
import com.ibm.appcenter.i18n.I18N;
import com.ibm.appcenter.i18n.I18NResource;
import com.ibm.appcenter.ui.IconCache;
import com.ibm.appcenter.ui.UIUtil;
import com.ibm.appcenter.util.BasicUtil;
import com.ibm.appcenter.util.IconBitmapProvider;
import net.rim.device.api.system.Bitmap;
import net.rim.device.api.ui.DrawStyle;
import net.rim.device.api.ui.Font;
import net.rim.device.api.ui.Graphics;
import net.rim.device.api.ui.XYRect;
import net.rim.device.api.ui.component.ListField;
import net.rim.device.api.ui.component.ListFieldCallback;
import net.rim.device.api.ui.decor.Background;


/**
 * List model for applications.
 * Used for installed apps,  app catalog, updated apps.
 */
public class AppListModel
extends BaseListModel
implements ListFieldCallback, IconBitmapProvider 
{
	// the static resources    
    private static Bitmap _fullStar = null;
    private static Bitmap _halfStar = null;
    private static Bitmap _emptyStar = null;
    private static Bitmap _fallbackAppIcon = null;
    
    // the elements of the model
	private AppSummary _apps[];
	private int _infoLineMode = AppSummary.VERSION_INFO;
	private int _textAreaWidth;
		
	// data used during rolling animation
	private RollingLineInfo _selNameLineRolling = new RollingLineInfo();
	private RollingLineInfo _selInfoLineRolling = new RollingLineInfo();
	private int _selectedVisibleCounter = 0;
	private int _animatedIndex = -1;
	private int _rollingIndex = -1;
	private Timer _animationTimer = null;
	
	/**
	 *  Constructor.
	 */
    public AppListModel(ListField list)
    {	     
		super(list);
    }
    
    /**
     * Calculates the row height.
     */
    protected int calcRowHeight()
    {
    	return getTitleFontHeightInPxl() + 
    			2 * getInfoFontHeightInPxl() +
    			UIUtil.getNormalXAdvanceInPxl();
    }
    
    /**
     * Sets the applications to be displayed.
     * @param apps
     */
    public void setApps(AppSummary[] apps)
    {
    	_apps = apps;
    	
	    if (apps == null || apps.length == 0) {
	    	_view.setSize(0);
	    } else {
	    	_view.setSize(apps.length);
	    	_view.setSelectedIndex(0);
	    }
	    _view.invalidate();
	}

    /**
     * Sets the info line mode.
     * @see AppSummary#VERSION_INFO
     * @see AppSummary#UPDATED_INFO
     * @see AppSummary#POPULARITY_INFO
     */
    public void setInfoLineMode(int mode)
    {
    	_infoLineMode = mode;
    }
    
    /**
     * Returns the model name, for logging.
     */
    protected String getModelName()
    {
    	return "AppLiMo";
    }
    	
	// ------------------------------------------------------------
	// ListFieldCallback methods
    
    /**
     * Renders a row.
     * @param list The list widget.
     * @param g The graphics.
     * @param index The item to render.
     * @param y The y drawing position.
     * @param w The width of the row.
     */
    public void drawListRow(ListField list, Graphics g, int index, int y, int w) 
    {
    	int margin = UIUtil.getNormalXAdvanceInPxl() / 2;
		  
    	boolean selected = (index == list.getSelectedIndex() && list.isFocus());
    
    	AppSummary app = _apps[index];
    	
    	if (selected) {
    		// init the rollings, so that we can do needsRolling later
    		if (index != _rollingIndex) {
    			_selNameLineRolling.reinit(app.getName(), getTitleFont());
    			_selInfoLineRolling.reinit(app.getInfoLine(_infoLineMode), getInfoFont());
    			_rollingIndex = index;
    		}
    	}
    	
    	// draw the background, unless selected
    	if (!selected) {
    		Background background = UIUtil.getNormalBackground();
        	if (background != null) {
        		XYRect rect = new XYRect(0, y, w, _rowHeight);
        		background.draw(g, rect);
        	}
    	}
    	    	
    	// draw the arrow to the right side
    	paintArrow(g, y, w, _rowHeight);
    	
    	// draw the bitmap to the left side
    	y += margin;
    	int iconsize = IconCache.getDefaultIconDimension();
    	Bitmap bitmap = UIUtil.getScaledBitmap(app.getIcon(this, index), iconsize, iconsize);
    	if (bitmap == null) {
    		bitmap = _fallbackAppIcon;
    	}
    	g.drawBitmap(margin, y, bitmap.getWidth(), bitmap.getHeight(), bitmap, 0, 0);
    	
        // left and right margin
        int lm = iconsize + 3 * margin;
        int rm = getArrowBitmap().getWidth() + 2 * UIUtil.getNormalXAdvanceInPxl();
        
        // effective text area width
        _textAreaWidth = w - lm - rm;
    	
        int oldColor = g.getColor();
    	Font oldFont = g.getFont();
    	XYRect oldRect = g.getClippingRect();
    	try {
    		// text draw color
    		g.setColor(UIUtil.getNormalTextColor());
    		// draw separator line
    		g.drawLine(0, y + _rowHeight - margin - 1, w, y + _rowHeight - margin - 1);
            // clip with left and right margin
            g.pushContext(
            		oldRect.x + lm, 
            		oldRect.y, 
            		oldRect.width - lm - rm, 
            		oldRect.height, 0, 0);

            // draw the name
            int shift = 0;
            int style = DrawStyle.LEADING | DrawStyle.ELLIPSIS;
			if (selected) {
				shift = _selNameLineRolling.shift;
				style = DrawStyle.LEADING;
			}
    		g.setFont(getTitleFont());
			g.drawText(app.getName(),
					   lm + shift, y,
					   style,
					   _textAreaWidth - shift);
    		y += getTitleFontHeightInPxl();
    		
    		// draw the info line
            shift = 0;
            style = DrawStyle.LEADING | DrawStyle.ELLIPSIS;
            g.setFont(getInfoFont());
			if (selected) {
				shift = _selInfoLineRolling.shift;
				style = DrawStyle.LEADING;
			}
			g.drawText(app.getInfoLine(_infoLineMode),
					   lm + shift, y,
					   style,
					   _textAreaWidth - shift);
    		y += getInfoFontHeightInPxl();
    		
    		// draw the rating line
    		shift = 0;
    		style = DrawStyle.LEADING | DrawStyle.ELLIPSIS;
    		g.setFont(getInfoFont());
    		String label = I18N.getString(I18NResource.AppArtifactRatingInfo);
    		g.drawText(label, lm, y);
    		shift += getInfoFont().getAdvance(label);
    		shift += getInfoFont().getAdvance("x");
    		double value = app.getRating();
    		long count = app.getNumberOfRatings();
    		shift += paintRating(g, lm + shift, y, value);
    		shift += getInfoFont().getAdvance("x");
    		g.drawText(I18N.getString(I18NResource.AppArtifactRatingNumberInfo, BasicUtil.formatRatingCount(count)),  
    				   lm + shift, 
    				   y,
    				   style,
    				   _textAreaWidth - shift);
    		y += getInfoFontHeightInPxl();
    		
    	} finally {
    		g.setFont(oldFont);
    		g.popContext();
    		g.setColor(oldColor);
    	}
    	
    	if (selected &&
    			(_selNameLineRolling.needsRolling(_textAreaWidth) ||
    			 _selInfoLineRolling.needsRolling(_textAreaWidth))) {
    		// selected index is visible, restart the animation
    		restartAnimation();
    	}
    }
	  
    /**
     * Returns the item in the list at the give index. 
     */
    public Object get(ListField list, int index)
    {
    	if (_apps == null || index < 0 || index >= _apps.length) {
    		return null;
    	}
    	return _apps[index];
    }
    
    /**
     * Returns the first occurrence of prefix p in the list.
     * Used for filtering list elements.
     * @param list The list field.
     * @param p The prefix to search for.
     * @param s The start index of the search.
     * @return The index of the first matching item,
     *    or the selected index, if no item was found. 
     */
    public int indexOfList(ListField list, String p, int s) 
    {
    	for (int i = s; i < _apps.length; i++) {
    		if (_apps[i].getName().startsWith(p)) {
    			return i;
    		}
    	}
    	return list.getSelectedIndex();
    }
    
	// ------------------------------------------------------------
    // Utilities for strings and fonts and bitmap resources
    
    private Font getTitleFont()
    {
    	return UIUtil.getBoldFont();
    }
    
    private Font getInfoFont()
    {
    	return UIUtil.getMicroFont();
    }
    
    private int getTitleFontHeightInPxl()
    {
    	return UIUtil.getBoldFontHeightInPxl();
    }
    
    private int getInfoFontHeightInPxl()
    {
    	return UIUtil.getMicroFontHeightInPxl();
    }
    
    /**
     * Initialize the resources.
     */
    protected void ensureResourcesInitialized()
    {
    	super.ensureResourcesInitialized();
    	
    	if (_fallbackAppIcon == null) {
        	int rowHeight = getTitleFontHeightInPxl() + 
        			2 * getInfoFontHeightInPxl() +
        			UIUtil.getNormalXAdvanceInPxl();
        	int iconsize = rowHeight - UIUtil.getNormalXAdvanceInPxl();
    		_fallbackAppIcon = UIUtil.getScaledBitmap(
    				"blackberry.png",
    				iconsize,
    				iconsize);
    	}
    	if (_fullStar == null) {
    		_fullStar = UIUtil.getScaledBitmap(
    				"fullstar.png",
    				getInfoFontHeightInPxl(),
    				getInfoFontHeightInPxl());
    	}
    	if (_halfStar == null) {
    		_halfStar = UIUtil.getScaledBitmap(
    				"halfstar.png",
    				getInfoFontHeightInPxl(),
    				getInfoFontHeightInPxl());
    	}
    	if (_emptyStar == null) {
    		_emptyStar = UIUtil.getScaledBitmap(
    				"emptystar.png",
    				getInfoFontHeightInPxl(),
    				getInfoFontHeightInPxl());
    	}
    }
     
    // -------------------------------------------------------------
    // Icon handling
    
    private RESTCallBatch _pendingBatch = new RESTCallBatch(null) {
    	// called when we are done with this batch.
    	public synchronized void clear() {
    		super.clear();
    		// start the next batch. Does nothing if the pending 
    		// batch is empty.
    		startPendingBatch();
    	}
    };
    private RESTCallBatch _activeBatch = null;
    
    /**
     * Returns the active batch for the icons.
     */
    public RESTCallBatch getActiveBatch()
    {
    	return _activeBatch;
    }
    
    /**
     * Start the pending batch. Always called from
     * the execution thread of the active batch.
     */
    private void startPendingBatch()
    {
    	synchronized (_pendingBatch) {
    		_activeBatch = _pendingBatch;
    		_pendingBatch = new RESTCallBatch(null) {
    			// called when we are done with the batch
    			public synchronized void clear() {
    				super.clear();
    				// start the next batch. Does nothing if the pending 
    				// batch is empty.
    				startPendingBatch();
    			}
    		};
    		if (!_activeBatch.isEmpty())
    			log("Try load more icons.");
    		else 
    			log("No further icons need loading.");
    		_activeBatch.start();
    	}
    }
    
    /**
     * Schedules a REST call for icon loading.
     * If other REST calls are running, it gets added to the
     * pending batch. If no other REST call is running, it gets
     * started immediately.
     * This is always called from the event thread.
     * @param call The REST call.
     */
    private void scheduleIconRESTCall(RESTCall call)
    {
    	synchronized (_pendingBatch) {
    		_pendingBatch.prepend(call);
    		if (_activeBatch == null || !_activeBatch.isRunning()) {
    			startPendingBatch();
    		} else {
    			// nothing to do. When the _activeBatch is running,
    			// it will start the pending batch just when it
    			// terminates, through the clear() method.
    		}
    	}
    }
     
    /**
     * Returns the icon bitmap for a given url.
     */
    public Bitmap getIcon(final String url, final int listIndex)
    {
    	Bitmap bm = IconCache.getInstance().get(url);
    	if (bm != null) {
    		if (bm == IconCache.NO_BITMAP)
    			return null;
    		return bm;
    	}
    	
    	log("getIcon " + url + " " + listIndex);
    	// use a REST call to get the icon bitmap

    	RESTCall call = new RESTCall(
    			ConnectionUtil.getURI(url),
				ConnectionUtil.getUsername(),
				ConnectionUtil.getPassword(),
				RESTCall.GET,
				null,
				null,
		        ConnectionUtil.getTimeout(),
				new RESTCallContinuation() {

					public void onSuccess(String contentType, byte[] bytes) {
						log("Retrieved successfully icon " + url);
						Bitmap bm = null;
						try {
							bm = Bitmap.createBitmapFromPNG(bytes, 0, bytes.length);
							int iconsize = _rowHeight - UIUtil.getNormalXAdvanceInPxl();
							bm = UIUtil.getScaledBitmap(bm, iconsize, iconsize);
							IconCache.getInstance().put(url, bm);
						} catch (Exception ex) {
							// ignore the exception. We cannot do much else,
							// since this is a bad time point for dialog etc.
							log("Could not decode bitmap " + url, ex);
							IconCache.getInstance().put(url, null);
						}
				    	// redraw the related list field. This will pickup the
				    	// bitmap from cache
						UIUtil.invokeInEventThread(new Runnable(){
							public void run() {
								_view.invalidate(listIndex);
							}}); 
					}

					public void onFailure(Exception ex, int status,
							String contentType, byte[] bytes) {
						log("Failed to retrieve icon " + url + ", status=" + status);
						// we simply ignore the failure, assuming the bitmap is null
						IconCache.getInstance().put(url, null);						
					}

					public void onCancel() {
						log("Cancelled to retrieve icon " + url);
					}},
				false,
				false);
    	
    	// try to start the call. If other calls are still busy, it does not
    	// start the call. The end of the other calls will then start this call,
    	// but adds it to the list of pending calls
    	scheduleIconRESTCall(call);
    	
    	return null;
    }
    
    // -------------------------------------------------------------
    // Various paint utilities
    
    /**
     * Paints the rating stars.
     * @param g The graphics.
     * @param x The x position on the screen.
     * @param y The y position on the screen.
     * @param value The rating value, between 0 and 5.
     * @return The horizontal width of the ratings.
     */
    private int paintRating(Graphics g, int x, int y, double value)
    {
    	return
        	super.paintRating(g,  x, y, value, 
        		_fullStar, _halfStar, _emptyStar);
    }
                
	// ------------------------------------------------------------
	// Related to the rolling animation of selected text 
    // that is too long to be displayed in the row.
    
    /**
     * Restarts the animation.
     * This is called whenever a selected item is
     * visible.
     */
    private void restartAnimation()
	{
    	if (_selectedVisibleCounter <= 0) {
    		// animation was not running, need to really start it
			_selectedVisibleCounter = 5;
			Timer timer = _animationTimer;
			if (timer == null) {
				_animationTimer = new Timer();
				_animationTimer.schedule(getAnimationTask(), 100, 100);
			} else {
				synchronized (timer) {
					// just to be sure (normally is should already be canceled)
					try {
						timer.cancel();
					} catch (Throwable th) {
					}
					_animationTimer = new Timer();
					_animationTimer.schedule(getAnimationTask(), 100, 100);
				}
			}
		} else {
			// animation was running, not much to do
			_selectedVisibleCounter = 5;
		}
	}
	
    /**
     * The animation timer task that performs the rolling animation.
     */
	private TimerTask getAnimationTask()
	{
		return new TimerTask() {
			public void run() {
				if (_selectedVisibleCounter <= 0) {
					// selected item is no longer visible, stop animation
					Timer old = _animationTimer;
					if (old != null) {
						synchronized (old) {
							_animationTimer = null;
							old.cancel();
						}
					}
					_selNameLineRolling.shift = 0;
					_selInfoLineRolling.shift = 0;
					_animatedIndex = -1;
				} else {
					_selectedVisibleCounter--;
					// invoke _view.invalidate(_view.getSelectedIndex());
					UIUtil.invokeInEventThread(new Runnable() {
						public void run() {
							if (_animatedIndex != _view.getSelectedIndex()) {
								// selection has changed, redraw the previous selected
								// without shifts
								int index = _view.getSelectedIndex();
								AppSummary app = (index > 0 && index < _apps.length ? _apps[index] : null);
								if (app != null) {
									_selNameLineRolling.reinit(app.getName(), getTitleFont());
									_selInfoLineRolling.reinit(app.getInfoLine(_infoLineMode), getInfoFont());
								}
								_view.invalidate(_animatedIndex);
								_animatedIndex = _view.getSelectedIndex();
							}
							// redraw the current selected
							_view.invalidate(_animatedIndex);
							// calculate new shift values
							_selNameLineRolling.doShift(_textAreaWidth);
							_selInfoLineRolling.doShift(_textAreaWidth);
						}
					});
				}
			}
		};
    }
}
