/*
 *  Licensed Materials - Property of IBM
 *  5725-G92 (C) Copyright IBM Corp. 2011, 2013. All Rights Reserved.
 *  US Government Users Restricted Rights - Use, duplication or
 *  disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 */

package com.ibm.appcenter.ui.model;

import com.ibm.appcenter.i18n.I18NResource;
import com.ibm.appcenter.ui.UIUtil;
import com.ibm.appcenter.util.Log;

import net.rim.device.api.system.Bitmap;
import net.rim.device.api.system.Display;
import net.rim.device.api.ui.Font;
import net.rim.device.api.ui.Graphics;
import net.rim.device.api.ui.component.ListField;
import net.rim.device.api.ui.component.ListFieldCallback;

/**
 * Base list model for potentially infinite lists.
 */
public abstract class BaseListModel
implements ListFieldCallback, I18NResource
{
	// the static resources    
    private static Bitmap _arrow = null;
    
    // the elements of the model
	protected ListField _view;
	protected int _rowHeight;
	
	// for logging
	private boolean _logEnabled = true;
		
	/** 
	 * Constructor.
	 */
    public BaseListModel(ListField list) 
    {	     
		ensureResourcesInitialized();
		
    	_rowHeight = calcRowHeight();
    	
	    // save a ref to the list view
	    _view = list;

	    // bind this model to the given view
	    list.setCallback(this);
	    
	    // initially we have no elements
	    list.setSize(0);

	    // set the default row height
	    _view.setRowHeight(_rowHeight);
    }
    
    /**
     * Calculates the row height.
     */
    protected abstract int calcRowHeight();
    
    /**
     * Returns the model name, for logging.
     */
    protected abstract String getModelName();
    
	/**
	 * Enable or disable logging for this model.
	 */
	public void setLogEnabled(boolean enable)
	{
		_logEnabled = enable;
	}

	/**
	 * Logs a throwable with a message.
	 * @param msg The message.
	 * @param th  The throwable.
	 */
	public void log(String msg, Throwable th)
	{
		if (_logEnabled)
			Log.log(getModelName() + " " + msg, th);
	}
	
	/**
	 * Logs a message.
	 * @param msg The message.
	 */
	public void log(String msg)
	{
		if (_logEnabled)
			Log.log(getModelName() + " " + msg);
	}
	
	// ------------------------------------------------------------
	// Various utilities
       
    /**
     * Returns the preferred width of the list in pixel.
     * Used when rendering the list.
     */
    public int getPreferredWidth(ListField list) 
    {
    	return Display.getWidth();
    }
        
    /**
     * Initialize the resources.
     */
    protected void ensureResourcesInitialized()
    {
    	if (_arrow == null) {
    		String key = (UIUtil.isLightOnDark() ? "light" : "dark");
    		_arrow = UIUtil.getScaledBitmap(
    				"arrowright" + key + ".png",
    				UIUtil.getBoldFontHeightInPxl());
    	}
    }
             
    /**
     * Paints the arrow at the right border of the screen.
     * @param g The graphics.
     * @param y The position of the row.
     * @param w The screen width.
     * @param h The row height.
     */
    protected void paintArrow(Graphics g, int y, int w, int h)
    {
    	if (_arrow != null) {
    		int ydelta = (h - _arrow.getHeight()) / 2;
    		int x = w - _arrow.getWidth() - UIUtil.getNormalXAdvanceInPxl();
            g.drawBitmap(x, y + ydelta, _arrow.getWidth(), _arrow.getHeight(), _arrow, 0, 0);
        }
    }
    
    /**
     * Returns the arrow bitmap.
     */
    protected Bitmap getArrowBitmap()
    {
    	return _arrow;
    }
    
    /**
     * Paints the rating stars. The star bitmaps must be square size.
     * @param g The graphics.
     * @param x The x position on the screen.
     * @param y The y position on the screen.
     * @param value The rating value, between 0 and 5.
     * @param fullStar
     * @param halfStar
     * @param emptyStar
     * @return The horizontal width of the ratings.
     */
    protected int paintRating(Graphics g, int x, int y, double value,
    		Bitmap fullStar, Bitmap halfStar, Bitmap emptyStar)
    {
    	if (fullStar == null || halfStar == null || emptyStar == null)
    		return 0;
    	int w = 0;
    	int delta = fullStar.getHeight();
    	Bitmap b;
    	for (int i = 0; i < 5; i++) {
    		if (value < 1.0/3) {
    			b = emptyStar;
    		} else if (value < 2.0/3) {
    			b = halfStar;
    		} else {
    			b = fullStar;
    		}
    		g.drawBitmap(x, y, b.getWidth(), b.getHeight(), b, 0, 0);
    		x += delta;
    		w += delta;
    		value = value - 1;
    	}
    	return w;
    }
    
	// ------------------------------------------------------------
	// Related to the rolling animation of selected text 
    // that is too long to be displayed in the row.
    	
	static class RollingLineInfo
	{
		// width of the text
		int width;
		// shift value
		int shift;
		// whether shifting forward or backward
		boolean backward;
				
		void reinit(String s, Font f)
		{
			shift = 0;
			width = f.getBounds(s);
		}
		
		void doShift(int textAreaWidth)
		{
			if (backward) {
				// shift goes from -max to 0
				if (shift >= 0)
					backward = false;
				else 
					shift += 3;
			} else {
				// shift goes from 0 to -max
				if (width + shift <= textAreaWidth)
					backward = true;
				else
					shift -= 3;
			}
		}
		
		boolean needsRolling(int textAreaWidth)
		{
			return width > textAreaWidth;
		}
	}
}