/*
 *  Licensed Materials - Property of IBM
 *  5725-G92 (C) Copyright IBM Corp. 2011, 2013. All Rights Reserved.
 *  US Government Users Restricted Rights - Use, duplication or
 *  disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 */

package com.ibm.appcenter.ui.model;

import net.rim.device.api.system.Bitmap;
import net.rim.device.api.ui.DrawStyle;
import net.rim.device.api.ui.Font;
import net.rim.device.api.ui.Graphics;
import net.rim.device.api.ui.XYRect;
import net.rim.device.api.ui.component.ListField;
import net.rim.device.api.ui.component.ListFieldCallback;
import net.rim.device.api.ui.decor.Background;

import com.ibm.appcenter.artifacts.Feedback;
import com.ibm.appcenter.i18n.I18N;
import com.ibm.appcenter.ui.UIUtil;

/**
 * List model for application feedbacks.
 */
public class FeedbackListModel 
extends BaseListModel
implements ListFieldCallback
{  
	// the static resources    
    private static Bitmap _fullStar = null;
    private static Bitmap _halfStar = null;
    private static Bitmap _emptyStar = null;
    
    // the elements of the model
	private Feedback _ratings[];
	private int _textAreaWidth;
	private boolean _displayAppVersion;
		
	/**
	 *  Constructor.
	 */
    public FeedbackListModel(ListField list, boolean displayAppVersion) 
    {	     
    	super(list);
    	_displayAppVersion = displayAppVersion;
    	// row height depends on _displayAppVersion
    	_rowHeight = calcRowHeight();
    	_view.setRowHeight(_rowHeight);
    }
    
    /**
     * Calculates the row height.
     */
    protected int calcRowHeight()
    {
    	int numLines = (_displayAppVersion ? 2 : 1);
    	return numLines * getInfoFontHeightInPxl() +
    			getCommentFontHeightInPxl() +
    			UIUtil.getNormalXAdvanceInPxl();
    }
        
    /**
     * Sets the ratings to be displayed.
     * @param ratings The ratings.
     * @param hasMode Whether to show a "Load mode" button.
     */
    public void setFeedbacks(Feedback[] ratings, boolean hasMore)
    {
    	_ratings = ratings;
    	
	    if (ratings == null || ratings.length == 0) {
	    	_view.setSize(0);
	    } else {
	    	_view.setSize(ratings.length + (hasMore ? 1 : 0));
	    	_view.setSelectedIndex(0);
	    }
	    _view.invalidate();
	}

    /**
     * Returns the model name, for logging.
     */
    protected String getModelName()
    {
    	return "FeeLiMo";
    }
    	
	// ------------------------------------------------------------
	// ListFieldCallback methods
    
    /**
     * Renders a row.
     * @param list The list widget.
     * @param g The graphics.
     * @param index The item to render.
     * @param y The y drawing position.
     * @param w The width of the row.
     */
    public void drawListRow(ListField list, Graphics g, int index, int y, int w) 
    {
    	if (index == _ratings.length) {
    		drawLastRow(list, g, index, y, w);
    		return;
    	}
    	
    	int margin = UIUtil.getNormalXAdvanceInPxl() / 2;
		  
    	boolean selected = (index == list.getSelectedIndex() && list.isFocus());
    
    	Feedback feedback = _ratings[index];
    	    	
    	// draw the background, unless selected
    	if (!selected) {
    		Background background = UIUtil.getNormalBackground();
        	if (background != null) {
        		XYRect rect = new XYRect(0, y, w, _rowHeight);
        		background.draw(g, rect);
        	}
    	}
    	    	    	
    	// top margin
    	y += margin;
    	
        // left and right margin
        int lm = margin;
        int rm = margin;
        
        // effective text area width
        _textAreaWidth = w - lm - rm;
    	
        int oldColor = g.getColor();
    	Font oldFont = g.getFont();
    	XYRect oldRect = g.getClippingRect();
    	try {
    		// text draw color
    		g.setColor(UIUtil.getNormalTextColor());
    		// draw separator line
    		g.drawLine(0, y + _rowHeight - margin - 1, w, y + _rowHeight - margin - 1);
            // clip with left and right margin
            g.pushContext(
            		oldRect.x + lm, 
            		oldRect.y, 
            		oldRect.width - lm - rm, 
            		oldRect.height, 0, 0);
    		
    		// draw the rating line
    		int style = DrawStyle.LEADING | DrawStyle.ELLIPSIS;
    		g.setFont(getInfoFont());    		
    		int shift = 0;
    		double value = feedback.getRating();
    		shift += paintRating(g, lm + shift, y, value);
    		shift += getInfoFont().getAdvance("x");
    		g.drawText(I18N.getString(FeedbackListModelByAuthorInfo, feedback.getAuthor()),
    				   lm + shift, 
    				   y,
    				   style,
    				   _textAreaWidth - shift);
    		y += getInfoFontHeightInPxl();
    		
    		if (_displayAppVersion) {
    			// draw the version
    			shift = 0;
    			g.drawText(I18N.getString(FeedbackListModelForVersionInfo, feedback.getVersion()),
    					   lm + shift, y,
    					   style,
    					   _textAreaWidth - shift);
    			y += getInfoFontHeightInPxl();
    		}
    		
    		// draw the comment line
            shift = 0;
            g.setFont(getCommentFont());
			g.drawText(formatComment(feedback.getComment()),
					   lm + shift, y,
					   style,
					   _textAreaWidth - shift);
    		y += getCommentFontHeightInPxl();
    	} finally {
    		g.setFont(oldFont);
    		g.popContext();
    		g.setColor(oldColor);
    	}
    }
	
    /**
     * Renders the last row which is special.
     * @param list The list widget.
     * @param g The graphics.
     * @param index The item to render.
     * @param y The y drawing position.
     * @param w The width of the row.
     */
    public void drawLastRow(ListField list, Graphics g, int index, int y, int w) 
    {    	
    	int margin = UIUtil.getNormalXAdvanceInPxl() / 2;
		  
    	boolean selected = (index == list.getSelectedIndex() && list.isFocus());
        	    	
    	// draw the background, unless selected
    	if (!selected) {
    		Background background = UIUtil.getNormalBackground();
        	if (background != null) {
        		XYRect rect = new XYRect(0, y, w, _rowHeight);
        		background.draw(g, rect);
        	}
    	}
    	    	    	    	
        // left and right margin
        int lm = margin;
        int rm = margin;
        
        // effective text area width
        _textAreaWidth = w - lm - rm;
    	
        int oldColor = g.getColor();
    	Font oldFont = g.getFont();
    	XYRect oldRect = g.getClippingRect();
    	try {
    		// text draw color
    		g.setColor(UIUtil.getNormalTextColor());
    		// draw separator line
    		g.drawLine(0, y + _rowHeight - 1, w, y + _rowHeight - 1);
            // clip with left and right margin
            g.pushContext(
            		oldRect.x + lm, 
            		oldRect.y, 
            		oldRect.width - lm - rm, 
            		oldRect.height, 0, 0);
    		
    		// draw the more indicator
    		int style = DrawStyle.LEADING | DrawStyle.ELLIPSIS;    		
    		int delta = (_rowHeight - this.getInfoFontHeightInPxl()) / 2;
    		g.setFont(getInfoFont()); 
			g.drawText(I18N.getString(FeedbackListModelMoreCommentsButtonTitle),
					   lm, y + delta,
					   style,
					   _textAreaWidth);
    	} finally {
    		g.setFont(oldFont);
    		g.popContext();
    		g.setColor(oldColor);
    	}
    }
    
    /**
     * Returns the item in the list at the give index. 
     */
    public Object get(ListField list, int index)
    {
    	if (_ratings == null || index < 0 || index >= _ratings.length) {
    		return null;
    	}
    	return _ratings[index];
    }
    
    /**
     * Returns the first occurrence of prefix p in the list.
     * Used for filtering list elements.
     * @param list The list field.
     * @param p The prefix to search for.
     * @param s The start index of the search.
     * @return The index of the first matching item,
     *    or the selected index, if no item was found. 
     */
    public int indexOfList(ListField list, String p, int s) 
    {
    	for (int i = s; i < _ratings.length; i++) {
    		if (_ratings[i].getComment().startsWith(p)) {
    			return i;
    		}
    	}
    	return list.getSelectedIndex();
    }
    
	// ------------------------------------------------------------
    // Utilities for strings and fonts and bitmap resources
        
    private Font getInfoFont()
    {
    	return UIUtil.getItalicFont();
    }
        
    private int getInfoFontHeightInPxl()
    {
    	return UIUtil.getItalicFontHeightInPxl();
    }
    
    private Font getCommentFont()
    {
    	return UIUtil.getNormalFont();
    }
        
    private int getCommentFontHeightInPxl()
    {
    	return UIUtil.getNormalFontHeightInPxl();
    }
    
    /**
     * Initialize the resources.
     */
    protected void ensureResourcesInitialized()
    {
    	super.ensureResourcesInitialized();
    	
    	if (_fullStar == null) {
    		_fullStar = UIUtil.getScaledBitmap(
    				"fullstar.png",
    				getInfoFontHeightInPxl(),
    				getInfoFontHeightInPxl());
    	}
    	if (_halfStar == null) {
    		_halfStar = UIUtil.getScaledBitmap(
    				"halfstar.png",
    				getInfoFontHeightInPxl(),
    				getInfoFontHeightInPxl());
    	}
    	if (_emptyStar == null) {
    		_emptyStar = UIUtil.getScaledBitmap(
    				"emptystar.png",
    				getInfoFontHeightInPxl(),
    				getInfoFontHeightInPxl());
    	}
    }
         
    // -------------------------------------------------------------
    // Various utilities
    
    /**
     * Since we display the comment in one line, we replace the
     * newline characters. 
     */
    private String formatComment(String s)
    {
    	return s.replace('\n', ' ');
    }
    
    /**
     * Paints the rating stars.
     * @param g The graphics.
     * @param x The x position on the screen.
     * @param y The y position on the screen.
     * @param value The rating value, between 0 and 5.
     * @return The horizontal width of the ratings.
     */
    private int paintRating(Graphics g, int x, int y, double value)
    {
    	return
    		super.paintRating(g,  x, y, value, 
    			_fullStar, _halfStar, _emptyStar);
    }
}