/*
 *  Licensed Materials - Property of IBM
 *  5725-G92 (C) Copyright IBM Corp. 2011, 2013. All Rights Reserved.
 *  US Government Users Restricted Rights - Use, duplication or
 *  disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 */

package com.ibm.appcenter.ui.model;

import java.util.Timer;
import java.util.TimerTask;

import com.ibm.appcenter.artifacts.AppVersion;
import com.ibm.appcenter.ui.UIUtil;
import net.rim.device.api.ui.DrawStyle;
import net.rim.device.api.ui.Font;
import net.rim.device.api.ui.Graphics;
import net.rim.device.api.ui.XYRect;
import net.rim.device.api.ui.component.ListField;
import net.rim.device.api.ui.component.ListFieldCallback;
import net.rim.device.api.ui.decor.Background;


/**
 * List model for apps versions.
 * This is used to select the app version which detail should be shown.
 */
public class VersionListModel 
extends BaseListModel
implements ListFieldCallback
{    
    // the elements of the model
	private AppVersion _versions[];
	private int _textAreaWidth;
	
	// data used during rolling animation
	private RollingLineInfo _selVersionLineRolling = new RollingLineInfo();
	private RollingLineInfo _selUpdatedLineRolling = new RollingLineInfo();
	private int _selectedVisibleCounter = 0;
	private int _animatedIndex = -1;
	private int _rollingIndex = -1;
	private Timer _animationTimer = null;
	
	/** 
	 * Constructor.
	 */
    public VersionListModel(ListField list)
    {	 
    	super(list);
    }
    
    /**
     * Calculates the row height.
     */
    protected int calcRowHeight()
    {
    	return getVersionFontHeightInPxl() + 
    			getInfoFontHeightInPxl() +
    			UIUtil.getNormalXAdvanceInPxl();
    }
    
    /**
     * Sets the application versions to be displayed.
     * @param versions The versions.
     */
    public void setVersions(AppVersion[] versions)
    {
    	_versions = versions;
    	
	    if (versions == null || versions.length == 0) {
	    	_view.setSize(0);
	    } else {
	    	_view.setSize(versions.length);
	    	_view.setSelectedIndex(0);
	    }
	    _view.invalidate();
	}
    
    /**
     * Returns the model name, for logging.
     */
    protected String getModelName()
    {
    	return "VerLiMo";
    }
    	
	// ------------------------------------------------------------
	// ListFieldCallback methods
    
    /**
     * Renders a row.
     * @param list The list widget.
     * @param g The graphics.
     * @param index The item to render.
     * @param y The y drawing position.
     * @param w The width of the row.
     */
    public void drawListRow(ListField list, Graphics g, int index, int y, int w) 
    {
    	int margin = UIUtil.getNormalXAdvanceInPxl() / 2;
		  
    	boolean selected = (index == list.getSelectedIndex() && list.isFocus());
    
    	AppVersion version = _versions[index];
    	
    	if (selected) {
    		// init the rollings, so that we can do needsRolling later
    		if (index != _rollingIndex) {
    			_selVersionLineRolling.reinit(version.getVersionInfoLine(), getTitleFont());
    			_selUpdatedLineRolling.reinit(version.getUpdatedInfoLine(), getInfoFont());
    			_rollingIndex = index;
    		}
    	}
    	
    	// draw the background, unless selected
    	if (!selected) {
    		Background background = UIUtil.getNormalBackground();
        	if (background != null) {
        		XYRect rect = new XYRect(0, y, w, _rowHeight);
        		background.draw(g, rect);
        	}
    	}
    	    	
    	// draw the arrow to the right side
    	paintArrow(g, y, w, _rowHeight);

    	y += margin;

        // left and right margin
        int lm = 3 * margin;
        int rm = getArrowBitmap().getWidth() + 2 * UIUtil.getNormalXAdvanceInPxl();
        
        // effective text area width
        _textAreaWidth = w - lm - rm;
    	
        int oldColor = g.getColor();
    	Font oldFont = g.getFont();
    	XYRect oldRect = g.getClippingRect();
    	try {
    		// text draw color
    		g.setColor(UIUtil.getNormalTextColor());
    		// draw separator line
    		g.drawLine(0, y + _rowHeight - margin - 1, w, y + _rowHeight - margin - 1);
            // clip with left and right margin
            g.pushContext(
            		oldRect.x + lm, 
            		oldRect.y, 
            		oldRect.width - lm - rm, 
            		oldRect.height, 0, 0);

            // draw the version
            int shift = 0;
            int style = DrawStyle.LEADING | DrawStyle.ELLIPSIS;
			if (selected) {
				shift = _selVersionLineRolling.shift;
				style = DrawStyle.LEADING;
			}
    		g.setFont(getTitleFont());
			g.drawText(version.getVersionInfoLine(),
					   lm + shift, y,
					   style,
					   _textAreaWidth - shift);
    		y += getVersionFontHeightInPxl();
    		
    		// draw the info line
            shift = 0;
            style = DrawStyle.LEADING | DrawStyle.ELLIPSIS;
            g.setFont(getInfoFont());
			if (selected) {
				shift = _selUpdatedLineRolling.shift;
				style = DrawStyle.LEADING;
			}
			g.drawText(version.getUpdatedInfoLine(),
					   lm + shift, y,
					   style,
					   _textAreaWidth - shift);
    		y += getInfoFontHeightInPxl();
    		    		
    	} finally {
    		g.setFont(oldFont);
    		g.popContext();
    		g.setColor(oldColor);
    	}
    	
    	if (selected &&
    			(_selVersionLineRolling.needsRolling(_textAreaWidth) ||
    			 _selUpdatedLineRolling.needsRolling(_textAreaWidth))) {
    		// selected index is visible, restart the animation
    		restartAnimation();
    	}
    }
	  
    /**
     * Returns the item in the list at the give index. 
     */
    public Object get(ListField list, int index)
    {
    	if (_versions == null || index < 0 || index >= _versions.length) {
    		return null;
    	}
    	return _versions[index];
    }
    
    /**
     * Returns the first occurrence of prefix p in the list.
     * Used for filtering list elements.
     * @param list The list field.
     * @param p The prefix to search for.
     * @param s The start index of the search.
     * @return The index of the first matching item,
     *    or the selected index, if no item was found. 
     */
    public int indexOfList(ListField list, String p, int s) 
    {
    	for (int i = s; i < _versions.length; i++) {
    		if (_versions[i].getVersion().startsWith(p)) {
    			return i;
    		}
    	}
    	return list.getSelectedIndex();
    }
    
	// ------------------------------------------------------------
    // Utilities for strings and fonts
    
    private Font getTitleFont()
    {
    	return UIUtil.getBoldFont();
    }
    
    private Font getInfoFont()
    {
    	return UIUtil.getMicroFont();
    }
    
    private int getVersionFontHeightInPxl()
    {
    	return UIUtil.getBoldFontHeightInPxl();
    }
    
    private int getInfoFontHeightInPxl()
    {
    	return UIUtil.getMicroFontHeightInPxl();
    }
            
	// ------------------------------------------------------------
	// Related to the rolling animation of selected text 
    // that is too long to be displayed in the row.
    
    /**
     * Restarts the animation.
     * This is called whenever a selected item is
     * visible.
     */
    private void restartAnimation()
	{
    	if (_selectedVisibleCounter <= 0) {
    		// animation was not running, need to really start it
			_selectedVisibleCounter = 5;
			Timer timer = _animationTimer;
			if (timer == null) {
				_animationTimer = new Timer();
				_animationTimer.schedule(getAnimationTask(), 100, 100);
			} else {
				synchronized (timer) {
					// just to be sure (normally is should already be canceled)
					try {
						timer.cancel();
					} catch (Throwable th) {
					}
					_animationTimer = new Timer();
					_animationTimer.schedule(getAnimationTask(), 100, 100);
				}
			}
		} else {
			// animation was running, not much to do
			_selectedVisibleCounter = 5;
		}
	}
	
    /**
     * The animation timer task that performs the rolling animation.
     */
	private TimerTask getAnimationTask()
	{
		return new TimerTask() {
			public void run() {
				if (_selectedVisibleCounter <= 0) {
					// selected item is no longer visible, stop animation
					Timer old = _animationTimer;
					if (old != null) {
						synchronized (old) {
							_animationTimer = null;
							old.cancel();
						}
					}
					_selVersionLineRolling.shift = 0;
					_selUpdatedLineRolling.shift = 0;
					_animatedIndex = -1;
				} else {
					_selectedVisibleCounter--;
					// invoke _view.invalidate(_view.getSelectedIndex());
					UIUtil.invokeInEventThread(new Runnable() {
						public void run() {
							if (_animatedIndex != _view.getSelectedIndex()) {
								// selection has changed, redraw the previous selected
								// without shifts
								int index = _view.getSelectedIndex();
								AppVersion app = (index > 0 && index < _versions.length ? _versions[index] : null);
								if (app != null) {
									_selVersionLineRolling.reinit(app.getVersionInfoLine(), getTitleFont());
									_selUpdatedLineRolling.reinit(app.getUpdatedInfoLine(), getInfoFont());
								}
								_view.invalidate(_animatedIndex);
								_animatedIndex = _view.getSelectedIndex();
							}
							// redraw the current selected
							_view.invalidate(_animatedIndex);
							// calculate new shift values
							_selVersionLineRolling.doShift(_textAreaWidth);
							_selUpdatedLineRolling.doShift(_textAreaWidth);
						}
					});
				}
			}
		};
    }
}
