/*
 *  Licensed Materials - Property of IBM
 *  5725-G92 (C) Copyright IBM Corp. 2011, 2013. All Rights Reserved.
 *  US Government Users Restricted Rights - Use, duplication or
 *  disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 */
package com.ibm.appcenter.ui.widget;

import com.ibm.appcenter.ui.UIUtil;

import net.rim.device.api.ui.FieldChangeListener;
import net.rim.device.api.ui.Graphics;
import net.rim.device.api.ui.XYEdges;
import net.rim.device.api.ui.component.AutoTextEditField;
import net.rim.device.api.ui.decor.Border;
import net.rim.device.api.ui.decor.BorderFactory;

/**
 * A text field with auto-correction and the ability to set
 * a minimal height.
 * When the field is empty, it shows the hint of the field.
 */
public class EnhancedAutoTextEditField extends AutoTextEditField
{
	private int _minHeight = -1;	
	private boolean _enabled = true;
	private FieldChangeListener _listener;
    private String _hint = "";
    private static int _backgroundColor = UIUtil.getEditBackgroundColor();
    private static int _normalColor = UIUtil.getEditTextColor();
    private static int _disabledColor = UIUtil.getDisabledTextColor();
    private static int _hintColor = UIUtil.getHintTextColor();

    /**
     * Constructor.
     * @param label The label of the field.
     * @param initialString The initial string shown for the field.
     * @param hint The hint.
     * @param maxChars The maximal number of characters.
     * @param style The style, see {@link TextField}. 
     */
    public EnhancedAutoTextEditField(String label, String initialString, String hint, int maxChars, long style)
    {
    	super(label, initialString, maxChars, style);
    	_hint = hint;
    	setBorder(BorderFactory.createRoundedBorder(new XYEdges(4, 4, 4, 4), 0xc6c6c6, Border.STYLE_SOLID));
    	_enabled = true;
    }
    
    /**
     * Paints the background of the field.
     */
    protected void paintBackground(Graphics g)
    {
        g.setBackgroundColor(_backgroundColor);
        g.clear();
    }
    
    /**
     * Paints the field.
     */
    protected void paint(Graphics g)
    {
        int oldColor = g.getColor();
        try {
        	g.setColor(_enabled ? _normalColor : _disabledColor);
            String test = super.getText();
            if ( test == null || test.length() < 1 ) {
                g.setColor(_enabled ? _hintColor : _disabledColor);
                g.drawText(_hint, 0, 0);
            }
            super.paint(g);
        } finally {
            g.setColor( oldColor );
        }
    }
    
    /**
     * Enables or disables this field.
     */
    public void setEnabled(boolean enable)
    {
    	// don't call super.setEnabled(enable), due to layout bug
    	if (enable != _enabled) {
    		if (enable) {
    			// enable the listener again
    			setChangeListener(_listener);
    		} else {
    			// just disable the listener so that it does not receive input
    			_listener = this.getChangeListener();
    			setChangeListener(null);
    		}	
    		_enabled = enable;
    	}
    }
    
    /**
     * Sets the minimal height of the field.
     * @param minHeight The minimal height.
     */
    public void setMinHeight(int minHeight)
    {
    	_minHeight = minHeight;
    }
    
    /**
     * Returns the minimal height of the field.
     */
    public int getMinHeight()
    {
    	return _minHeight;
    }
    
    /**
     * Lays out this field.
     */
	protected void layout(int width, int height)
	{
		super.layout(width, height);
		if (getMinHeight() > 0) {
			height = Math.max(getMinHeight(), getExtent().height);
			super.layout(width, height);
			setExtent(width, height);
		}
	}
}

