/*
 *  Licensed Materials - Property of IBM
 *  5725-G92 (C) Copyright IBM Corp. 2011, 2013. All Rights Reserved.
 *  US Government Users Restricted Rights - Use, duplication or
 *  disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 */

package com.ibm.appcenter.ui.widget;

import com.ibm.appcenter.ui.UIUtil;
import com.ibm.appcenter.ui.widget.advancedui.HorizontalButtonFieldSet;
import com.ibm.appcenter.util.ProgressIndicator;

import net.rim.device.api.ui.Field;
import net.rim.device.api.ui.FieldChangeListener;
import net.rim.device.api.ui.UiApplication;
import net.rim.device.api.ui.XYEdges;
import net.rim.device.api.ui.component.Dialog;
import net.rim.device.api.ui.component.GaugeField;
import net.rim.device.api.ui.decor.Border;
import net.rim.device.api.ui.decor.BorderFactory;

/**
 * This widget has two modes. In normal mode,
 * it contains several buttons that are laid out
 * at equal size horizontally.
 * In progress mode, it displays a progress bar
 * and the buttons disappear. However the progress
 * bar has a small cancel icon.
 */
public class ProgressBarAndButtonSet 
extends HorizontalButtonFieldSet
implements ProgressIndicator
{
	private GaugeField _progress;
	private Field _cancelButton;
	private Field[] _fields;
	private int _max;
	private boolean _progressMode = false;
	private boolean _doLayout = true;
	private int _width, _height;
	
	/**
	 * Constructor.
	 */
	public ProgressBarAndButtonSet()
	{
		this(0);
	}
	
	/**
	 * Constructor.
	 * @param style The style.
	 */
	public ProgressBarAndButtonSet(long style)
	{
		super(style);
		XYEdges edges = new XYEdges(4,4,4,4);
        _progress = new GaugeField(null, 0, 100, 50, GaugeField.PERCENT);
        _progress.setFont(UIUtil.getMicroFont());
        Border border = BorderFactory.createRoundedBorder(edges);
        _progress.setBorder(border);
        _cancelButton = new HeaderButtonField("redx", Field.FIELD_VCENTER);
        _cancelButton.setChangeListener(new FieldChangeListener() {

			public void fieldChanged(Field field, int context) {
				setProgressMode(false);
				onCancel();
			}});
        
        _max = 15;
	}
	
	/**
	 * Sets the maximal value of the progress bar.
	 * @param max The maximal value.
	 */
	public void setProgressMaxValue(int max)
	{
		if (max <= 0)
			max = 1;
		_max = max;
	}
	
	/**
	 * Sets the value of the progress bar.
	 * @param value The current value.
	 */
	public void setProgressValue(int value)
	{
		int v = (100 * value) / _max;
		if (v > 100) v = 100;
		if (v < 0) v = 0;
		if (UiApplication.getApplication().isEventThread()) {
			_progress.setValue(v);
		} else {
			final int vv = v;
			UIUtil.invokeInEventThread(
				new Runnable() {
					public void run() {
						_progress.setValue(vv);
					}
				});
		}
	}
	
	/**
	 * Increases the value of the progress bar
	 * to the input value. If the progress bar
	 * value is already higher, it does nothing.
	 * @param value The increased value.
	 */
	public void increaseProgressValueTo(int value)
	{
		int v = (100 * value) / _max;
		if (v > 100) v = 100;
		if (v < _progress.getValue()) return;
		final int vv = v;
		UIUtil.invokeInEventThread(
			new Runnable() {
				public void run() {
					_progress.setValue(vv);
				}
			});
	}
	
	/**
	 * Enables or disables the cancel button.
	 * @param enable true to enable.
	 */
	public void setCancelEnabled(final boolean enable)
	{
		UIUtil.invokeInEventThread(
			new Runnable() {
				public void run() {
					_cancelButton.setEnabled(enable);
				}
			});
	}
	
	/**
	 * Enables or disables the progress mode.
	 * In progress mode, the progress bar is
	 * displayed. In normal mode, the normal buttons
	 * are displayed. 
	 * @param mode true if the progress mode is enabled.
	 */
	public void setProgressMode(final boolean mode)
	{
		UIUtil.invokeInEventThread(
			new Runnable() {
				public void run() {
					setProgressModeImpl(mode);
				}
			});
	}
	
	private void setProgressModeImpl(boolean mode)
	{
		if (mode == _progressMode) return;
		_progressMode = mode;
		if (mode) {
			_width = getWidth();
			_height = getHeight();
			_progress.setValue(0);
			int n = getFieldCount();
			_fields = new Field[n];
			for (int i = 0; i < n; i++) {
				_fields[i] = getField(i);
			}
			// disable layout until both progress and cancelButton are added
			try {
				_doLayout = false;
				deleteAll();
				add(_progress);
			} finally {
				_doLayout = true;
			}
			// this triggers the layout
			_cancelButton.setEnabled(true);
			add(_cancelButton);
			
		} else {
			try {
				_doLayout = false;
				deleteAll();
			} finally {
				_doLayout = true;
			}
			// this triggers the layout
			insertAll(_fields, 0);
		}
	}
	
	/**
	 * Layout of the container.
	 */
	protected void sublayout( int width, int height )
    {
		if (!_doLayout) return;
		if (_progressMode) {
			int h = _height;
			layoutChild(_cancelButton, width, height);
			int cw = _cancelButton.getWidth();
			int ch = _cancelButton.getHeight();
			int cd = (h - ch) / 2;
			int w = width - cw;
			layoutChild(_progress, w, h);
			int ph = _progress.getHeight();
			int pd = (h - ph) / 2; 
			if (UIUtil.isLeftToRight(this)) {
				setPositionChild(_progress, 0, pd);
				setPositionChild(_cancelButton, w, cd);
			} else {
				setPositionChild(_cancelButton, 0, cd);
				setPositionChild(_progress, cw, pd);
			}			
			setExtent(_width, _height);
		} else {
			super.sublayout(width, height);
		}
    }
		
	/**
	 * Callback when cancel the progress bar.
	 */
	protected void onCancel()
	{
		Dialog.alert("Canceled");
	}
}


