/*
 *  Licensed Materials - Property of IBM
 *  5725-G92 (C) Copyright IBM Corp. 2011, 2013. All Rights Reserved.
 *  US Government Users Restricted Rights - Use, duplication or
 *  disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 */

package com.ibm.appcenter.ui.widget;

import com.ibm.appcenter.ui.UIUtil;
import com.ibm.appcenter.ui.widget.advancedui.BitmapButtonField;

import net.rim.device.api.system.Bitmap;
import net.rim.device.api.ui.Field;
import net.rim.device.api.ui.FieldChangeListener;
import net.rim.device.api.ui.FocusChangeListener;
import net.rim.device.api.ui.container.HorizontalFieldManager;

/**
 * An interactive rating field that allows
 * to select from 0 - 5 stars.
 */
public class RatingEditField extends HorizontalFieldManager
{
	private int _rating;
	private BitmapButtonField[] _fields;
	private boolean _blockListener = false;
	
	private static Bitmap _emptyStar = null;
	private static Bitmap _fullStar = null;
	private static Bitmap _minusIcon = null;
	private static Bitmap _plusIcon = null;
	   
	/**
	 * Constructor.
	 * Notice that we assume that we use all rating fields with the
	 * same star size and arrow color.
	 */
	public RatingEditField(int starSize, long style)
	{
		super(style);
		
		String key = (UIUtil.isLightOnDark() ? "light" : "dark");
		if (_emptyStar == null) {
			_emptyStar = UIUtil.getScaledBitmap("emptystarsmall.png", starSize);
		}
		if (_fullStar == null) {
			_fullStar = UIUtil.getScaledBitmap("fullstar.png", starSize);
		}
		if (_minusIcon == null) {
			_minusIcon = UIUtil.getScaledBitmap("triangleleft" + key + ".png", starSize);
		}
		if (_plusIcon == null) {
			_plusIcon = UIUtil.getScaledBitmap("triangleright" + key + ".png", starSize);
		}
		final BitmapButtonField minus = new BitmapButtonField(_minusIcon) {
			protected void onFocus(int dir) {
				if (dir == -1) {
					setRating(0);
				}
				super.onFocus(dir);
			}
		};
		BitmapButtonField plus = new BitmapButtonField(_plusIcon) {
			protected void onFocus(int dir) {
				if (dir == -1) {
					int r = getRating();
					if (r <= 0) {
						minus.setFocus();
					} else if (r <= 5) {
						_fields[r-1].setFocus();
					} else {
						super.onFocus(dir);
					}
				} else {
					super.onFocus(dir);
				}
			}
		};
		add(minus);
		_fields = new BitmapButtonField[5];
		for (int i = 0; i < _fields.length; i++) {
			_fields[i] = new BitmapButtonField(_emptyStar, _fullStar);
			add(_fields[i]);
		}
		add(plus);
		minus.setChangeListener(new FieldChangeListener() {
			public void fieldChanged(Field field, int context) {
				if (_blockListener) return;
				try {
					_blockListener = true;
					int r = getRating();
					if (r > 0) r--;
					setRating(r);
				} finally {
					_blockListener = false;
				}
			}});
		plus.setChangeListener(new FieldChangeListener() {
			public void fieldChanged(Field field, int context) {
				if (_blockListener) return;
				try {
					_blockListener = true;
					int r = getRating();
					if (r < 5) r++;
					setRating(r);
				} finally {
					_blockListener = false;
				}
			}});
		for (int i = 0; i < _fields.length; i++) {
			final int fi = i + 1;
			_fields[i].setChangeListener(new FieldChangeListener() {
				public void fieldChanged(Field field, int context) {
					if (_blockListener) return;
					try {
						_blockListener = true;
						setRating(fi);
					} finally {
						_blockListener = false;
					}
				}});
			_fields[i].setFocusListener(new FocusChangeListener() {
				public void focusChanged(Field field, int eventType) {
					if (eventType != FocusChangeListener.FOCUS_GAINED) return;
					if (_blockListener) return;
					try {
						_blockListener = true;
						setRating(fi);
					} finally {
						_blockListener = false;
					}
				}});
		}
		setBackground(Field.VISUAL_STATE_FOCUS, UIUtil.getFocusBackground());
	}
	
	/**
	 * Changes the rating of this field.
	 * @param rating The rating.
	 */
	public void setRating(int rating)
	{
		if (rating != _rating) {
			_rating = rating;
			for (int i = 0; i < _fields.length; i++) {
				Bitmap bm = (i < rating ? _fullStar : _emptyStar);
				_fields[i].setImage(bm);
			}
			invalidate();
			onRating(rating);
		}
	}
	
	/**
	 * Called when the rating changes.
	 * @param val The new rating.
	 */
	protected void onRating(int val)
	{
		// overridden in subclasses, e.g. to show the rating in a label
	}

	/**
	 * Returns the rating of this field.
	 */
	public int getRating()
	{
		return _rating;
	}
}
