/*
 *  Licensed Materials - Property of IBM
 *  5725-G92 (C) Copyright IBM Corp. 2011, 2013. All Rights Reserved.
 *  US Government Users Restricted Rights - Use, duplication or
 *  disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 */

package com.ibm.appcenter.util;

import javax.microedition.global.Formatter;

import net.rim.device.api.io.http.HttpDateParser;

/**
 * Various general basic utilities.
 */
public class BasicUtil
{
	/**
	 * Private constructor since this is a singleton.
	 */
	private BasicUtil() {}
	
	
	/**
	 * Compares two versions.
	 * They should have the format x.y or x.y.z.
	 * We have 3.01 < 3.1 < 3.9 < 3.11, that is, it is
	 * nor interpreted as float number but each part between
	 * the dot is interpreted individually; and parts starting
	 * with 0 are interpreted as fraction of 1, that is are always
	 * smaller than parts starting not with 1.
	 * In the above example, the major version is always 3 and the
	 * minor versions are 01 < 1 < 9 < 11.
	 * This mechanism works also with 3.4.7 < 3.4.56 and even with
	 * arbitrary separator characters, e.g. 3|4|7 < 3|4|56
	 * @param v1
	 * @param v2
	 * @return -1 if v1 < v2, 1 if v1 > v2, 0 if v1 = v2
	 */
	public static int compareVersion(String v1, String v2)
	{
		if (v1 == v2) return 0;
		if (v1 == null) return -1;
		if (v2 == null) return 1;
		
		// search the first difference between both strings
		int n = Math.min(v1.length(), v2.length());
		
		for (int i = 0; i < n; i++) {
			char c1 = v1.charAt(i);
			char c2 = v2.charAt(i);
			if (c1 != c2) {
				// position i is the first character that is different
				if (Character.isDigit(c1)) {
					if (Character.isDigit(c2)) {
						// get both numbers
						double d1 = extractNumber(v1, i);
						double d2 = extractNumber(v2, i);
						if (d1 < d2)
							return -1;
						else
							return 1;
					} else {
						// example 334 vs 33.3 or 1.011.5 vs 1.01.7
						return 1;
					}
				} else {
					if (Character.isDigit(c2)) {
						// example 33.3 vs 334 or 1.01.7 vs 1.011.5
						return -1;
					} else {
						if (c1 < c2)
							return -1;
						else
							return 1;
					}
				}
			}
		}
		
		// one is prefix of the other, or both are equal
		if (v1.length() < v2.length()) return -1;
		if (v1.length() > v2.length()) return 1;
		return 0;
	}
	
	/**
	 * Extract the longest sequence of digits that contain position i,
	 * which is a digit. Interpret this as integer if not starting with 0,
	 * and as fractional < 1 if starting with 0.
	 * @param s The string.
	 * @param i The position inside the string.
	 * @return The corresponding interpretation as number.
	 */
	private static double extractNumber(String s, int i)
	{
		int n = s.length();
		while (i >= 0) {
			char c = s.charAt(i);
			if (!Character.isDigit(c)) break;
			i--;
		}
		i++;
		// now i is at the first digit
		long number = 0;
		long divisor = 0;
		if (s.charAt(i) == '0') {
			divisor = 1;
		}
		while (i < n) {
			char c = s.charAt(i);
			if (!Character.isDigit(c)) break;
			divisor *= 10;
			number = number * 10 + (c - '0');
			i++;
		}
		
		if (divisor > 0)
			return (double)number / (double)divisor;
		
		return (double)number;
	}
	
	/**
	 * Converts a average rating from server side to client side.
	 * The server deals with ratings 0..10 while the client deals
	 * with ratings 0..5.
	 */
	public static double avgRatingFromServer(double rating)
	{
		rating = rating / 2;
		if (rating < 0) rating = 0;
		if (rating > 5) rating = 5;
		return rating;
	}
	
	/**
	 * Converts a rating from server side to client side.
	 * The server deals with ratings 0..10 while the client deals
	 * with ratings 0..5.
	 */
	public static int ratingFromServer(int rating)
	{
		rating = rating / 2;
		if (rating < 0) rating = 0;
		if (rating > 5) rating = 5;
		return rating;
	}
	
	/**
	 * Converts a rating from client side to server side.
	 * The server deals with ratings 0..10 while the client deals
	 * with ratings 0..5.
	 */
	public static int ratingToServer(int rating)
	{
		rating = rating * 2;
		if (rating < 0) rating = 0;
		if (rating > 10) rating = 10;
		return rating;
	}
		
	/**
	 * Parses a string to a date.
	 * @param s The string.
	 * @return The date, in milliseconds from 1.1.1970 00:00:00 GMT.
	 */
	public static long parseDate(String s)
	{
		// it is not documented, but the HttpDateParser actually
		// can parse ISO 8601 dates
		return HttpDateParser.parse(s);
	}
	
	/**
	 * Formats a number to show only a fixed number of digits after
	 * the decimal point.
	 * @param value The value to be formatted.
	 * @param precision The number of digits.
	 * @return The formatted result.
	 */
	public static String formatNumber(double value, int precision)
	{
		if (precision == 0) {
			// avoid the trailing dot. Works only for small numbers
			// but I this this is ok right now
			long x = (long)value;
			return "" + x;
		}
    	try {
    		Formatter fmt = new Formatter();
    		return fmt.formatNumber(value, precision);
    	} catch (Exception ex) {
    		// Happens on Blackberry 6.
    		// We do the formatting ourself
    		StringBuffer s = new StringBuffer();
    		int x = (int)Math.floor(value);
    		s.append("" + x + ".");
    		for (int i = 0; i < precision; i++) {
    			value = (value - x) * 10;
    			x = (int)Math.floor(value);
    			s.append("" + x);
    		}
    		return s.toString();
    	}
	}
	
    /**
     * Formats the rating count to a string.
     * @param count The value.
     * @return The string to display.
     */
    public static String formatRatingCount(long count)
    {	
    	return "" + count;
    }
}
