/*
 *  Licensed Materials - Property of IBM
 *  5725-G92 (C) Copyright IBM Corp. 2011, 2013. All Rights Reserved.
 *  US Government Users Restricted Rights - Use, duplication or
 *  disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 */

package com.ibm.appcenter.util;

import java.util.Date;

import net.rim.device.api.i18n.Locale;
import net.rim.device.api.i18n.SimpleDateFormat;
import net.rim.device.api.system.EventLogger;

/**
 * Logging facilities.
 */
public class Log 
{
	private static StringBuffer _logBuffer = new StringBuffer();
	private static int _maxLogBufferSize = 5000;
	private static long _guid = 0x4f8a3351d7784ffcL;
	private static boolean _useEventLogger = true;
	private static int _eventLoggerLevel = EventLogger.INFORMATION; // DEBUG_INFO;
	
	/**
	 * Only called once.
	 */
	public static void register()
	{
		EventLogger.register(_guid, "IBMAppCenter", EventLogger.VIEWER_STRING);
		EventLogger.setMinimumLevel(EventLogger.DEBUG_INFO);
	}
	
	/**
	 * Logs a throwable.
	 * @param th The throwable.
	 */
	public static void log(Throwable th)
	{
		if (th == null) return;
		System.err.print(getDebugPrefix());
		_logBuffer.append(getDebugPrefix());
		th.printStackTrace();
		logThrowableToBuffer(th);
		cutBuffer();
	}
	
	/**
	 * Logs a throwable with a message.
	 * @param msg The message.
	 * @param th  The throwable.
	 */
	public static void log(String msg, Throwable th)
	{
		if (_useEventLogger) {
			String msg1 = msg + "\n";
			EventLogger.logEvent(_guid, msg1.getBytes(), _eventLoggerLevel);
		}
		System.err.println(getDebugPrefix() + msg);
		_logBuffer.append(getDebugPrefix() + msg + "\n");
		if (th != null) {
			th.printStackTrace();
			logThrowableToBuffer(th);
		}
		cutBuffer();
	}
	
	/**
	 * Logs a message.
	 * @param msg The message.
	 */
	public static void log(String msg)
	{
		if (_useEventLogger) {
			String msg1 = msg + "\n";
			EventLogger.logEvent(_guid, msg1.getBytes(), _eventLoggerLevel);
		}
		System.err.println(getDebugPrefix() + msg);
		_logBuffer.append(getDebugPrefix() + msg + "\n");
		cutBuffer();
	}
	
	/**
	 * Logs a message with arguments.
	 * @param msg The message.
	 * @param args The arguments.
	 */
	public static void log(String msg, Object[] args)
	{
		if (_useEventLogger) {
			StringBuffer buf = new StringBuffer(msg + " args: [ ");
			if (args != null) {
				for (int i = 0; i < args.length; i++) {
					buf.append(" " + args[i]);
				}
			}
			buf.append(" ]\n");
			EventLogger.logEvent(_guid, buf.toString().getBytes(), _eventLoggerLevel);
		}
		System.err.print(getDebugPrefix() + msg + " args: [ ");
		_logBuffer.append(getDebugPrefix() + msg + " args: [ ");
		if (args != null) {
			for (int i = 0; i < args.length; i++) {
				System.err.print("  " + args[i]);
				_logBuffer.append(" " + args[i]);
			}
		}
		System.err.println(" ]");
		_logBuffer.append(" ]\n");
		cutBuffer();
	}
	
	/**
	 * Put the throwable into the log buffer.
	 * @param th The throwable.
	 */
	private static void logThrowableToBuffer(Throwable th)
	{
		if (_useEventLogger) {
			String msg = th.getMessage() + "\n";
			EventLogger.logEvent(_guid, msg.getBytes(), _eventLoggerLevel);
		}
		_logBuffer.append(th.getMessage() + "\n");
		// It's a pity but there is no way to get the 
		// stack trace into the log.
		// Neither System.err can be redirected nor
		// th.printStackTrace(PrintStream) exists.
		// In this case, only switching to debug mode
		// and watching the Console output helps, maybe,
		// to find the stack trace in the millions of other
		// output that the Blackberry operating system creates.
	}
	
	/**
	 * Strip the buffer to its maximum length.
	 */
	private static void cutBuffer()
	{
		if (_logBuffer.length() > _maxLogBufferSize) {
			int n = _logBuffer.length() - _maxLogBufferSize;
			_logBuffer.delete(0, n);
		}
	}
	
	/**
	 * Returns the log messages.
	 */
	public static String getLog()
	{
		return _logBuffer.toString();
	}
	
	/**
	 * Return the prefix for debug messages print to System.err.
	 */
	public static String getDebugPrefix()
	{
		Date d = new Date(System.currentTimeMillis());
		SimpleDateFormat fmt = new SimpleDateFormat("yyyy-MM-dd'T'HH:mm:ss:SSS", Locale.get(Locale.LOCALE_en_US));
		String t = fmt.format(d);
		return "#### APPCNTR [" + t + "] ";
	}
	
	/**
	 * Hides the password in the log.
	 */
	public static String hidePassword(String s)
	{
		int n = s.length();
		StringBuffer buf = new StringBuffer();
		for (int i = 0; i < n; i++) {
			buf.append("*");
		}
		return buf.toString();
	}
}
