/*
 *  Licensed Materials - Property of IBM
 *  5725-G92 (C) Copyright IBM Corp. 2011, 2013. All Rights Reserved.
 *  US Government Users Restricted Rights - Use, duplication or
 *  disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 */
package com.ibm.appcenter.util;

import java.io.IOException;
import java.io.InputStream;
import java.util.Hashtable;
import com.ibm.appcenter.ui.widget.EnhancedGradientBackground;

import net.rim.device.api.io.LineReader;
import net.rim.device.api.ui.decor.Background;
import net.rim.device.api.ui.decor.BackgroundFactory;

/**
 * Properties mechanism very similar to the standard Java properties
 * mechanism.
 */
public class Properties 
{
	private Hashtable _properties;
	
	/**
	 * The constructor.
	 * @param propFileName The property file name.
	 */
	public Properties(Class clazz, String propFileName)
	{
		_properties = new Hashtable();
		
		try {
			InputStream stream = clazz.getResourceAsStream(propFileName);
			load(stream);
		} catch (Exception e) {
			Log.log("Properties Error when reading.", e);
		}
	}

	/**
	 * Parse input stream retrieving data.
	 * The sharp sign (#) at the beginning indicates comment lines.
	 * Otherwise, it is key=value pairs (with = as separator).
	 * @param stream input stream
	 * @throws IOException on stream read error or on stream end
	 */
	private void load(InputStream stream)
	throws IOException 
	{
		String line;
		String key = "";
		String value = "";
		LineReader reader = new LineReader(stream);
		byte[] data = null;
		boolean first = true;
		boolean continuation = false;
		int index;
		
		while (first || reader.lengthUnreadData() > 0) {
			first = false;
			data = reader.readLine();
			line = new String(data);
			if (continuation) {
				// continue a property from the previous line
				value = value.substring(0, value.length() - 1) + line;
				continuation = value.length() > 0 && value.charAt(value.length()-1) == '\\';
				_properties.put(key, value);
				continue;
			}
			// this is only used for debugging.
			// If we think the properties are not properly read, we prefix
			// lines with 5 sharp to see in the log file what was read.
			if (line.startsWith("#####")) {
				Log.log("Read: " + line);
			}
			if (line.startsWith("#")) {
				continue;
			}
			index = line.indexOf("=");
			if (index > 0) {
				key = line.substring(0, index).trim();
				index++;
				while (index < line.length() && line.charAt(index) == ' ')
					index++;
				if (index >= line.length())
					value = "";
				else
					value = line.substring(index).trim();
				continuation = value.length() > 0 && value.charAt(value.length()-1) == '\\';
				_properties.put(key, value);
			}
		}
	}

	/**
	 * Returns the properties.
	 */
	public Hashtable getProperties()
	{
		return _properties;
	}
	
	/**
	 * Returns a Color property.
	 */
	public int getColor(String key, int defaultColor)
	{
		String s = (String)_properties.get(key);
		if (s == null)
			return defaultColor;
		return parseColor(s);
	}
	
	/**
	 * Returns a Background property.
	 */
	public Background getBackground(String key, Background defaultBackground)
	{
		String s = (String)_properties.get(key);
		if (s == null)
			return defaultBackground;
		int[] colors = parseBackground(s);
		if (colors.length == 4) {
			return 
				BackgroundFactory.createLinearGradientBackground(
						colors[0], colors[1], colors[2], colors[3]);
		} else if (colors.length == 3) {
			return
				new EnhancedGradientBackground(colors[0], colors[1], colors[2]);
		} else if (colors.length == 2) {
			return
				BackgroundFactory.createLinearGradientBackground(
						colors[0], colors[0], colors[1], colors[1]);
		} else { // colors.length == 1
			return BackgroundFactory.createSolidBackground(colors[0]);
		}
	}
	
	/**
	 * Returns a boolean property.
	 */
	public boolean getBoolean(String key, boolean defaultValue)
	{
		String s = (String)_properties.get(key);
		if (s == null)
			return defaultValue;
		s = s.toLowerCase().trim();
		return "true".equals(s);
	}
	
	/**
	 * Returns a double property.
	 */
	public double getDouble(String key, double defaultValue)
	{
		String s = (String)_properties.get(key);
		if (s == null)
			return defaultValue;
		s = s.toLowerCase().trim();
		try {
			return Double.parseDouble(s);
		} catch (Exception ex) {
			return defaultValue;
		}
	}
	
	// -----------------------------------------------------------------
	
	/**
	 * Parses a color. Format 0xaarrggbb (alpha, red green blue)
	 * @param s The string to parse.
	 * @return The color.
	 */
	private int parseColor(String s)
	{
		s = s.toLowerCase().trim();
		int n = s.length();
		int i = 0;
		long result = 0;
		if (s.startsWith("#")) {
			i = 1;
		}
		if (s.startsWith("0x")) {
			i = 2;
		}
		while (i < n) {
			char c = s.charAt(i);
			if (c >= 'a' && c <= 'f') {
				result = result * 16 + (c - 'a' + 10);
			}
			else if (c >= '0' && c <= '9') {
				result = result * 16 + (c - '0');
			}
			i++;
		}
		return (int)result;
	}
	
	/**
	 * Parses a background.
	 * @param s The string to parse.
	 * @return An array of the colors of the background.
	 *    If 1 element, it is a solid background.
	 *    If 2 elements it is a vertical linear gradient.
	 *    If 3 elements it is a vertical double gradient.
	 *    If 4 elements it is a 4 corner gradient.
	 */
	private int[] parseBackground(String s)
	{
		s = s.toLowerCase().trim();
		int[] colors = new int[4];
		int n = 0;
		int a = 0;
		int e = s.length();
		if (s.startsWith("solid(")) {
			a = 6;
		}
		if (s.startsWith("linear(")) {
			a = 7;
		}
		if (s.startsWith("corner(")) {
			a = 7;
		}
		while (a < e && s.charAt(a) == ' ')
			a++;
		while (e > a && (s.charAt(e-1) == ')' || s.charAt(e-1) == ' '))
			e--;
		s = s.substring(a,e);
		while (n < 4) {
			int idx = s.indexOf(',');
			if (idx < 0) {
				// last color
				colors[n++] = parseColor(s);
				break;
			}
			String p = s.substring(0, idx);
			colors[n++] = parseColor(p);
			s = s.substring(idx+1);
		}
		if (n == 0)
			return new int[] { 0 };
		int[] result = new int[n];
		System.arraycopy(colors, 0, result, 0, n);
		return result;
	}
}
